package ru.crp.lvms;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.handler.annotation.Payload;
import ru.crp.cmls.api.NodeGreeting;
import ru.crp.cmls.calculations.solvers.dto.SolverPresenceRequest;
import ru.crp.lvms.cluster.jobs.CheckSpaceClustersJob;
import ru.crp.lvms.managers.NodesManager;
import ru.crp.lvms.managers.solvers.SolverConfigManager;
import ru.crp.lvms.model.entities.nodes.NodeEntity;
import ru.crp.lvms.utils.Mapper;

import java.io.IOException;
import java.util.List;

import static ru.crp.lvms.cluster.jobs.BaseCheckClusterJob.SOLVERS_LIST_TYPE_TOKEN;

public class SolverStatusReceiver {
    private static final Logger LOGGER = LoggerFactory.getLogger(SolverStatusReceiver.class);

    @Autowired
    private ObjectMapper jacksonMapper;

    @Autowired
    private NodesManager nodesManager;

    @Autowired
    private CheckSpaceClustersJob checkSpaceClustersJob;

    @Autowired
    private SolverConfigManager solverConfigManager;

    @Value("${solverNodeOwnerId}")
    private String ownerId;

    @RabbitListener(queues = "solverStatusQueue")
    public void process(@Payload String entry) throws IOException {
        NodeGreeting nodeGreeting;
        try {
            nodeGreeting = jacksonMapper.readValue(entry, NodeGreeting.class);
        } catch (Exception e) {
            LOGGER.error("Unable to map message from rmq to object", e);
            return;
        }
        try {
            if (nodeGreeting.getStartup()) {
                startupNode(nodeGreeting.getUrl());
            } else {
                removeNode(nodeGreeting.getUrl());
            }
        } catch (Exception e) {
            LOGGER.error("Unable to update solver status", e);
        }
    }

    private void removeNode(String url) {
        String nodeUrl = url;
        if (!nodeUrl.endsWith("/")) {
            nodeUrl = nodeUrl + "/";
        }
        List<NodeEntity> entitiesWithThatUrl = nodesManager.findByUrl(nodeUrl);
        if (!entitiesWithThatUrl.isEmpty()) {
            nodesManager.delete(entitiesWithThatUrl.get(0), null);
            LOGGER.info("Node with url " + nodeUrl + " disconnected");
        }
    }

    private void startupNode(String url) {
        String nodeUrl = url;
        if (!nodeUrl.endsWith("/")) {
            nodeUrl = nodeUrl + "/";
        }
        List<NodeEntity> entitiesWithThatUrl = nodesManager.findByUrl(nodeUrl);
        NodeEntity nodeToWorkWith;
        if (entitiesWithThatUrl.isEmpty()) {
            nodeToWorkWith = new NodeEntity();
            nodeToWorkWith.setUrl(nodeUrl);
            nodeToWorkWith.setName(nodeUrl);
            nodeToWorkWith.setOwnerId(Integer.parseInt(ownerId));
            nodeToWorkWith = nodesManager.createAndRead(nodeToWorkWith, null);
        } else {
            nodeToWorkWith = entitiesWithThatUrl.get(0);
        }
        List<SolverPresenceRequest> solversToCheck = Mapper.map(solverConfigManager.getAllConfigs(), SOLVERS_LIST_TYPE_TOKEN);
        checkSpaceClustersJob.updateNodeStatus(nodeToWorkWith, solversToCheck);
        LOGGER.info("Node with url " + nodeToWorkWith.getUrl() + " connected");
    }
}
