package ru.crp.lvms.cluster;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.stereotype.Repository;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.lvms.cluster.model.NodeForClusterInfoEntity;
import ru.crp.lvms.cluster.model.NodeStatistics;

import javax.annotation.Nullable;
import javax.sql.DataSource;
import java.util.Date;
import java.util.List;

@Repository
public class NodesStatisticsDao extends JdbcDaoSupport {
    private static final String NODE_TASKS_COUNT_SQL = "SELECT COUNT(*) FROM tasks WHERE node_id = ? AND status_id <> ?";
    private static final String AVERAGE_TASK_SOLVING_TIME_SQL = "SELECT AVG(EXTRACT ( EPOCH FROM(date_modified - start_exec_date))::INT/60)::INT FROM tasks WHERE node_id = ? AND status_id = ?";
    private static final String MIN_TASK_SOLVING_TIME_SQL = "SELECT MIN(EXTRACT ( EPOCH FROM(date_modified - start_exec_date))::INT/60)::INT FROM tasks WHERE node_id = ? AND status_id = ?";
    private static final String MAX_TASK_SOLVING_TIME_SQL = "SELECT MAX(EXTRACT ( EPOCH FROM(date_modified - start_exec_date))::INT/60)::INT FROM tasks WHERE node_id = ? AND status_id = ?";
    private static final String MEMORY_USAGE_SQL = "SELECT AVG(memory) FROM tasks WHERE node_id = ? AND (status_id = ? OR status_id = ?) AND memory > 0";
    private static final String LAST_USAGE_SQL = "SELECT MAX(date_modified) FROM tasks WHERE node_id = ? AND (status_id = ? OR status_id = ?)";

    public NodeStatistics getNodesStatistics(int nodeId) {
        NodeStatistics nodeStatistics = new NodeStatistics(nodeId);

        nodeStatistics.setTasksCount(queryForTasksCount(nodeId));
        nodeStatistics.setAverageTime(queryForTime(nodeId, AVERAGE_TASK_SOLVING_TIME_SQL));
        nodeStatistics.setMinimumTime(queryForTime(nodeId, MIN_TASK_SOLVING_TIME_SQL));
        nodeStatistics.setMaximumTime(queryForTime(nodeId, MAX_TASK_SOLVING_TIME_SQL));
        nodeStatistics.setMemoryUsage(queryForMemoryUsage(nodeId));
        nodeStatistics.setLastUsage(queryForLastUsage(nodeId));

        return nodeStatistics;
    }

    private Long queryForMemoryUsage(int nodeId) {
        return getJdbcTemplate().queryForObject(MEMORY_USAGE_SQL, Long.class,
                nodeId, CalculationStatus.Finished.getId(), CalculationStatus.Failed.getId());
    }

    @Nullable
    private Date queryForLastUsage(int nodeId) {
        return getJdbcTemplate().queryForObject(LAST_USAGE_SQL, Date.class,
                nodeId, CalculationStatus.Finished.getId(), CalculationStatus.Failed.getId());
    }

    @Nullable
    private Integer queryForTime(int nodeId, String sql) {
        return getJdbcTemplate().queryForObject(sql, Integer.class, nodeId, CalculationStatus.Finished.getId());
    }


    private Integer queryForTasksCount(int nodeId) {
        return getJdbcTemplate().queryForObject(NODE_TASKS_COUNT_SQL, Integer.class, nodeId, CalculationStatus.Canceled.getId());
    }


    public List<NodeForClusterInfoEntity> getAllNodes() {
        return getJdbcTemplate().query("SELECT * FROM nodes_for_cluster_info_view", new NodeForClusterInfoRowMapper());
    }


    @Autowired
    public void setDataSourceAutowired(DataSource dataSource) {
        super.setDataSource(dataSource);
    }
}
