package ru.crp.lvms.cluster.jobs;

import org.modelmapper.TypeToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.util.Assert;
import ru.crp.cmls.api.NodeStatus;
import ru.crp.cmls.calculations.solvers.dto.SolverPresenceRequest;
import ru.crp.lvms.cluster.ClustersManager;
import ru.crp.lvms.cluster.model.ClusterEntity;
import ru.crp.lvms.cluster.model.ClusterType;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.managers.NodesManager;
import ru.crp.lvms.managers.solvers.SolverConfigManager;
import ru.crp.lvms.model.entities.nodes.NodeEntity;
import ru.crp.lvms.utils.Mapper;

import javax.annotation.PostConstruct;
import java.util.List;

abstract public class BaseCheckClusterJob {
    private static final Logger LOGGER = LoggerFactory.getLogger(BaseCheckClusterJob.class);

    public static final TypeToken<List<SolverPresenceRequest>> SOLVERS_LIST_TYPE_TOKEN = new TypeToken<List<SolverPresenceRequest>>() {
    };

    @Autowired
    protected ApplicationEventPublisher applicationEventPublisher;

    @Autowired
    protected NodesManager nodeManager;

    @Autowired
    protected SolverConfigManager solverConfigManager;

    @Autowired
    protected ClustersManager clustersManager;

    protected ClusterType clusterType;

    @PostConstruct
    public void init() {
        clusterType = getClusterType();
        Assert.notNull(clusterType);
    }


    public void checkClusters() {
        List<SolverPresenceRequest> solversToCheck = Mapper.map(solverConfigManager.getAllConfigs(), SOLVERS_LIST_TYPE_TOKEN);
        clustersManager.getClustersByType(clusterType).stream()
                .filter(ClusterEntity::getIsEnabled)
                .forEach(cluster -> {
                    try {
                        checkCluster(cluster, solversToCheck);
                    } catch (Exception ex) {
                        LOGGER.error("Exception while checking cluster {} status", cluster.getName(), ex);
                    }
                });
    }


    // expects that <nodeToCheck> belongs to cluster with type <clusterType>
    public NodeStatus getMostRecentNodeStatus(NodeEntity nodeToCheck) {
        List<SolverPresenceRequest> solversToCheck = Mapper.map(solverConfigManager.getAllConfigs(), SOLVERS_LIST_TYPE_TOKEN);
        return doGetMostRecentNodeStatus(nodeToCheck, solversToCheck);
    }


    abstract public void checkCluster(ClusterEntity clusterToCheck, List<SolverPresenceRequest> solversToCheck) throws BadRequestException;

    abstract public ClusterType getClusterType();

    abstract protected NodeStatus doGetMostRecentNodeStatus(NodeEntity nodeToCheck, List<SolverPresenceRequest> solversToCheck);
}
