package ru.crp.lvms.cluster.model;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ClusterInfo {
    private int id;
    private String displayName;

    private int clusterTypeId;
    private String clusterTypeName;

    private ClusterProperties total = ClusterProperties.createEmptyProperties();
    private Map<String, ClusterProperties> clusterInfoBySolver = new HashMap<>();

    public ClusterInfo(int clusterId, String clusterName, ClusterType clusterType, List<String> allSolverNames) {
        this.id = clusterId;
        this.displayName = clusterName;

        this.clusterTypeId = clusterType.getId();
        this.clusterTypeName = clusterType.toString();

        allSolverNames.forEach(solverName -> clusterInfoBySolver.put(solverName, ClusterProperties.createEmptyProperties()));
    }

    public String getDisplayName() {
        return displayName;
    }

    public Map<String, ClusterProperties> getClusterInfoBySolver() {
        return clusterInfoBySolver;
    }

    public ClusterProperties getTotal() {
        return total;
    }

    public void addInfoForSolver(String solverName, NodePropertyInfo nodes, NodePropertyInfo cores, NodePropertyInfo memory) {
        clusterInfoBySolver.put(solverName, new ClusterProperties(nodes, cores, memory));
    }

    public void setTotal(ClusterProperties total) {
        this.total = total;
    }

    public void addNode(NodeForClusterInfoEntity node) {
        total.addNode(node);
        clusterInfoBySolver.forEach((solverName, props) -> {
            props.addNode(node, solverName);
        });
    }

    public int getId() {
        return id;
    }

    public int getClusterTypeId() {
        return clusterTypeId;
    }

    public String getClusterTypeName() {
        return clusterTypeName;
    }
}
