package ru.crp.lvms.events.listeners;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.messaging.MessagingException;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.stereotype.Component;
import ru.crp.lvms.cluster.ClustersManager;
import ru.crp.lvms.cluster.model.ClusterInfo;
import ru.crp.lvms.events.NodeStatusUpdatedEvent;
import ru.crp.lvms.events.TaskStateChangedEvent;

import java.util.List;

import static ru.crp.lvms.cluster.ClustersManager.CLUSTER_INFO_QUEUE;

// listen all events which updates cluster info
@Component
public class ClusterInfoEventListener {
    private static final Logger LOGGER = LoggerFactory.getLogger(ClusterInfoEventListener.class);

    @Autowired
    private SimpMessagingTemplate messagingTemplate;

    @Autowired
    private ClustersManager clustersManager;

    @EventListener
    public void onNodeUpdated(NodeStatusUpdatedEvent nodeStatusUpdatedEvent) {
        publishToWebsocket(clustersManager.getFreshClustersInfo());
    }


    @EventListener
    public void onTaskUpdated(TaskStateChangedEvent taskStateChangedEvent) {
        publishToWebsocket(clustersManager.getFreshClustersInfo());
    }


    private void publishToWebsocket(List<ClusterInfo> clusterInfos) {
        try {
            messagingTemplate.convertAndSend(CLUSTER_INFO_QUEUE, clusterInfos);
        } catch (MessagingException ex) {
            LOGGER.error("Can't send notification via web-socket", ex);
        }
    }
}
