package ru.crp.lvms.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.model.AccessRights;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.dao.EquipmentDao;
import ru.crp.lvms.model.entities.EquipmentEntity;
import ru.crp.lvms.model.entities.UserEntity;

import javax.annotation.Nullable;
import java.util.List;
import java.util.Map;

import static java.util.stream.Collectors.toList;

@Service
public class EquipmentManager extends BaseManagerSuggest<EquipmentEntity> {

    @Autowired
    private EquipmentDao equipmentDao;

    @Autowired
    private TechnologyManager technologyManager;

    @Override
    protected BaseSpaceDao<EquipmentEntity> aliveDao() {
        return equipmentDao;
    }

    public List<EquipmentEntity> getTechnologyEquipment(int technologyId, @Nullable UserEntity currentUser) {
        technologyManager.checkEntityExistsAndAccesible(technologyId, AccessRights.view, currentUser);
        return equipmentDao.getTechnologyEquipment(technologyId);
    }

    public boolean isEquipmentBelongsToTechnology(Integer equipmentId, Integer technologyId) {
        List<Integer> technologyEquipments = equipmentDao.getTechnologyEquipment(technologyId).stream()
                .map(EquipmentEntity::getId)
                .collect(toList());
        return technologyEquipments.contains(equipmentId);
    }

    public PageImpl<EquipmentEntity> getTechnologyEquipmentPage(Integer technologyId, Map<String, Object> filters,
                                                                Pageable pageable, UserEntity currentUser) {
        technologyManager.checkEntityExistsAndAccesible(technologyId, AccessRights.view, currentUser);
        return equipmentDao.find(technologyId, filters, pageable);
    }

    public void deleteFromTechnology(int equipmentId, int technologyId, UserEntity user) {
        technologyManager.removeLinkToEquipment(technologyId, equipmentId);
    }
}
