package ru.crp.lvms.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.model.AccessRights;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.dao.TechnologyParametersDao;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.technology.TechnologyParametersEntity;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;
import java.util.Map;

@Service
public class TechnologyParametersManager extends BaseManagerSuggest<TechnologyParametersEntity> {

    @Autowired
    private TechnologyParametersDao technologyParametersDao;

    @Autowired
    private TechnologyManager technologyManager;

    @Override
    protected BaseSpaceDao<TechnologyParametersEntity> aliveDao() {
        return technologyParametersDao;
    }

    public List<TechnologyParametersEntity> getTechnologyParameters(int technologyId, UserEntity currentUser) {
        technologyManager.checkEntityExistsAndAccesible(technologyId, AccessRights.view, currentUser);
        return technologyParametersDao.getTechnologyEquipment(technologyId);
    }

    public PageImpl<TechnologyParametersEntity> getTechnologyParametersPage(int technologyId, Map<String, Object> filters,
                                                                            Pageable pageable, UserEntity currentUser) {
        technologyManager.checkEntityExistsAndAccesible(technologyId, AccessRights.view, currentUser);
        filters.put("technologyId", technologyId);
        return technologyParametersDao.find(filters, pageable);
    }

    @Nonnull
    @Override
    public TechnologyParametersEntity createAndRead(TechnologyParametersEntity entity, @Nullable UserEntity currentUser) {
        technologyManager.checkEntityExistsAndAccesible(entity.getTechnologyId(), AccessRights.update, currentUser);
        TechnologyParametersEntity existed = technologyParametersDao
                .findByTechnologyIdAndName(entity.getTechnologyId(), entity.getName());
        if (existed != null) {
            throw new BadRequestException("Параметр с таким именем уже добавлен к технологии");
        }
        return super.createAndRead(entity, currentUser);
    }


    @Nonnull
    @Override
    public TechnologyParametersEntity updateAndRead(TechnologyParametersEntity updated, @Nullable UserEntity currentUser) {
        technologyManager.checkEntityExistsAndAccesible(updated.getTechnologyId(), AccessRights.update, currentUser);
        TechnologyParametersEntity existed = technologyParametersDao
                .findByTechnologyIdAndName(updated.getTechnologyId(), updated.getName());
        if (existed != null && existed.getId() != updated.getId()) {
            throw new BadRequestException("Параметр с таким именем уже добавлен к технологии");
        }
        return super.updateAndRead(updated, currentUser);
    }



}
