package ru.crp.lvms.managers.construction.elements;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.model.AccessRights;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.dao.construction.elements.ConstructionElementDocsDao;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementDocsEntity;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementDocsEntityTable;
import ru.crp.lvms.model.entities.misc.FileType;

import java.util.List;

@Service
public class ConstructionElementDocsManager extends BaseManagerSuggest<ConstructionElementDocsEntity> {

    @Autowired
    private FileManager fileManager;

    @Autowired
    private ConstructionElementDocsDao constructionElementDocsDao;

    @Autowired
    private ConstructionElementManager constructionElementManager;

    @Override
    protected BaseSpaceDao<ConstructionElementDocsEntity> aliveDao() {
        return constructionElementDocsDao;
    }


    public ConstructionElementDocsEntity createFile(int constructionElementId, MultipartFile fileContent, UserEntity currentUser) {
        constructionElementManager.checkEntityExistsAndAccesible(constructionElementId, AccessRights.update, currentUser);
        FileEntity newFile = fileManager.createFile(fileContent, constructionElementId, FileType.DescriptionFile, currentUser);
        ConstructionElementDocsEntityTable newDoc = new ConstructionElementDocsEntityTable();
        newDoc.setConstructionElementId(constructionElementId);
        newDoc.setFileId(newFile.getId());
        return constructionElementDocsDao.insertNewDoc(newDoc);
    }


    public ConstructionElementDocsEntity updateFile(int constructionElementId, int docId, MultipartFile newFileContent, UserEntity currentUser) {
        constructionElementManager.checkEntityExistsAndAccesible(constructionElementId, AccessRights.update, currentUser);
        ConstructionElementDocsEntity toUpdate = checkEntityExistsAndAccesible(docId, AccessRights.update, currentUser);
        fileManager.updateFileContent(toUpdate.getFileId(), newFileContent, currentUser);
        return findOne(docId, currentUser);
    }


    public void deleteFile(Integer constructionElementId, int docId, UserEntity currentUser) {
        constructionElementManager.checkEntityExistsAndAccesible(constructionElementId, AccessRights.update, currentUser);
        ConstructionElementDocsEntity toDelete = checkEntityExistsAndAccesible(docId, AccessRights.delete, currentUser);
        delete(docId, currentUser);
        fileManager.deleteFile(toDelete.getFileId(), currentUser);
    }

    public List<ConstructionElementDocsEntity> getFiles(Integer constructionElementId, UserEntity currentUser) {
        constructionElementManager.checkEntityExistsAndAccesible(constructionElementId, AccessRights.view, currentUser);
        return constructionElementDocsDao.findByConstructionElementId(constructionElementId);
    }
}
