package ru.crp.lvms.managers.misc;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.mail.*;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@Component
public class MailSender {
    private static final Logger LOGGER = LoggerFactory.getLogger(MailSender.class);

    @Value("${mail.server.url}")
    private String server;

    @Value("${mail.server.port}")
    private String port;

    @Value("${mail.username}")
    private String mailUsername;

    @Value("${mail.password}")
    private String mailPassword;

    @Value("${testEnviroment}")
    private boolean testEnviroment;

    private ExecutorService executor = new ThreadPoolExecutor(0, 5, 60L, TimeUnit.SECONDS, new LinkedBlockingQueue<>());

    public void sendNotificationMailAsync(final String messageText, final String recipientAddress, final String userName) {
        executor.submit((Runnable) () -> {
            try {
                sendNotificationMail(messageText, recipientAddress, userName);
            } catch (MessagingException e) {
                LOGGER.error("Cannot send notification email to user " + userName + " at " + recipientAddress, e);
            }
        });
    }

    public void sendNotificationMail(final String messageText, final String recipientAddress, final String userName)
            throws MessagingException {
        Properties props = new Properties();
        props.put("mail.smtp.auth", "true");
        props.put("mail.smtp.starttls.enable", "true");
        props.put("mail.smtp.host", server);
        props.put("mail.smtp.port", port);

        Session session = Session.getInstance(props, new javax.mail.Authenticator() {
            @Override
            protected PasswordAuthentication getPasswordAuthentication() {
                return new PasswordAuthentication(mailUsername, mailPassword);
            }
        });

        String address = testEnviroment ? mailUsername : recipientAddress;

        Message message = new MimeMessage(session);
        message.setFrom(new InternetAddress(mailUsername));
        message.setRecipients(Message.RecipientType.TO, InternetAddress.parse(address));
        message.setSubject("Уведомление от ИСКПИ");
        message.setText("Уважаемый " + userName + ", \n\n" + messageText + "\n\n" +
                new SimpleDateFormat("HH:mm:ss dd/MM/yyyy").format(new Date()));

        Transport.send(message);
    }
}
