package ru.crp.lvms.managers.tasks.resultsHandlers;

import org.apache.commons.io.FilenameUtils;
import ru.crp.cmls.calculations.solvers.xml.config.ResultType;
import ru.crp.lvms.managers.tasks.ResultsParent;
import ru.crp.lvms.model.entities.ResultEntity;
import ru.crp.lvms.model.entities.misc.CreatedBy;

import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

import static java.util.Collections.singletonList;

public class PicturesResultHandler extends KeyResultHandler {
    public PicturesResultHandler(Path pathToStorage) {
        super(pathToStorage);
    }

    @Override
    public boolean canHandle(ResultType resultType, Path pathToFile) {
        return resultType == ResultType.pictures && Files.isRegularFile(pathToFile) && isPicture(pathToFile);
    }

    @Override
    protected List<ResultEntity> doHandleResultsFile(Path pathToFile, ResultsParent resultsParent, CreatedBy createdBy) {
        ResultEntity newPicture = initCommonResultProperties(pathToFile, resultsParent, createdBy);
        if (createdBy == CreatedBy.Auto) {
            extractDescription(pathToFile, newPicture);
        }

        newPicture.setName(FilenameUtils.removeExtension(pathToFile.getFileName().toString()));
        newPicture.setType(ResultType.pictures);

        return singletonList(newPicture);
    }

    private boolean isPicture(Path pictureFile) {
        String extension = FilenameUtils.getExtension(pictureFile.toString());
        return "png".equalsIgnoreCase(extension) || "jpg".equalsIgnoreCase(extension) || "gif".equalsIgnoreCase(extension);
    }
}
