package ru.crp.lvms.model.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.cmlb.model.SuggestRequest;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.FileEntityTable;
import ru.crp.lvms.model.entities.misc.FileType;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Transactional(propagation = Propagation.REQUIRED)
@Repository
@Primary
public class FileDao extends BaseCompositeDao<FileEntityTable, FileEntity> {
    @Repository
    public static class TableDao extends BaseSpaceDao<FileEntityTable> {
        public void deleteAllForMaterial(int materialId, FileType fileType) {
            getJdbcTemplate().update("DELETE FROM files WHERE pid = ? AND file_type_id = ?",
                    materialId, fileType.getId());
        }
    }

    @Autowired
    private TableDao tableDao;

    @Override
    protected TableDao tableDao() {
        return tableDao;
    }


    public List<FileEntity> findByParentIdAndType(int parentId, FileType fileType) {
        return find(" WHERE pid = ? AND file_type_id = ?", parentId, fileType.getId());
    }


    public List<FileEntity> findByRelativePath(String relativePath) {
        return find(" WHERE path_to_file = ?", relativePath);
    }


    public void deleteAllForMaterial(int materialId, FileType fileType) {
        tableDao().deleteAllForMaterial(materialId, fileType);
    }


    public List<FileEntity> getConstructionElementFiles(int constructionElementId, FileType fileType) {
        return find("INNER JOIN construction_elements_files cef ON cef.file_id = " + getAlias() +
                        ".id WHERE cef.construction_element_id = ? AND " + getAlias() + ".file_type_id = ?",
                constructionElementId, fileType.getId());
    }

    public List<FileEntity> suggestConstructionElementFiles(int constructionElementId, FileType fileType,
                                                            SuggestRequest request, int currentUserId) {
        String additionalWhere = String.format("INNER JOIN construction_elements_files cef ON cef.file_id = %s.id " +
                        "WHERE cef.construction_element_id = %d AND %s.file_type_id = %d",
                getAlias(), constructionElementId, getAlias(), fileType.getId());
        return suggest(additionalWhere, request, currentUserId);
    }

    public List<FileEntity> getConstructionElementFiles(int constructionElementId) {
        return find("INNER JOIN construction_elements_files cef ON cef.file_id = " + getAlias() +
                ".id WHERE cef.construction_element_id = ?", constructionElementId);
    }

    public PageImpl<FileEntity> getConstructionElementFiles(Integer constructionElementId, @Nullable FileType fileType,
                                                            Map<String, Object> filters, Pageable pageable) {
        String sql = "INNER JOIN construction_elements_files cef ON cef.file_id = " + getAlias() +
                ".id WHERE cef.construction_element_id = ?";
        List<Object> params = new ArrayList<>();
        params.add(constructionElementId);
        if (fileType != null) {
            sql += "AND " + getAlias() + ".file_type_id = ?";
            params.add(fileType.getId());
        }
        return find(sql, filters, pageable, params.toArray());
    }


    public List<FileEntity> findOptimizationFiles(Integer optimizationId, @Nullable Integer fileTypeId) {
        String fileTypeFilter = "";
        List<Object> params = new ArrayList<>();
        params.add(optimizationId);
        if (fileTypeId != null) {
            fileTypeFilter = " AND " + getAlias() + ".file_type_id = ?";
            params.add(fileTypeId);
        }
        return find("INNER JOIN optimization_files of ON of.file_id = " + getAlias() + ".id" +
                " WHERE of.optimization_id= ?" + fileTypeFilter + " ORDER BY of.ordering", params.toArray());
    }

    public PageImpl<FileEntity> findOptimizationFilesPage(int optimizationId, Map<String, Object> filters, Pageable page) {
        return find("INNER JOIN optimization_files of ON of.file_id = " + getAlias() + ".id" +
                " WHERE of.optimization_id= ?", filters, page, optimizationId);
    }
}
