package ru.crp.lvms.model.dao;

import org.springframework.stereotype.Repository;
import ru.crp.lvms.model.entities.misc.ParametersDictionaryType;
import ru.crp.lvms.model.entities.modfrontier.ParametersDictionary;
import ru.crp.lvms.optimizations.tosca.enums.ToscaJsonParametersParents;

import javax.annotation.Nullable;
import javax.annotation.PostConstruct;
import java.util.List;

@Repository
public class ParametersDictionaryDao extends BaseSpaceDao<ParametersDictionary> {

    @PostConstruct
    public void init() {
        getJdbcTemplate().query("SELECT id, name FROM parameters_dictionary_type", rs -> {
            int id = rs.getInt("id");
            String uniqueName = rs.getString("name");
            ParametersDictionaryType parametersDictionaryType = ParametersDictionaryType.valueOf(uniqueName);
            parametersDictionaryType.setId(id);
        });

        getJdbcTemplate().query("SELECT id, name FROM parameters_dictionary WHERE type_id IN (?, ?)", rs -> {
            int id = rs.getInt("id");
            String uniqueName = rs.getString("name");
            ToscaJsonParametersParents optimizationMethod = ToscaJsonParametersParents.fromUniqueName(uniqueName);
            optimizationMethod.setId(id);
        }, ParametersDictionaryType.ToskaOptimizationMethod.getId(), ParametersDictionaryType.ToskaPostproc.getId());
    }

    public List<ParametersDictionary> findByTypeName(ParametersDictionaryType typeName) {
        return find("WHERE type_id = ?", typeName.getId());
    }

    @Nullable
    public ParametersDictionary findByTypeNameAndId(ParametersDictionaryType typeName, int id) {
        return findOne("WHERE id = ? AND type_id = ?", id, typeName.getId());
    }
}
