package ru.crp.lvms.model.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Repository;
import ru.crp.lvms.model.entities.technology.TechnologyEntity;
import ru.crp.lvms.model.entities.technology.TechnologyEntityTable;
import ru.crp.lvms.utils.MappingHelpers;

import javax.annotation.Nullable;
import java.util.List;
import java.util.Map;

@Repository
@Primary
public class TechnologyDao extends BaseCompositeDao<TechnologyEntityTable, TechnologyEntity> {
    @Repository
    public static class TableDao extends BaseSpaceDao<TechnologyEntityTable> {

    }

    @Autowired
    private TableDao tableDao;

    @Autowired
    private MappingHelpers mappingHelpers;

    @Override
    protected BaseSpaceDao<TechnologyEntityTable> tableDao() {
        return tableDao;
    }

    public void addLinkToEquipment(int technologyId, int equipmentId) {
        jdbcTemplate().update("INSERT INTO technologies_equipments(technology_id, equipment_id) VALUES (?,?)",
                technologyId, equipmentId);
    }

    public void removeLinkToEquipment(int technologyId, int equipmentId) {
        jdbcTemplate().update("DELETE FROM technologies_equipments WHERE equipment_id = ?", equipmentId);
    }

    @Override
    public TechnologyEntity insertAndRead(TechnologyEntity t) {
        return setPathToImage(super.insertAndRead(t));
    }


    @Override
    public TechnologyEntity updateAndRead(TechnologyEntity t) {
        return setPathToImage(super.updateAndRead(t));
    }


    @Override
    public List<TechnologyEntity> find(String whereSql, Object... args) {
        return setPathToImage(super.find(whereSql, args));
    }

    @Override
    public List<TechnologyEntity> findDistinct(String whereSql, Object... args) {
        return setPathToImage(super.findDistinct(whereSql, args));
    }


    @Override
    public PageImpl<TechnologyEntity> find(String sql, Map<String, Object> filters, Pageable pageable, Object... params) {
        PageImpl<TechnologyEntity> technologyEntities = super.find(sql, filters, pageable, params);
        setPathToImage(technologyEntities.getContent());
        return technologyEntities;
    }

    @Override
    protected TechnologyEntity findOne(String whereSql, Object... args) {
        return setPathToImage(super.findOne(whereSql, args));
    }

    @Override
    public TechnologyEntity findOne(int id) {
        return setPathToImage(super.findOne(id));
    }

    @Override
    public List<TechnologyEntity> findAll() {
        return setPathToImage(super.findAll());
    }

    private TechnologyEntity setPathToImage(@Nullable TechnologyEntity technology) {
        if (technology == null) {
            return null;
        }
        technology.setPathToImage(mappingHelpers.getPathToTechnologyImage(technology));
        return technology;
    }

    private List<TechnologyEntity> setPathToImage(List<TechnologyEntity> technologies) {
        technologies.forEach(this::setPathToImage);
        return technologies;
    }
}
