package ru.crp.lvms.model.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Repository;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.UserEntityTable;
import ru.crp.lvms.web.dto.user.UserRole;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;

@Repository
@Primary
public class UserDao extends BaseCompositeDao<UserEntityTable, UserEntity> {
    @Repository
    public static class TableDao extends BaseSpaceDao<UserEntityTable> {
    }

    @Autowired
    private TableDao tableDao;

    @Override
    protected TableDao tableDao() {
        return tableDao;
    }

    public UserEntity findByName(String name) {
        return findOne("where login=?", name);
    }

    public List<UserEntity> findByRole(int roleId) {
        return find("where role_id=?", roleId);
    }

    @PostConstruct
    void init() {
        initUserRoleEnum();
    }

    private void initUserRoleEnum() {
        getJdbcTemplate().query("select id, name, uname, priority from roles", rs -> {
            String uname = rs.getString("uname");
            for (UserRole role : UserRole.values()) {
                if (role.getUname().equals(uname)) {
                    role.setId(rs.getInt("id"));
                    role.setName(rs.getString("name"));
                    role.setPriority(rs.getInt("priority"));
                }
            }
        });
    }

    public List<UserEntity> findAllActiveUsers() {
        return find(" where active=true");
    }

    public PageImpl<UserEntity> findAllActiveUsers(Map<String, Object> filters, Pageable pageable) {
        return find(" where active=true", filters, pageable);
    }

    public List<UserEntity> findMeOnly(UserEntity user) {
        return find(" where id=?", user.getId());
    }

    public PageImpl<UserEntity> findMeOnly(UserEntity user, Map<String, Object> filters, Pageable pageable) {
        return find(" where id=?", filters, pageable, user.getId());
    }
}
