package ru.crp.lvms.model.dao.simulations;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Repository;
import ru.crp.lvms.model.dao.BaseCompositeDao;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.entities.simulations.SimulationEntity;
import ru.crp.lvms.model.entities.simulations.SimulationEntityTable;
import ru.crp.lvms.model.entities.simulations.SimulationType;
import ru.crp.lvms.model.entities.simulations.SolvingStatus;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;

@Repository
public class SimulationDao extends BaseCompositeDao<SimulationEntityTable, SimulationEntity> {
    @Repository
    public static class TableDao extends BaseSpaceDao<SimulationEntityTable> {

    }

    @Autowired
    private TableDao tableDao;

    @PostConstruct
    public void init() {
        getJdbcTemplate().query("SELECT id, display_name FROM virtual_simulation_types", rs -> {
            String displayName = rs.getString("display_name");
            int id = rs.getInt("id");
            SimulationType.valueOf(id).setDisplayName(displayName);
        });
        getJdbcTemplate().query("SELECT id, display_name FROM virtual_simulation_statuses", rs -> {
            String displayName = rs.getString("display_name");
            int id = rs.getInt("id");
            SolvingStatus.valueOf(id).setDisplayName(displayName);
        });
    }

    @Override
    protected BaseSpaceDao<SimulationEntityTable> tableDao() {
        return tableDao;
    }

    public List<SimulationEntity> findByStandardId(int standardId) {
        return find("WHERE standard_id = ?", standardId);
    }

    public List<SimulationEntity> findByMaterialId(int materialId) {
        return find("WHERE composite_material_id = ?", materialId);
    }

    public List<SimulationEntity> findByECId(Integer ecId) {
        return find("WHERE ec_id = ?", ecId);
    }

    public List<SimulationEntity> findSimulationsForTechnology(Integer technologyId) {
        return find("WHERE composite_material_id IN " +
                "(SELECT id FROM composite_materials WHERE technology_id = ?)", technologyId);
    }

    public PageImpl<SimulationEntity> findSimulationsPageForTechnology(Integer technologyId, Pageable page,
                                                                       Map<String, Object> filters) {
        return find("WHERE composite_material_id IN " +
                "(SELECT id FROM composite_materials WHERE technology_id = ?)", filters, page, technologyId);
    }
}
