package ru.crp.lvms.model.entities.compositeMaterials;

import com.fasterxml.jackson.annotation.JsonIgnore;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.Layer;
import ru.crp.lvms.model.entities.ParameterValues;
import ru.crp.lvms.model.entities.technology.TechnologyType;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;

import java.util.List;


@Table("composite_materials_join")
public class CompositeMaterialEntity extends CompositeMaterialEntityTable {
    @GridColumn(name = "owner", label = "Автор", cell = GridColumnType.uri, order = 6)
    @Column("owner")
    private String owner;

    @GridColumn(name = "technology", label = "Технология", order = 7)
    @Column("technology_name")
    private String technology;

    @GridColumn(name = "equipment", label = "Оборудование", order = 8)
    @Column("equipment_name")
    private String equipment;

    @GridColumn(name = "matrix", label = "Матрица", cell = GridColumnType.uri, order = 9)
    @Column("matrix_name")
    private String matrix;

    @GridColumn(name = "fiber", label = "Волокно", cell = GridColumnType.uri, order = 11)
    @Column("fiber_name")
    private String fiber;

    @GridColumn(name = "epoxide", label = "Эпоксид", cell = GridColumnType.uri, order = 10)
    @Column("epoxide_name")
    private String epoxide;

    @Column("technology_type")
    private TechnologyType technologyType;

    private String technologyTypeName;

    private List<Layer> layers;

    private List<FileEntity> materialFiles;

    private List<FileEntity> sampleFiles;

    private List<ParameterValues> parameterValues;

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public String getTechnology() {
        return technology;
    }

    public void setTechnology(String technology) {
        this.technology = technology;
    }

    public String getEquipment() {
        return equipment;
    }

    public void setEquipment(String equipment) {
        this.equipment = equipment;
    }

    public String getMatrix() {
        return matrix;
    }

    public void setMatrix(String matrix) {
        this.matrix = matrix;
    }

    public String getFiber() {
        return fiber;
    }

    public void setFiber(String fiber) {
        this.fiber = fiber;
    }

    public List<Layer> getLayers() {
        return layers;
    }

    public void setLayers(List<Layer> layers) {
        this.layers = layers;
    }

    public List<FileEntity> getMaterialFiles() {
        return materialFiles;
    }

    public void setMaterialFiles(List<FileEntity> materialFiles) {
        this.materialFiles = materialFiles;
    }

    public List<FileEntity> getSampleFiles() {
        return sampleFiles;
    }

    public void setSampleFiles(List<FileEntity> sampleFiles) {
        this.sampleFiles = sampleFiles;
    }

    public List<ParameterValues> getParameterValues() {
        return parameterValues;
    }

    public void setParameterValues(List<ParameterValues> parameterValues) {
        this.parameterValues = parameterValues;
    }

    public boolean containsSampleFile(Integer sampleFileId) {
        if (sampleFiles == null) {
            throw new IllegalStateException("Can't check sample files because material is not fully initialized");
        }
        return sampleFiles.stream()
                .map(FileEntity::getId)
                .anyMatch(id -> id.equals(sampleFileId));

    }

    public TechnologyType getTechnologyType() {
        return technologyType;
    }

    public void setTechnologyType(TechnologyType technologyType) {
        this.technologyType = technologyType;
        if (technologyType == null) {
            technologyTypeName = null;
        } else {
            technologyTypeName = technologyType.getDisplayName();
        }
    }

    public String getTechnologyTypeName() {
        return technologyTypeName;
    }

    public String getEpoxide() {
        return epoxide;
    }

    public void setEpoxide(String epoxide) {
        this.epoxide = epoxide;
    }

    @JsonIgnore
    public boolean isEpoxidePresent() {
        if (technologyType == null) {
            return false;
        }
        return technologyType == TechnologyType.ADDITIVE_3CM;
    }
}
