package ru.crp.lvms.model.entities.effective.characteristics;

import com.fasterxml.jackson.annotation.JsonIgnore;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.web.dto.columns.GridColumn;

import javax.validation.constraints.NotNull;

@Table("effective_characteristics_parameters_view")
public class ECParamEntity implements Entity {
    public enum Type {Microstructure, Mesostructure}

    private int id;

    @NotNull(message = "Эффективная характеристика должна быть задана")
    @Column("effective_characteristic_id")
    private Integer ecId;

    @NotNull(message = "Параметр должен быть задан")
    @Column("param_id")
    private Integer parameterId;

    @GridColumn(label = "Параметр", name = "parameterName", order = 1)
    @Column("param_name")
    private String parameterName;

    @GridColumn(label = "Значение", name = "parameterValue", order = 2)
    @Column("param_value")
    private String parameterValue;

    @Column("param_type")
    private Type type;

    @Column("parent_param_id")
    private Integer parentParamId;

    @Column("parent_param_name")
    private String parentParamName;

    public ECParamEntity() {
    }

    public ECParamEntity(Integer ecId, Integer parameterId) {
        this.ecId = ecId;
        this.parameterId = parameterId;
    }

    public Integer getEcId() {
        return ecId;
    }

    public void setEcId(Integer ecId) {
        this.ecId = ecId;
    }

    public Integer getParameterId() {
        return parameterId;
    }

    public void setParameterId(Integer parameterId) {
        this.parameterId = parameterId;
    }

    public String getParameterValue() {
        return parameterValue;
    }

    public void setParameterValue(String parameterValue) {
        this.parameterValue = parameterValue;
    }

    public String getParameterName() {
        return parameterName;
    }

    public void setParameterName(String parameterName) {
        this.parameterName = parameterName;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    @Override
    public int getId() {
        return id;
    }

    @Override
    public void setId(int id) {
        this.id = id;
    }

    public Integer getParentParamId() {
        return parentParamId;
    }

    public void setParentParamId(Integer parentParamId) {
        this.parentParamId = parentParamId;
    }

    public String getParentParamName() {
        return parentParamName;
    }

    public void setParentParamName(String parentParamName) {
        this.parentParamName = parentParamName;
    }

    @JsonIgnore
    public static boolean isMicrostructure(ECParamEntity ecParamEntity) {
        return ecParamEntity.type == Type.Microstructure;
    }
}
