package ru.crp.lvms.model.entities.misc;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import static java.util.Collections.emptySet;

public enum ItemType {
    Folder,
    ConstructionElementFolder,
    OptimizationFolder,
    File,
    Material,
    CompositeMaterial,
    Technology,
    ConstructionElement;

    public static final String SQL_TYPE_NAME = "tree_node_type";
    // key - parent type, values - set of allowed children types
    private static final Map<ItemType, Set<ItemType>> ALLOWED_CHILDREN = new HashMap<>(ItemType.values().length);
    static {
        ALLOWED_CHILDREN.put(Folder, EnumSet.of(Material, CompositeMaterial, Folder, File, ConstructionElement));
        ALLOWED_CHILDREN.put(Material, EnumSet.of(File));
        ALLOWED_CHILDREN.put(CompositeMaterial, EnumSet.of(File));
        ALLOWED_CHILDREN.put(Technology, emptySet());
        ALLOWED_CHILDREN.put(ConstructionElement, EnumSet.of(ConstructionElementFolder, OptimizationFolder));
        ALLOWED_CHILDREN.put(ConstructionElementFolder, EnumSet.of(File));
        ALLOWED_CHILDREN.put(OptimizationFolder, EnumSet.of(OptimizationFolder));
        ALLOWED_CHILDREN.put(File, emptySet());
    }

    public boolean canBeChildOf(ItemType parentType) {
         if (ALLOWED_CHILDREN.get(parentType) == null) {
             throw new IllegalArgumentException("Unexpected item type: " + parentType);
         }
        return ALLOWED_CHILDREN.get(parentType).contains(this);
    }
}
