package ru.crp.lvms.model.entities.simulations;

import com.fasterxml.jackson.annotation.JsonIgnore;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.cmlb.model.entities.base.Named;
import ru.crp.lvms.managers.image.ImageFileContainer;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntity;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.StandardEntity;
import ru.crp.lvms.validators.EntityExists;
import ru.crp.lvms.validators.Unique;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;
import ru.crp.lvms.web.dto.misc.Auditable;

import javax.annotation.Nullable;
import javax.validation.constraints.NotNull;
import java.util.Date;

@Table("virtual_simulations")
@Unique(entityClass = SimulationEntity.class, fieldName = "name", columnNane = "name", message = "Имя испытания должно быть уникально")
public class SimulationEntityTable implements Entity, Auditable, Named, ImageFileContainer {
    private int id;

    @GridColumn(name = "name", label = "Название", order = 0, cell = GridColumnType.uri)
    @NotNull(message = "Имя должно быть задано")
    @Column("name")
    private String name;

    @GridColumn(name = "description", label = "Описание", order = 99)
    @Column("description")
    private String description;

    @Column("owner_id")
    private int ownerId;

    @NotNull(message = "Тип должен быть задан")
    @Column("type_id")
    private int typeId;

    @Column("experiment_results_file_id")
    private Integer experimentFileId;

    @NotNull(message = "Микроструктура должна быть задана")
    @Column("ec_id")
    private int ecId;

    @NotNull(message = "Композиционный материал должен быть задан")
    @EntityExists(entityClass = CompositeMaterialEntity.class, message = "Материал не найден")
    @Column("composite_material_id")
    private Integer compositeMaterialId;

    @EntityExists(entityClass = FileEntity.class, message = "Файл образца не найден")
    @Column("sample_file_id")
    private Integer sampleFileId;

    @EntityExists(entityClass = StandardEntity.class, message = "ГОСТ не найден")
    @Nullable
    @Column("standard_id")
    private Integer standardId;

    @GridColumn(name = "creationDate", label = "Создано", cell = GridColumnType.date, order = 100)
    @Column(value = "date_created", onlySelect = true)
    private Date creationDate;

    @Column("status_id")
    private int statusId;

    @Column("manual_size_h")
    private Double manualSizeH;

    @Column("manual_size_b")
    private Double manualSizeB;

    @Column("manual_size_l")
    private Double manualSizeL;

    @Column(value = "image_file_id", onlySelect = true)
    @JsonIgnore
    private Integer imageFileId;

    @Override
    public int getId() {
        return id;
    }

    @Override
    public void setId(int id) {
        this.id = id;
    }

    @Override
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    @Override
    public int getOwnerId() {
        return ownerId;
    }

    public void setOwnerId(int ownerId) {
        this.ownerId = ownerId;
    }

    public int getTypeId() {
        return typeId;
    }

    public void setTypeId(int typeId) {
        this.typeId = typeId;
    }

    public Integer getExperimentFileId() {
        return experimentFileId;
    }

    public void setExperimentFileId(Integer experimentFileId) {
        this.experimentFileId = experimentFileId;
    }

    public int getEcId() {
        return ecId;
    }

    public void setEcId(int ecId) {
        this.ecId = ecId;
    }

    public Integer getCompositeMaterialId() {
        return compositeMaterialId;
    }

    public void setCompositeMaterialId(Integer compositeMaterialId) {
        this.compositeMaterialId = compositeMaterialId;
    }

    public Integer getSampleFileId() {
        return sampleFileId;
    }

    public void setSampleFileId(Integer sampleFileId) {
        this.sampleFileId = sampleFileId;
    }

    @Nullable
    public Integer getStandardId() {
        return standardId;
    }

    public void setStandardId(@Nullable Integer standardId) {
        this.standardId = standardId;
    }

    @Override
    public Date getCreationDate() {
        return creationDate;
    }

    @Override
    public void setCreationDate(Date creationDate) {
        this.creationDate = creationDate;
    }

    public int getStatusId() {
        return statusId;
    }

    public void setStatusId(int statusId) {
        this.statusId = statusId;
    }

    public Double getManualSizeH() {
        return manualSizeH;
    }

    public void setManualSizeH(Double manualSizeH) {
        this.manualSizeH = manualSizeH;
    }

    public Double getManualSizeB() {
        return manualSizeB;
    }

    public void setManualSizeB(Double manualSizeB) {
        this.manualSizeB = manualSizeB;
    }

    public Double getManualSizeL() {
        return manualSizeL;
    }

    public void setManualSizeL(Double manualSizeL) {
        this.manualSizeL = manualSizeL;
    }

    @Override
    public Integer getImageFileId() {
        return imageFileId;
    }

    public void setImageFileId(@Nullable Integer imageFileId) {
        this.imageFileId = imageFileId;
    }
}
