package ru.crp.lvms.model.entities.tree;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import org.hibernate.validator.constraints.NotEmpty;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.cmlb.model.entities.base.Named;
import ru.crp.lvms.model.FolderColor;
import ru.crp.lvms.model.entities.misc.ItemType;
import ru.crp.lvms.model.entities.misc.PathEntry;

import javax.validation.constraints.NotNull;
import java.util.List;


@Table("tree")
@JsonIgnoreProperties(ignoreUnknown = true) // For altText and Color
public class TreeNode implements Entity, Named {
    private int id;

    @Column("pid")
    private Integer pid;

    @Column(value = "item_type", dbType = ItemType.SQL_TYPE_NAME)
    @NotNull(message = "Тип должен быть задан")
    private ItemType itemType;

    @Column("name")
    @NotEmpty(message = "Имя должно быть задано")
    private String name;

    @Column(value = "path", isArray = true, onlySelect = true)
    private List<PathEntry> path;

    @Column(value = "path_names", onlySelect = true)
    private String pathNames;

    @Column(value = "children_types", onlySelect = true)
    private ItemType[] childrenTypes;

    @JsonProperty("isParent")
    private Boolean isParent;

    @Override
    public int getId() {
        return id;
    }

    @Override
    public void setId(int id) {
        this.id = id;
    }

    public Integer getPid() {
        return pid;
    }

    public void setPid(Integer pid) {
        this.pid = pid;
    }

    public ItemType getItemType() {
        return itemType;
    }

    public void setItemType(ItemType itemType) {
        this.itemType = itemType;
    }

    @Override
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<PathEntry> getPath() {
        return path;
    }

    public void setPath(List<PathEntry> path) {
        this.path = path;
    }

    public String getPathNames() {
        return pathNames;
    }

    public void setPathNames(String pathNames) {
        this.pathNames = pathNames;
    }

    public ItemType[] getChildrenTypes() {
        return childrenTypes;
    }


    public Boolean getParent() {
        return isParent;
    }


    public void setParent(Boolean parent) {
        isParent = parent;
    }

    public void setChildrenTypes(ItemType[] childrenTypes) {
        this.childrenTypes = childrenTypes;
        this.isParent = childrenTypes != null && childrenTypes.length > 0;
    }

    @JsonIgnore
    public boolean isFolder() {
        return itemType == ItemType.Folder;
    }

    @JsonIgnore
    public boolean isOptimizationFolder() {
        return itemType == ItemType.OptimizationFolder;
    }

    public String getAltText() {
        switch (itemType) {
            case Material:
                return String.format("Материал - %s", getName());
            case CompositeMaterial:
                return String.format("Композиционный материал - %s", getName());
            case Technology:
                return String.format("Технология - %s", getName());
            case ConstructionElement:
                return String.format("Элемент конструкции - %s", getName());
            default:
                return name;
        }
    }

    public FolderColor getColor() {
        return FolderColor.BLACK;
    }
}
