package ru.crp.lvms.model.mappers;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.RowMapper;
import ru.crp.lvms.model.entities.ParameterSample;
import ru.crp.lvms.model.entities.ParameterValues;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class ParameterValuesRowMapper implements RowMapper<ParameterValues> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ParameterValuesRowMapper.class);

    @Override
    public ParameterValues mapRow(ResultSet rs, int rowNum) throws SQLException {
        ParameterValues parameterValues = new ParameterValues();

        parameterValues.setId(rs.getInt("id"));
        parameterValues.setMaterialId(rs.getInt("material_id"));
        parameterValues.setName(rs.getString("name"));
        Integer parameterFileId = rs.getInt("parameter_file_id");
        if (rs.wasNull()) {
            parameterFileId = null;
        }
        parameterValues.setParameterFileId(parameterFileId);

        try {
            Integer[] idArray = (Integer[]) rs.getArray("id_list").getArray();
            BigDecimal[] temperatureArray = (BigDecimal[]) rs.getArray("temp_list").getArray();
            BigDecimal[] valuesArray = (BigDecimal[]) rs.getArray("values_list").getArray();
            BigDecimal[] funcValuesArray = (BigDecimal[]) rs.getArray("func_values_list").getArray();
            BigDecimal[] simValuesArray = (BigDecimal[]) rs.getArray("sim_values_list").getArray();
            List<ParameterSample> tempValueTuples = new ArrayList<>(temperatureArray.length);
            for (int i = 0; i < temperatureArray.length; ++i) {
                tempValueTuples.add(new ParameterSample(idArray[i], temperatureArray[i], valuesArray[i],
                        funcValuesArray[i], simValuesArray[i]));
            }
            parameterValues.setValuesList(tempValueTuples);

            return parameterValues;
        } catch (ClassCastException ex) {
            LOGGER.error("unexpected array format", ex);
            throw new SQLException("can't map parameter values", ex);
        }
    }
}
