package ru.crp.lvms.optimizations.base.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.managers.misc.PreferencesManager;
import ru.crp.lvms.model.AccessRights;
import ru.crp.lvms.model.entities.misc.CreatedBy;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.base.entities.OptimizationReportEntity;
import ru.crp.lvms.optimizations.base.managers.OptimizationManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.ColumnsController;
import ru.crp.lvms.web.dto.pagination.DataRequest;
import ru.crp.lvms.web.dto.pagination.PageingResponse;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

import static ru.crp.lvms.utils.URLs.*;
import static ru.crp.lvms.web.controllers.misc.CrudController.deleteSeveralEntities;

@RestController
@RequestMapping(value = OPTIMIZATION_REPORTS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class OptimizationReportsController {
    @RestController
    @RequestMapping(value = OPTIMIZATIONS_URL + REPORTS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
    public static class OptimizationReportsColumnsController implements ColumnsController {
        @Autowired
        private PreferencesManager preferencesManager;

        @Override
        public Class getOutVoClass() {
            return OptimizationReportEntity.class;
        }

        @Override
        public PreferencesManager getPreferencesManager() {
            return preferencesManager;
        }
    }

    @Autowired
    private OptimizationManager optimizationManager;


    @RequestMapping(method = RequestMethod.GET)
    public Map<CreatedBy, List<OptimizationReportEntity>> getAll(@PathVariable("optimizationId") Integer optimizationId,
                                                                 @CurrentUser PortalUser currentUser) {
        OptimizationEntity optimization = optimizationManager.checkEntityExistsAndAccesible(optimizationId, AccessRights.view, currentUser.getUser());
        return optimizationManager.getOptimizationReports(optimization, currentUser.getUser());
    }


    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public PageingResponse<OptimizationReportEntity> getPage(@PathVariable("optimizationId") Integer optimizationId,
                                                             @RequestBody DataRequest dataRequest,
                                                             @CurrentUser PortalUser currentUser) {
        OptimizationEntity optimization = optimizationManager.checkEntityExistsAndAccesible(optimizationId, AccessRights.view, currentUser.getUser());
        Pageable pageRequest = dataRequest.getPageable().toSpringPageable();
        PageImpl<OptimizationReportEntity> page = optimizationManager.getOptimizationReportsPage(optimization, dataRequest.getFilters(),
                pageRequest, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }


    @RequestMapping(value = "/{reportId}", method = RequestMethod.GET)
    public OptimizationReportEntity getOne(@PathVariable("optimizationId") Integer optimizationId,
                                           @PathVariable("reportId") Integer reportId,
                                           @CurrentUser PortalUser currentUser) {
        optimizationManager.checkEntityExistsAndAccesible(optimizationId, AccessRights.view, currentUser.getUser());
        return optimizationManager.getReport(reportId);
    }


    @RequestMapping(value = "/{reportId}", method = RequestMethod.PUT)
    public OptimizationReportEntity update(@PathVariable("optimizationId") Integer optimizationId,
                                           @PathVariable("reportId") Integer reportId,
                                           @RequestBody @Valid OptimizationReportEntity updatedEntity,
                                           @CurrentUser PortalUser currentUser) {
        optimizationManager.checkEntityExistsAndAccesible(optimizationId, AccessRights.view, currentUser.getUser());
        optimizationManager.updateReport(updatedEntity);
        return optimizationManager.getReport(reportId);
    }


    @RequestMapping(value = "/{reportId}", method = RequestMethod.DELETE)
    @ResponseStatus(HttpStatus.OK)
    public void delete(@PathVariable("optimizationId") Integer optimizationId,
                       @PathVariable("reportId") Integer reportId,
                       @CurrentUser PortalUser currentUser) {
        optimizationManager.checkEntityExistsAndAccesible(optimizationId, AccessRights.view, currentUser.getUser());
        optimizationManager.deleteReport(reportId);
    }

    @RequestMapping(method = RequestMethod.DELETE)
    @ResponseStatus(HttpStatus.OK)
    public void deleteSeveral(@PathVariable("optimizationId") Integer optimizationId,
                              @RequestParam("ids") List<Integer> reportIds,
                              @CurrentUser PortalUser currentUser) throws BadRequestException {
        deleteSeveralEntities(OptimizationReportEntity.class, reportIds, (id) -> {
            delete(optimizationId, id, currentUser);
        });
    }
}
