package ru.crp.lvms.optimizations.base.entities;

import com.fasterxml.jackson.annotation.JsonView;
import org.modelmapper.TypeToken;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.optimizations.base.enums.JsonSettingsType;
import ru.crp.lvms.utils.Mapper;

import javax.annotation.Nullable;
import java.util.List;

// json_settings
public class OptimizationJsonSetting implements Entity {
    @JsonView(SettingsView.Short.class)
    private int id;

    @JsonView(SettingsView.Short.class)
    private JsonSettingsType type;

    @JsonView(SettingsView.Short.class)
    private int optimizationId;

    @JsonView(SettingsView.Short.class)
    private String name;

    // if null then this settings is not used in generated file but is used directly on solver or elsewhere
    @Nullable
    private String xmlName;

    @JsonView(SettingsView.All.class)
    @Nullable
    private Integer parentId;

    @JsonView(SettingsView.All.class)
    @Nullable
    private Boolean boolValue;

    @JsonView(SettingsView.All.class)
    @Nullable
    private String plainValue;

    @JsonView(SettingsView.All.class)
    @Nullable
    private List<OptimizationListSetting> listValue;

    @JsonView(SettingsView.All.class)
    private boolean isMultipleSelectableInList;

    @JsonView(SettingsView.All.class)
    @Nullable
    private List<OptimizationJsonSetting> compositeValue;

    private Object value;

    private Object defaultValue;

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public JsonSettingsType getType() {
        return type;
    }

    public void setType(JsonSettingsType type) {
        this.type = type;
        if (this.value != null) {
            extractValue();
        }
    }

    @Nullable
    public Integer getParentId() {
        return parentId;
    }

    public void setParentId(@Nullable Integer parentId) {
        this.parentId = parentId;
    }

    @Nullable
    public Boolean getBoolValue() {
        return boolValue;
    }

    public void setBoolValue(@Nullable Boolean boolValue) {
        this.boolValue = boolValue;
        if (type == JsonSettingsType.bool) value = boolValue;
    }

    @Nullable
    public String getPlainValue() {
        return plainValue;
    }

    public void setPlainValue(@Nullable String plainValue) {
        this.plainValue = plainValue;
        if (type == JsonSettingsType.plain) value = plainValue;
    }

    @Nullable
    public List<OptimizationListSetting> getListValue() {
        return listValue;
    }

    public void setListValue(@Nullable List<OptimizationListSetting> listValue) {
        this.listValue = listValue;
        if (type == JsonSettingsType.list) value = listValue;
    }

    public int getOptimizationId() {
        return optimizationId;
    }

    public void setOptimizationId(int optimizationId) {
        this.optimizationId = optimizationId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @JsonView(SettingsView.Short.class)
    public Object getValue() {
        return value;
    }

    public void setValue(Object value) {
        this.value = value;
        if (this.type != null) {
            extractValue();
        }
    }

    private void extractValue() {
        if (this.type == JsonSettingsType.plain) {
            this.plainValue = value == null ? null : value.toString();
        }
        if (this.type == JsonSettingsType.bool) {
            this.boolValue = value == null ? null : (boolean) value;
        }
        if (this.type == JsonSettingsType.list) {
            List<OptimizationListSetting> map = Mapper.map(value, new TypeToken<List<OptimizationListSetting>>() {
            });
            this.listValue = value == null ? null : map;
        }
        if (this.type == JsonSettingsType.composite) {
            List<OptimizationJsonSetting> map = Mapper.map(value, new TypeToken<List<OptimizationJsonSetting>>() {
            });
            this.compositeValue = value == null ? null : map;
        }
    }

    public static OptimizationJsonSetting newBoolSetting(int optimizationId, String name, String xmlName, boolean initialValue) {
        return newBoolSetting(optimizationId, null, name, xmlName, initialValue);
    }

    public static OptimizationJsonSetting newBoolSetting(int optimizationId, @Nullable Integer parentId, String name, String xmlName, boolean initialValue) {
        OptimizationJsonSetting s = new OptimizationJsonSetting();
        s.setOptimizationId(optimizationId);
        s.setType(JsonSettingsType.bool);
        s.setName(name);
        s.setXmlName(xmlName);
        s.setParentId(parentId);
        s.setBoolValue(initialValue);
        s.setDefaultValue(initialValue);
        return s;
    }

    public static OptimizationJsonSetting newListSetting(int optId, String name, String xmlName, List<OptimizationListSetting> listSettings) {
        return newListSetting(optId, null, name, xmlName, listSettings);
    }

    public static OptimizationJsonSetting newListSetting(int optId, @Nullable Integer parentId, String name, String xmlName, List<OptimizationListSetting> listSettings) {
        return newListSetting(optId, parentId, name, xmlName, listSettings, false);
    }

    public static OptimizationJsonSetting newListSetting(int optId, @Nullable Integer parentId, String name,
                                                         String xmlName, List<OptimizationListSetting> listSettings,
                                                         Boolean isMultipleSelectableInList) {
        OptimizationJsonSetting s = new OptimizationJsonSetting();
        s.setOptimizationId(optId);
        s.setType(JsonSettingsType.list);
        s.setName(name);
        s.setXmlName(xmlName);
        s.setParentId(parentId);
        s.setListValue(listSettings);
        s.setIsMultipleSelectableInList(isMultipleSelectableInList);
        listSettings.stream()
                .filter(OptimizationListSetting::isSelected)
                .findFirst()
                .ifPresent(s::setDefaultValue);
        return s;
    }

    public static OptimizationJsonSetting newPlainSetting(int optId, String name, String xmlName, @Nullable String initialValue) {
        return newPlainSetting(optId, null, name, xmlName, initialValue);
    }

    public static OptimizationJsonSetting newPlainSetting(int optId, @Nullable Integer parentId, String name, @Nullable String xmlName, @Nullable String initialValue) {
        OptimizationJsonSetting s = new OptimizationJsonSetting();
        s.setOptimizationId(optId);
        s.setType(JsonSettingsType.plain);
        s.setName(name);
        s.setXmlName(xmlName);
        s.setParentId(parentId);
        s.setPlainValue(initialValue);
        s.setDefaultValue(initialValue);
        return s;
    }


    public static OptimizationJsonSetting newCompositeSetting(int optId, @Nullable Integer parentId, String name, String xmlName, List<OptimizationJsonSetting> compositeSettings) {
        OptimizationJsonSetting s = new OptimizationJsonSetting();
        s.setOptimizationId(optId);
        s.setType(JsonSettingsType.composite);
        s.setName(name);
        s.setXmlName(xmlName);
        s.setParentId(parentId);
        s.setCompositeValue(compositeSettings);
        // default value not needed because it's composition of subsettings
        return s;
    }

    public String getXmlName() {
        return xmlName;
    }

    public void setXmlName(String xmlName) {
        this.xmlName = xmlName;
    }

    public boolean getIsMultipleSelectableInList() {
        return isMultipleSelectableInList;
    }

    public void setIsMultipleSelectableInList(boolean multipleSelectableInList) {
        isMultipleSelectableInList = multipleSelectableInList;
    }

    public Object getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(Object defaultValue) {
        this.defaultValue = defaultValue;
    }

    @Nullable
    public List<OptimizationJsonSetting> getCompositeValue() {
        return compositeValue;
    }

    public void setCompositeValue(@Nullable List<OptimizationJsonSetting> compositeValue) {
        this.compositeValue = compositeValue;
        if (type == JsonSettingsType.composite) value = compositeValue;
    }
}
