package ru.crp.lvms.optimizations.base.entities.tree;

import com.fasterxml.jackson.annotation.JsonGetter;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmlb.model.entities.base.Entity;

import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;

@Table("toska_technological_constraints_tree_extended")
public class TechnologicalConstraintTreeNode implements Entity {
    public static final String SELECT_TYPE = "select";
    public static final String CHECKBOX_TYPE = "checkbox";

    private int id;

    @Column("name")
    private String name;

    @Column("display_name")
    private String displayName;

    @Column("is_value_editable")
    private Boolean isValueEditable;

    @Column("parent_id")
    private Integer parentId;

    @Column("tc_id")
    private Integer technologicalConstraintId;

    @Column("is_parent")
    private boolean isParent;

    @Nullable
    @Column("type")
    private String type;

    @Nullable
    @Column("tooltip")
    private String tooltip;

    @Nullable
    @Column("group_name")
    private String groupName;

    private List<TechnologicalConstraintTreeNode> children = Collections.emptyList();

    @Nullable
    private DependenciesDescription dependencies;

    @Override
    public int getId() {
        return id;
    }

    @Override
    public void setId(int id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDisplayName() {
        return displayName;
    }

    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    public Boolean getIsValueEditable() {
        return isValueEditable;
    }

    public void setIsValueEditable(Boolean valueSetable) {
        isValueEditable = valueSetable;
    }

    public Integer getParentId() {
        return parentId;
    }

    public void setParentId(Integer parentId) {
        this.parentId = parentId;
    }

    public Integer getTechnologicalConstraintId() {
        return technologicalConstraintId;
    }

    public void setTechnologicalConstraintId(Integer technologicalConstraintId) {
        this.technologicalConstraintId = technologicalConstraintId;
    }

    public boolean getIsParent() {
        return isParent;
    }

    public void setIsParent(boolean isParent) {
        this.isParent = isParent;
    }

    @Nullable
    public String getType() {
        return type;
    }

    public void setType(@Nullable String type) {
        this.type = type;
    }

    @Nullable
    public String getTooltip() {
        return tooltip;
    }

    public void setTooltip(@Nullable String tooltip) {
        this.tooltip = tooltip;
    }

    @JsonGetter("isSelectable")
    public boolean getIsSelectable() {
        return isParent && (isValueEditable || SELECT_TYPE.equals(type));
    }

    // for json serialization
    public void setIsSelectable(boolean unused) {

    }

    @Nullable
    public List<TechnologicalConstraintTreeNode> getChildren() {
        return children;
    }

    public void setChildren(List<TechnologicalConstraintTreeNode> children) {
        if (children == null) {
            this.children = Collections.emptyList();
        } else {
            this.children = children;
        }
    }

    @Nullable
    public String getGroupName() {
        return groupName;
    }

    public void setGroupName(@Nullable String groupName) {
        this.groupName = groupName;
    }

    @Nullable
    public DependenciesDescription getDependencies() {
        return dependencies;
    }

    public void setDependencies(@Nullable DependenciesDescription dependencies) {
        this.dependencies = dependencies;
    }
}
