

package ru.crp.lvms.optimizations.mf.config.xml;

import ru.crp.lvms.model.entities.modfrontier.AlgorithmType;
import ru.crp.lvms.model.entities.modfrontier.DoeType;

import javax.xml.bind.annotation.*;
import java.util.ArrayList;
import java.util.List;



@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "PLUG_INType", propOrder = {
    "property"
})
public class PlugInType {

    @XmlElement(name = "PROPERTY")
    protected List<PropertyType> property;
    @XmlAttribute(name = "name")
    protected String name;
    @XmlAttribute(name = "class")
    protected String clazz;

    public static PlugInType create(DoeType doeType) {
        PlugInType plugin = new PlugInType();
        switch (doeType) {
            case ULH:
                plugin.setName("Uniform Latin Hypercube");
                plugin.setClazz("it.esteco.frontier.fpi.plugins.doe.uniformlatinhypercube.plugin.UniformLatinHypercubePlugIn");
                break;
            case Random:
                plugin.setName("Random");
                plugin.setClazz("it.esteco.frontier.fpi.plugins.doe.doerand.plugin.DoeRandPlugIn");
                break;
            case Sobol:
                plugin.setName("Sobol");
                plugin.setClazz("it.esteco.frontier.fpi.plugins.doe.sobol.plugin.SobolPlugIn");
                break;
            default:
                throw new RuntimeException("Doe Type not found");
        }
        return plugin;
    }

    public static PlugInType create(AlgorithmType algorithmType) {
        PlugInType plugin = new PlugInType();
        switch (algorithmType) {
            case SIMPLEX:
                plugin.setName("SIMPLEX");
                plugin.setClazz("it.esteco.frontier.fpi.plugins.schedulers.simplex.plugin.SIMPLEXPlugIn");
                break;
            case MOGA_II:
                plugin.setName("MOGA-II");
                plugin.setClazz("it.esteco.frontier.fpi.plugins.schedulers.moga.plugin.MOGAPlugIn");
                break;
            case DOESequence:
                plugin.setName("DOE Sequence");
                plugin.setClazz("it.esteco.frontier.fpi.plugins.schedulers.doeseq.plugin.DOESequencePlugIn");
                break;
        }
        return plugin;
    }

    public List<PropertyType> getProperty() {
        if (property == null) {
            property = new ArrayList<PropertyType>();
        }
        return this.property;
    }

    public void setProperty(List<PropertyType> property) {
        this.property = property;
    }

    public String getName() {
        return name;
    }


    public void setName(String value) {
        this.name = value;
    }


    public String getClazz() {
        return clazz;
    }


    public void setClazz(String value) {
        this.clazz = value;
    }

}
