package ru.crp.lvms.optimizations.mf.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import ru.crp.lvms.optimizations.base.dao.OptimizationJsonSettingsDao;
import ru.crp.lvms.optimizations.base.enums.OptimizationCao;
import ru.crp.lvms.optimizations.mf.entities.MFOptimizationBaseSetting;
import ru.crp.lvms.optimizations.mf.enums.MFBaseSettingType;
import ru.crp.lvms.optimizations.mf.enums.MFSettingsType;
import ru.crp.lvms.optimizations.mf.managers.MFOptimizationSettingsManager;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettingsController;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettingsManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.utils.URLs;

import static ru.crp.lvms.optimizations.mf.enums.MFBaseSettingType.NUM_CAO_VARYING_PARAMS;
import static ru.crp.lvms.optimizations.mf.enums.MFBaseSettingType.NUM_TARGET_FUNCS;

@RestController
@RequestMapping(value = URLs.MF_BASE_SETTINGS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class MFBaseSettingsController extends AbstractOptimizationSettingsController<MFOptimizationBaseSetting, MFSettingsType> {

    @Autowired
    private MFOptimizationSettingsManager MFOptimizationSettingsManager;

    @Autowired
    private OptimizationJsonSettingsDao optimizationJsonSettingsDao;


    @Override
    protected MFSettingsType getSettingsType() {
        return MFSettingsType.BaseSetting;
    }

    @Override
    protected Integer getSettingsCount(int optimizationId) {
        return MFBaseSettingType.values().length;
    }

    @Override
    protected AbstractOptimizationSettingsManager<MFSettingsType> getSettingsManager() {
        return MFOptimizationSettingsManager;
    }

    @Override
    protected OptimizationCao getOptimizationSolverType() {
        return OptimizationCao.ModeFrontier;
    }

    @Override
    public MFOptimizationBaseSetting updateSettings(@PathVariable("optimizationId") int optimizationId,
                                                    @PathVariable("id") int settingsId,
                                                    @RequestBody MFOptimizationBaseSetting updatedSettings,
                                                    @CurrentUser PortalUser currentUser) {
        MFOptimizationBaseSetting resp = (MFOptimizationBaseSetting) super.updateSettings(optimizationId, settingsId, updatedSettings, currentUser);
        if (resp.getTypeId() == NUM_TARGET_FUNCS.getId()) {
            MFOptimizationSettingsManager.selectDefaultAlgorithm(optimizationId, resp.getCurrentValue());
        }
        if (resp.getTypeId() == NUM_TARGET_FUNCS.getId() || resp.getTypeId() == NUM_CAO_VARYING_PARAMS.getId()) {
            Integer selectedAlgorithmId = MFOptimizationSettingsManager.findSelectedAlgorithmId(optimizationId);
            optimizationJsonSettingsDao.resetDefaultDoeParams(optimizationId, selectedAlgorithmId);
        }
        return resp;
    }
}
