package ru.crp.lvms.optimizations.misc;

import ru.crp.lvms.exceptions.NotFoundException;
import ru.crp.lvms.model.entities.UserEntity;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;

public abstract class AbstractOptimizationSettingsManager<SETTING_TYPE> {

    protected Map<SETTING_TYPE, AbstractOptimizationSettignsDao<? extends AbstractOptimizationSettings>> settingsDaos;

    @PostConstruct
    public void setup() {
        settingsDaos = getSettingsDaos();
    }

    protected abstract Map<SETTING_TYPE, AbstractOptimizationSettignsDao<? extends AbstractOptimizationSettings>> getSettingsDaos();

    public List<? extends AbstractOptimizationSettings> findOptimizationSettings(int optimizationId, SETTING_TYPE type,
                                                                                 UserEntity currentUser) {
        return checkDaoExists(type).findForOptimization(optimizationId);
    }

    private <T extends AbstractOptimizationSettings> AbstractOptimizationSettignsDao<T> checkDaoExists(SETTING_TYPE type) {
        if (!settingsDaos.containsKey(type)) {
            throw new NotFoundException("Settings with type " + type + " is not found");
        }
        return (AbstractOptimizationSettignsDao<T>) settingsDaos.get(type);
    }

    public <T extends AbstractOptimizationSettings> AbstractOptimizationSettings updateSettings(T updatedSettings, SETTING_TYPE type) {
        return checkDaoExists(type).updateAndRead(updatedSettings);
    }

    public AbstractOptimizationSettings findOptimizationSettingsById(int settingsId, SETTING_TYPE settingsType, UserEntity user) {
        return checkDaoExists(settingsType).findOne(settingsId);
    }
}
