package ru.crp.lvms.optimizations.optistruct.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructGroupedTFParameters;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructTF;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructTFOptimizationType;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructTechnologicalConstraint;
import ru.crp.lvms.optimizations.optistruct.enums.ComponentType;
import ru.crp.lvms.optimizations.optistruct.enums.LnSetType;
import ru.crp.lvms.optimizations.optistruct.managers.OptistructOptimizationTechnologicalConstraintManager;
import ru.crp.lvms.optimizations.optistruct.managers.OptistructSuggestManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;
import static ru.crp.lvms.utils.URLs.OPTISTRUCT_SUGGEST_URL;

@RestController
@RequestMapping(value = OPTISTRUCT_SUGGEST_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class OptistructSuggestController {

    @Autowired
    private OptistructSuggestManager optistructSuggestManager;

    @Autowired
    private OptistructOptimizationTechnologicalConstraintManager optistructOptimizationTechnologicalConstraintManager;

    @RequestMapping(value = "/targetFunctions", method = RequestMethod.GET)
    public List<OptistructTF> findAllTF() {
        return optistructSuggestManager.findAllTF();
    }

    @RequestMapping(value = "/targetFunctions/types", method = RequestMethod.GET)
    public List<OptistructTFOptimizationType> findAllTFOptimizationTypes() {
        return optistructSuggestManager.findAllTFOptimizationTypes();
    }

    @RequestMapping(value = "/targetFunctions/{tfId}/attributes", method = RequestMethod.GET)
    public OptistructGroupedTFParameters findAllowedTFAttributes(@PathVariable int tfId) {
        return new OptistructGroupedTFParameters(
                optistructSuggestManager.findAllowedAtta(tfId),
                optistructSuggestManager.findAllowedAttb(tfId),
                optistructSuggestManager.findAllowedAtti(tfId),
                optistructSuggestManager.findAllowedPtype(tfId)
        );
    }

    @RequestMapping(value = "/constraints", method = RequestMethod.GET)
    public List<OptistructTF> findAllConstraints() {
        return optistructSuggestManager.findAllTF();
    }

    @RequestMapping(value = "/constraints/{tfId}/attributes", method = RequestMethod.GET)
    public OptistructGroupedTFParameters findAllowedCAttributes(@PathVariable int tfId) {
        return new OptistructGroupedTFParameters(
                optistructSuggestManager.findAllowedAtta(tfId),
                optistructSuggestManager.findAllowedAttb(tfId),
                optistructSuggestManager.findAllowedAtti(tfId),
                optistructSuggestManager.findAllowedPtype(tfId)
        );
    }


    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints", method = RequestMethod.GET)
    public List<OptistructTechnologicalConstraint> getTCs(@CurrentUser PortalUser currentUser) {
        return optistructSuggestManager.findAllTechnologicalConstraints();
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints/{constraintId}/fields/{fieldId}", method = RequestMethod.GET)
    public TechnologicalConstraintTreeNode getFieldOfTC(@PathVariable Integer constraintId,
                                                        @PathVariable Integer fieldId) {
        return optistructOptimizationTechnologicalConstraintManager.getField(fieldId);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints/{constraintId}/fields", method = RequestMethod.GET)
    public List<TechnologicalConstraintTreeNode> getFieldsOfTC(@PathVariable Integer constraintId,
                                                               @RequestParam(required = false) Integer parentId) {
        return optistructOptimizationTechnologicalConstraintManager.getTCTree(constraintId, parentId);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints/{constraintId}/tree", method = RequestMethod.GET)
    public List<TechnologicalConstraintTreeNode> getFullTreeForTC(@PathVariable Integer constraintId) {
        List<TechnologicalConstraintTreeNode> flattenTree = optistructOptimizationTechnologicalConstraintManager.getFullTreeOfTC(constraintId);
        Map<Integer, List<TechnologicalConstraintTreeNode>> treeByParent = flattenTree.stream()
                .filter(node -> node.getParentId() != null)
                .collect(Collectors.groupingBy(TechnologicalConstraintTreeNode::getParentId));
        flattenTree.forEach(node -> node.setChildren(treeByParent.getOrDefault(node.getId(), Collections.emptyList())));

        return flattenTree.stream()
                .filter(node -> node.getParentId() == null)
                .collect(toList());
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/sets/types", method = RequestMethod.GET)
    public ComponentType[] getComponentTypes() {
        return ComponentType.values();
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/lnsets/types", method = RequestMethod.GET)
    public LnSetType[] getLnSetTypes() {
        return LnSetType.values();
    }
}
