package ru.crp.lvms.optimizations.optistruct.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructBaseSettingType;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructSettingsType;
import ru.crp.lvms.optimizations.optistruct.managers.OptistructTechnologicalConstraintTreeValueManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;

import java.util.List;
import java.util.Map;
import java.util.function.Function;

import static java.util.stream.Collectors.toMap;
import static ru.crp.lvms.utils.URLs.OPTISTRUCT_TECH_CONSTRAINT_URL;

@RestController
@RequestMapping(value = OPTISTRUCT_TECH_CONSTRAINT_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class OptistructTechConstraintController extends OptistructAbstractOptimizationSettingsController<OptistructOptimizationTechnologicalConstraint> {


    @Autowired
    private OptistructTechnologicalConstraintTreeValueManager optistructTechnologicalConstraintTreeValueManager;

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/{optimizationTCId}/fields", method = RequestMethod.GET)
    public Map<Integer, TechnologicalConstraintTreeNodeValue> getFieldsOfOptimizationTC(@PathVariable Integer optimizationTCId) {
        return optistructTechnologicalConstraintTreeValueManager.findValuesOfTC(optimizationTCId).stream()
                .collect(toMap(TechnologicalConstraintTreeNodeValue::getTechnologicalConstraintTreeId, Function.identity()));
    }

//    @PreAuthorize("hasRole('ROLE_GUEST')")
//    @RequestMapping(value = "/{optimizationTCId}/fields", method = RequestMethod.POST)
//    public TechnologicalConstraintTreeNodeValue createFieldOfOptimizationTC(@PathVariable Integer constraintId,
//                                                                             @RequestBody TechnologicalConstraintTreeNodeValue created,
//                                                                             @CurrentUser PortalUser user) {
//        created.setTechnologicalConstraintId(constraintId);
//        return optistructTechnologicalConstraintTreeValueManager.createAndRead(created, user.getUser());
//    }

    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{optimizationTCId}/fields/{id}", method = RequestMethod.PUT)
    public TechnologicalConstraintTreeNodeValue updateFieldOfOptimizationTC(@PathVariable Integer optimizationTCId,
                                                                            @PathVariable(value = "id") Integer fieldTreeId,
                                                                            @RequestBody TechnologicalConstraintTreeNodeValue updated,
                                                                            @CurrentUser PortalUser user) {
        updated.setTechnologicalConstraintId(optimizationTCId);
        updated.setTechnologicalConstraintTreeId(fieldTreeId);
        TechnologicalConstraintTreeNodeValue existed = optistructTechnologicalConstraintTreeValueManager.finByTreeId(updated.getTechnologicalConstraintId(), updated.getTechnologicalConstraintTreeId());
        if (existed != null) {
            return optistructTechnologicalConstraintTreeValueManager.updateAndRead(updated, user.getUser());
        } else {
            return optistructTechnologicalConstraintTreeValueManager.createAndRead(updated, user.getUser());
        }

    }

    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{optimizationTCId}/fields/{id}", method = RequestMethod.DELETE)
    public void deleteFieldOfOptimizationTC(@PathVariable Integer optimizationTCId, @PathVariable Integer id, @CurrentUser PortalUser user) {
        optistructTechnologicalConstraintTreeValueManager.deleteByTreeId(id, optimizationTCId);
    }

    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{optimizationTCId}/fields", method = RequestMethod.DELETE)
    public void deleteSeveralFieldsByFieldIds(@PathVariable Integer optimizationTCId, @RequestParam("ids") List<Integer> ids, @CurrentUser PortalUser currentUser) {
        optistructTechnologicalConstraintTreeValueManager.deleteSeveralFieldsByFieldIds(ids, optimizationTCId);
    }


    @Override
    protected OptistructBaseSettingType getOptistructOptimizationBaseSettingType() {
        return OptistructBaseSettingType.NUM_TECH_CONSTRAINTS;
    }

    @Override
    protected OptistructSettingsType getSettingsType() {
        return OptistructSettingsType.TechConstraint;
    }

}
