package ru.crp.lvms.optimizations.optistruct.entities;

import com.fasterxml.jackson.annotation.JsonFormat;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettings;
import ru.crp.lvms.optimizations.optistruct.enums.ComponentType;

import java.util.Objects;

@Table("optistruct_sets_join")
public class OptistructSet extends AbstractOptimizationSettings {

    @Column("is_optimization_space")
    private boolean isOptimizationSpace;

    @Column("is_selected")
    private boolean isSelected;

    @Column("component_type_id")
    private Integer componentTypeId;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Column("component_type_name")
    private ComponentType componentTypeName;

    public OptistructSet() {
        super();
        this.isOptimizationSpace = false;
        this.isSelected = false;
    }

    public OptistructSet(int optimizationId, ComponentType componentTypeName, String name) {
        super(optimizationId, name);
        this.isOptimizationSpace = false;
        this.isSelected = false;
        this.componentTypeName = componentTypeName;
        this.componentTypeId = componentTypeName.getId();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        OptistructSet that = (OptistructSet) o;

        if (isOptimizationSpace != that.isOptimizationSpace) return false;
        if (isSelected != that.isSelected) return false;
        if (getOptimizationId() != that.getOptimizationId()) return false;
        if (!Objects.equals(getName(), that.getName())) return false;
        if (getComponentTypeId() != null ? !getComponentTypeId().equals(that.getComponentTypeId()) : that.getComponentTypeId() != null)
            return false;
        return getComponentTypeName() == that.getComponentTypeName();

    }

    @Override
    public int hashCode() {
        int result = (isOptimizationSpace ? 1 : 0);
        result = 31 * result + (isSelected ? 1 : 0);
        result = 31 * result + (getComponentTypeId() != null ? getComponentTypeId().hashCode() : 0);
        result = 31 * result + (getComponentTypeName() != null ? getComponentTypeName().hashCode() : 0);
        return result;
    }

    public boolean getIsOptimizationSpace() {
        return isOptimizationSpace;
    }

    public void setIsOptimizationSpace(boolean optimizationSpace) {
        isOptimizationSpace = optimizationSpace;
    }

    public boolean getIsSelected() {
        return isSelected;
    }

    public void setIsSelected(boolean selected) {
        isSelected = selected;
    }

    public Integer getComponentTypeId() {
        return componentTypeId;
    }

    public void setComponentTypeId(Integer componentTypeId) {
        this.componentTypeId = componentTypeId;
    }

    public ComponentType getComponentTypeName() {
        return componentTypeName;
    }

    public void setComponentTypeName(ComponentType componentTypeName) {
        this.componentTypeName = componentTypeName;
    }
}
