package ru.crp.lvms.optimizations.optistruct.tcl.tc;

import ru.crp.lvms.exceptions.InternalException;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationTechnologicalConstraint;

import java.util.List;
import java.util.stream.Stream;

public class ExtrusionTCConverter extends OptistructTCConverter {
    public static final int EXTRUSION_TC_ID = 9;
    public static final String WITHOUT_ROTATION_NODE_IDS_COMMAND = "WITHOUT_ROTATION_NODE_IDS";

    public static final String WITH_ROTATION_FIRST_NODE_IDS_COMMAND = "WITH_ROTATION_NODE_IDS_1";
    public static final String WITH_ROTATION_SECOND_NODE_IDS_COMMAND = "WITH_ROTATION_NODE_IDS_2";

    @Override
    public Stream<String> convert(OptistructOptimizationTechnologicalConstraint currentConstraint,
                                  List<TechnologicalConstraintTreeNodeValue> currentConstraintValues,
                                  List<OptistructOptimizationTechnologicalConstraint> allConstraints,
                                  String setName, List<String> alreadyConvertedConstraints) {
        if (currentConstraintValues.isEmpty()) {
            throw new InternalException("Not enough arguments");
        }
        if (currentConstraintValues.stream().anyMatch(v -> v.getName().equals(WITHOUT_ROTATION_NODE_IDS_COMMAND))) {
            return convertWithoutRotation(currentConstraintValues, setName);
        } else {
            return convertWithRotation(currentConstraintValues, setName);
        }
    }


    private Stream<String> convertWithoutRotation(List<TechnologicalConstraintTreeNodeValue> currentConstraintValues, String setName) {
        String createListCommand = generateCreateListCommand(currentConstraintValues, "1", WITHOUT_ROTATION_NODE_IDS_COMMAND);
        String tcCommand = String.format("*topologyextrusionupdate \"%s\" 1 1 1", setName);
        return Stream.of(createListCommand, tcCommand);
    }


    private Stream<String> convertWithRotation(List<TechnologicalConstraintTreeNodeValue> currentConstraintValues, String setName) {
        String createList1Command = generateCreateListCommand(currentConstraintValues, "1", WITH_ROTATION_FIRST_NODE_IDS_COMMAND);
        String createList2Command = generateCreateListCommand(currentConstraintValues, "2", WITH_ROTATION_SECOND_NODE_IDS_COMMAND);
        String tcCommand = String.format("*topologyextrusionupdate \"%s\" 2 1 2", setName);
        return Stream.of(createList1Command, createList2Command, tcCommand);
    }

    private String generateCreateListCommand(List<TechnologicalConstraintTreeNodeValue> currentConstraintValues, String listName, String tcValueName) {
        String nodeIds = currentConstraintValues.stream().filter(c -> c.getName().equals(tcValueName))
                .map(TechnologicalConstraintTreeNodeValue::getValue)
                .map(v -> v == null ? "" : v)
                .findAny().orElse("");
        return String.format("*createlist nodes %s %s", listName, nodeIds);
    }

    @Override
    public boolean canConvert(OptistructOptimizationTechnologicalConstraint constraint) {
        return constraint.getTechnologicalConstraintId().equals(EXTRUSION_TC_ID);
    }
}
