package ru.crp.lvms.optimizations.tosca.config;

import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.base.enums.OptimizationTypes;
import ru.crp.lvms.optimizations.tosca.entities.ToscaSet;
import ru.crp.lvms.optimizations.tosca.enums.SetType;

import java.util.function.Predicate;

public enum OptimizationTypeParameters {
    Bead("_BEAD", "ND_GROUP", set -> set.getSetTypeId() == SetType.Nodes.getId()),
    Shape("_SHAPE", "ND_GROUP", set -> set.getSetTypeId() == SetType.Nodes.getId()),
    Topo("_TOPO", "EL_GROUP", set -> set.getSetTypeId() == SetType.Elements.getId());

    private String dvName;
    private String groupName;
    private Predicate<? super ToscaSet> predicate;

    OptimizationTypeParameters(String dvName, String groupName, Predicate<? super ToscaSet> predicate) {
        this.dvName = dvName;
        this.groupName = groupName;
        this.predicate = predicate;
    }

    public static OptimizationTypeParameters valueOf(OptimizationEntity optimization) {
        switch (OptimizationTypes.valueOf(optimization.getOptimizationTypeId())) {
            case TopographicOptimization:
                return Bead;
            case ShapeOptimization:
                return Shape;
            default:
                return Topo;
        }
    }

    public String getDvName() {
        return dvName;
    }

    public String getGroupName() {
        return groupName;
    }

    public Predicate<? super ToscaSet> getPredicate() {
        return predicate;
    }
}
