package ru.crp.lvms.optimizations.tosca.config.commandConverters;

import org.apache.commons.lang3.tuple.Pair;
import ru.crp.lvms.optimizations.tosca.config.Command;
import ru.crp.lvms.optimizations.tosca.config.OptimizationTypeParameters;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

import static java.util.Arrays.asList;
import static java.util.stream.Collectors.groupingBy;

public class TopographicCommandConverter extends CommandConverter {
    private static final List<String> TOPOGRAPHIC_LINK_PARAM_NAMES = asList("CS", "TYPE");

    public TopographicCommandConverter(ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager) {
        super(toscaOptimizationTechnologicalConstraintManager);
    }

    @Override
    public Stream<Command> convert(OptimizationTypeParameters typeParameters, ToscaOptimizationTechnologicalConstraint tc, Integer index, String space, List<TechnologicalConstraintTreeNodeValue> values) {
        // true - values that should be added to LINK command, false - values that should be added to DVCON command
        Map<Boolean, List<Pair<String, String>>> paramsByCommand = convertTCValues(tc, values).stream()
                .collect(groupingBy(v -> TOPOGRAPHIC_LINK_PARAM_NAMES.contains(v.getKey())));

        List<Pair<String, String>> linkParams = paramsByCommand.getOrDefault(true, Collections.emptyList());
        if (linkParams.isEmpty()) {
            return Stream.of(createDvConCommand(typeParameters, tc, index)
                    .addParameter(typeParameters.getGroupName(), space)
                    .addParameters(paramsByCommand.getOrDefault(false, Collections.emptyList())));
        } else {
            Command linkCommand = new Command("LINK" + typeParameters.getDvName())
                    .addParameter("ID_NAME", LINK_ID_PREFIX + index)
                    .addParameters(linkParams);

            Command dvconCommand = createDvConCommand(typeParameters, tc, index)
                    .addParameter(typeParameters.getGroupName(), space)
                    .addParameters(paramsByCommand.getOrDefault(false, Collections.emptyList()))
                    .addParameter("CHECK_LINK", LINK_ID_PREFIX + index);

            return Stream.of(linkCommand, dvconCommand);
        }
    }


    @Override
    public boolean canConvert(ToscaOptimizationTechnologicalConstraint tc) {
        return tc.getTechnologicalConstraintId() == TOPOGRAPHIC_TC_ID;
    }
}
