package ru.crp.lvms.optimizations.tosca.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.optimizations.tosca.entities.ToscaTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;
import static ru.crp.lvms.utils.URLs.TOSCA_SUGGEST_URL;

@RestController
@RequestMapping(value = TOSCA_SUGGEST_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class ToscaTCSuggestController {

    @Autowired
    private ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager;

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints", method = RequestMethod.GET)
    public List<ToscaTechnologicalConstraint> getTCs(@CurrentUser PortalUser currentUser,
                                                     @PathVariable Integer optimizationTypeId) {
        return toscaOptimizationTechnologicalConstraintManager.findAllToscaTechnologicalConstraint(optimizationTypeId, currentUser.getUser());
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints/{constraintId}/fields/{fieldId}", method = RequestMethod.GET)
    public TechnologicalConstraintTreeNode getFieldOfTC(@PathVariable Integer constraintId,
                                                        @PathVariable Integer fieldId) {
        return toscaOptimizationTechnologicalConstraintManager.getField(fieldId);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints/{constraintId}/fields", method = RequestMethod.GET)
    public List<TechnologicalConstraintTreeNode> getFieldsOfTC(@PathVariable Integer constraintId,
                                                               @RequestParam(required = false) Integer parentId) {
        return toscaOptimizationTechnologicalConstraintManager.getTCTree(constraintId, parentId);
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/technologicalConstraints/{constraintId}/tree", method = RequestMethod.GET)
    public List<TechnologicalConstraintTreeNode> getFullTreeForTC(@PathVariable Integer constraintId) {
        List<TechnologicalConstraintTreeNode> flattenTree = toscaOptimizationTechnologicalConstraintManager.getFullTreeOfTC(constraintId);
        Map<Integer, List<TechnologicalConstraintTreeNode>> treeByParent = flattenTree.stream()
                .filter(node -> node.getParentId() != null)
                .collect(Collectors.groupingBy(TechnologicalConstraintTreeNode::getParentId));
        flattenTree.forEach(node -> node.setChildren(treeByParent.getOrDefault(node.getId(), Collections.emptyList())));

        return flattenTree.stream()
                .filter(node -> node.getParentId() == null)
                .collect(toList());
    }
}
