package ru.crp.lvms.optimizations.tosca.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.CannotSerializeTransactionException;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.lvms.optimizations.tosca.dao.ToscaTargetFunctionParametersDao;
import ru.crp.lvms.optimizations.tosca.entities.ToscaTargetFuncOrConstraintParameter;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

@Service
@Transactional(isolation = Isolation.SERIALIZABLE)
// Set retry on transaction isolation fail, use random exponential back off
@Retryable(include = CannotSerializeTransactionException.class, maxAttempts = 5,
        backoff = @Backoff(delay = 100, multiplier = 2, random = true))
public class ToscaSuggestManager {

    @Autowired
    private ToscaTargetFunctionParametersDao toscaTargetFunctionParametersDao;

    public Map<String, List<ToscaTargetFuncOrConstraintParameter>> getAvailableParameters(Integer optimizationId,
                                                                                          boolean isForTF) {
        return toscaTargetFunctionParametersDao.findAllowed(optimizationId, isForTF).stream()
                .collect(Collectors.groupingBy(ToscaTargetFuncOrConstraintParameter::getGroupName));
    }

    public List<String> getAvailableParametersGroups(Integer optimizationId,
                                                     boolean isForTF) {
        return toscaTargetFunctionParametersDao.findAllowed(optimizationId, isForTF).stream()
                .map(ToscaTargetFuncOrConstraintParameter::getGroupName)
                .distinct()
                .collect(toList());
    }
}
