package ru.crp.lvms.security;

import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import ru.crp.lvms.model.entities.UserEntity;

import javax.annotation.Nullable;
import java.util.Collection;

public class PortalUser extends User {

    private UserEntity user;

    @Nullable
    private Boolean admin = null;

    public PortalUser(String username, String password, boolean enabled, boolean accountNonExpired,
                      boolean credentialsNonExpired, boolean accountNonLocked,
                      Collection<? extends GrantedAuthority> authorities, UserEntity user) {
        super(username, password, enabled, accountNonExpired, credentialsNonExpired, accountNonLocked, authorities);
        this.user = user;
    }

    public UserEntity getUser() {
        return user;
    }

    // When user is logged in - it will return null
    @Nullable
    public static PortalUser getCurrentUser() {
        SecurityContext context = SecurityContextHolder.getContext();
        if (context == null) {
            return null;
        }
        Authentication authentication = context.getAuthentication();
        return authentication == null ? null : ((PortalUser) (authentication.getPrincipal()));
    }

    public boolean isAdmin() {
        if (admin == null) {
            admin = Boolean.FALSE;
            for (GrantedAuthority authority : getAuthorities()) {
                if (authority.getAuthority().endsWith("_ADMIN")) {
                    admin = Boolean.TRUE;
                    break;
                }
            }
        }
        return admin;
    }


    @Override
    public boolean equals(Object o) {
        return this == o || !(o == null || getClass() != o.getClass()) && user != null
                && user.equals(((PortalUser) o).user);
    }

    @Override
    public int hashCode() {
        return user != null ? user.hashCode() : 0;
    }
}
