package ru.crp.lvms.tasks.misc;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.concurrent.ListenableFuture;
import ru.crp.lvms.cluster.model.ClusterEntity;
import ru.crp.lvms.model.entities.tasks.SlurmTaskInfo;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.slurm.SlurmTaskInfoDao;

import java.util.Optional;

abstract public class BaseTaskCommand<T, S> extends CommandCallbackHelper {
    @Autowired
    protected SlurmTaskInfoDao slurmTaskInfoDao;

    // check if we can try to start task in separate thread
    public boolean isTaskReadyForProcessing(TaskEntity task, ClusterEntity taskCluster) {
        return taskCluster.getIsEnabled();
    }

    abstract public ListenableFuture<T> processTask(TaskEntity task, Object additionalParams) throws Exception;

    protected boolean isSlurmTaskStarted(TaskEntity task) {
        return Optional.ofNullable(slurmTaskInfoDao.getSlurmTaskInfo(task.getId()))
                .map(SlurmTaskInfo::getSlurmTaskId)
                .filter(id -> id > 0)
                .isPresent();
    }
}
