package ru.crp.lvms.tasks.misc;

import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.ListenableFutureCallback;
import ru.crp.lvms.cluster.model.ClusterEntity;
import ru.crp.lvms.cluster.model.ClusterType;
import ru.crp.lvms.model.entities.tasks.TaskEntity;

public interface ClusterCommandsManager {
    ClusterType getClusterType();

    default boolean isTaskReadyForStart(TaskEntity taskEntity, ClusterEntity taskCluster) {
        return getStartCommand().isTaskReadyForProcessing(taskEntity, taskCluster);
    }

    default void startTask(TaskEntity task) throws Exception {
        runCommand(getStartCommand(), getStartCommandCallback(), task, null);
    }


    default boolean isTaskReadyForCheck(TaskEntity taskEntity, ClusterEntity taskCluster) {
        return getCheckCommand().isTaskReadyForProcessing(taskEntity, taskCluster);
    }

    default void checkTaskStatus(TaskEntity task) throws Exception {
        runCommand(getCheckCommand(), getCheckCommandCallback(), task, null);
    }


    default boolean isTaskReadyForDownloadResults(TaskEntity taskEntity, ClusterEntity taskCluster) {
        return getDownloadResultsCommand().isTaskReadyForProcessing(taskEntity, taskCluster);
    }

    default void downloadResultsForTask(TaskEntity task) throws Exception {
        runCommand(getDownloadResultsCommand(), getDownloadResultsCommandCallback(), task, null);
    }


    default boolean isTaskReadyForCancel(TaskEntity taskEntity, ClusterEntity taskCluster) {
        return getCancelCommand().isTaskReadyForProcessing(taskEntity, taskCluster);
    }

    default void cancelTask(TaskEntity task, Object additionalParams) throws Exception {
        runCommand(getCancelCommand(), getCancelCommandCallback(), task, additionalParams);
    }


    default ListenableFuture runCommand(BaseTaskCommand<?, ?> command, BaseTaskCommandCallback<?> callback, TaskEntity task, Object additionalParams) throws Exception {
        ListenableFuture futureResult = command.processTask(task, additionalParams);
        if (callback != null) {
            futureResult.addCallback(new ListenableFutureCallback() {
                @Override
                public void onFailure(Throwable ex) {
                    callback.onFailure(task, ex);
                }

                @Override
                public void onSuccess(Object result) {
                    callback.onSuccess(task, result);
                }
            });
        }
        return futureResult;
    }

    BaseTaskCommand<?, ?> getStartCommand();

    BaseTaskCommandCallback<?> getStartCommandCallback();

    BaseTaskCommand<?, ?> getCheckCommand();

    BaseTaskCommandCallback<?> getCheckCommandCallback();

    BaseTaskCommand<?, ?> getDownloadResultsCommand();

    BaseTaskCommandCallback<?> getDownloadResultsCommandCallback();

    BaseTaskCommand<?, ?> getCancelCommand();

    BaseTaskCommandCallback<?> getCancelCommandCallback();

}
