package ru.crp.lvms.tasks.slurm;

import org.apache.commons.io.input.ReversedLinesFileReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.cmls.calculations.solvers.xml.config.*;
import ru.crp.cmls.calculations.solvers.xml.config.files.FileResolver;
import ru.crp.lvms.cluster.ClustersManager;
import ru.crp.lvms.managers.solvers.SolverConfigManager;
import ru.crp.lvms.managers.tasks.TaskStatusUpdater;
import ru.crp.lvms.model.entities.tasks.TaskEntity;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.Optional;
import java.util.function.BiFunction;
import java.util.function.Consumer;

public class SlurmCalculationStatusExtractor implements Consumer<TaskEntity> {
    public static final Logger LOGGER = LoggerFactory.getLogger(SlurmCalculationStatusExtractor.class);

    // number of lines which need to be checked from the end of log file
    private static final int LAST_LOG_FILE_LINES_COUNT = 1000;

    @Autowired
    private SolverConfigManager solverConfigManager;

    @Autowired
    private TaskStatusUpdater taskStatusUpdater;

    @Autowired
    private ClustersManager clustersManager;

    @Override
    public void accept(TaskEntity task) {
        if (!clustersManager.isSlurmTask(task)) {
            return;
        }
        CalculationStatus calculationStatus = checkResultDir(task);
        if (calculationStatus == CalculationStatus.Failed) {
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Finished);
        }
    }


    private CalculationStatus checkResultDir(TaskEntity task) {
        SolverConfig config = solverConfigManager.getSolverByName(task.getSolverName());
        Type type = task.getType();

//        Path outputDir = compositeTaskParentManager.getResultsDir(task);
//        Path inputDir = compositeTaskParentManager.getSlurmInputDir(task);
//        BasePlaceholdersResolver placeholderResolver = new BasePlaceholdersResolver(task.getParentName(), inputDir, outputDir);
//        // todo change to use real task files
//        List<FileData> taskFiles = emptyList();
//        CompositeFileResolver fileResolver = new CompositeFileResolver(placeholderResolver, inputDir, outputDir, taskFiles);
//
//        List<LogFile> logFiles = config.getLogFiles(type);
//
//        ParsingStatus parsingStatus = parseLogsReverse(fileResolver, logFiles);
//        return parsingStatus.toCalculationStatus();
        return null;
    }


    private ParsingStatus parseLogsReverse(FileResolver fileResolver, List<LogFile> logFiles) {
        return parseLogs(fileResolver, logFiles, (pathToLog, checkLines) -> {
            try (ReversedLinesFileReader reader = new ReversedLinesFileReader(pathToLog.toFile(), 4096, StandardCharsets.UTF_8.toString())) {
                ParsingStatus status = ParsingStatus.uncompleted;
                String line;
                int checkedLinesCount = 0;
                while ((line = reader.readLine()) != null && checkedLinesCount < LAST_LOG_FILE_LINES_COUNT) {
//                    solvingProgressMonitor.checkLogFileLine(line);
                    Optional<ParsingStatus> logParsingStatus = checkLogFileLine(checkLines, line);
                    if (logParsingStatus.isPresent()) {
                        status = logParsingStatus.get();
                        // if we found error then search is finished otherwise we need to check remaining lines to ensure
                        // that they doesn't contain error pattern
                        if (status == ParsingStatus.error) {
                            break;
                        }
                    }
                    checkedLinesCount++;
                }
//                solvingProgressMonitor.checkProgress();
                return status;
            } catch (IOException e) {
                LOGGER.error("IO exception while parsing logs", e);
            }
            return ParsingStatus.uncompleted;
        });
    }


    private ParsingStatus parseLogs(FileResolver fileResolver, List<LogFile> logFiles, BiFunction<Path, List<CheckLine>, ParsingStatus> linesParser) {
        if (logFiles != null) {
            for (LogFile logFile : logFiles) {
                List<Path> pathsToLog = fileResolver.resolve(logFile.getFile());
                for (Path pathToLog : pathsToLog) {
                    if (pathToLog == null || !Files.isRegularFile(pathToLog)) {
                        continue;
                    }
                    LOGGER.info("Checking log file: " + pathsToLog);
                    return linesParser.apply(pathToLog, logFile.getCheckLines());
                }
            }
        }
        LOGGER.info("No log file found. Returning uncompleted status");
        return ParsingStatus.uncompleted;
    }


    private Optional<ParsingStatus> checkLogFileLine(List<CheckLine> checkLines, String line) {
        return checkLines.stream()
                .filter(checkLine -> checkLine.getCompiledPattern().matcher(line).matches())
                .map(CheckLine::getStatus)
                .findFirst();
    }
}
