package ru.crp.lvms.tasks.slurm;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.crp.cmlb.model.entities.CalculationStatus;

import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import static java.util.Collections.emptyList;
import static java.util.stream.Collectors.toList;

@Service
public class SlurmTaskInfoMapper {
    private static final Logger LOGGER = LoggerFactory.getLogger(SlurmTaskInfoMapper.class);

    private static final Pattern MULTIPLE_NODES_PATTERN = Pattern.compile("(\\w*)\\[(\\d+)-(\\d+)\\]");
    public static final String NODES_LIST_KEY = "NodeList";

    public List<String> extractTaskNodes(Map<String, String> taskInfo) {
        String nodeListStr = taskInfo.get(NODES_LIST_KEY);
        if (nodeListStr.equals("(null)")) {
            return emptyList();
        }
        return Stream.of(nodeListStr.split(","))
                .flatMap(this::parseNodes)
                .collect(toList());
    }


    private Stream<String> parseNodes(String nodesInfo) {
        Matcher matcher = MULTIPLE_NODES_PATTERN.matcher(nodesInfo);
        if (!matcher.matches()) {
            return Stream.of(nodesInfo);
        }
        String prefix = matcher.group(1) == null ? "" : matcher.group(1);
        Integer startNodeNumber = Integer.parseInt(matcher.group(2));
        Integer endNodeNumber = Integer.parseInt(matcher.group(3));
        return IntStream.rangeClosed(startNodeNumber, endNodeNumber)
                .mapToObj(nodeNumber -> prefix + nodeNumber);
    }


    //todo extract slurm job statuses to enum
    public CalculationStatus extractTaskStatus(Map<String, String> taskInfo) {
        String jobState = taskInfo.getOrDefault("JobState", "");
        if (jobState.equalsIgnoreCase("PENDING")) {
            return CalculationStatus.Waiting;
        }
        if (jobState.equalsIgnoreCase("COMPLETED")) {
            return CalculationStatus.CalculationFinished;
        }
        if (jobState.equalsIgnoreCase("FAILED")) {
            return CalculationStatus.Failed;
        }
        if (jobState.equalsIgnoreCase("RUNNING")) {
            return CalculationStatus.Solving;
        }
        if (jobState.equalsIgnoreCase("CANCELLED")) {
            return CalculationStatus.Canceled;
        }
        LOGGER.error(String.format("Unexpected slurm job status '%s'", jobState));
        return CalculationStatus.Solving;
    }
}
