package ru.crp.lvms.tasks.slurm.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.misc.BaseTaskCommandCallback;

@Service
public class SlurmDownloadResultsCommandCallback extends BaseTaskCommandCallback<Void> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SlurmDownloadResultsCommandCallback.class);

    @Override
    public void onSuccess(TaskEntity task, Object unused) {
        LOGGER.info("Successfully extracted results for task {}", task.getId());
        if (task.getCalculationFinishedWithException()) {
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
        } else {
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Finished);
            taskStatusUpdater.findNextTask(task.getId()).ifPresent(nextTask ->
                    taskStatusUpdater.changeTaskStatus(nextTask, CalculationStatus.Waiting)
            );
        }
    }

    @Override
    public void onFailure(TaskEntity task, Throwable error) {
        LOGGER.error("Exception while extracting results for task {}", task.getId(), error);
        taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
    }
}
