package ru.crp.lvms.tasks.slurm.enums;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static java.util.Arrays.asList;

public enum SlurmNodeStatus {
    ALLOCATED("ALLOCATED"),
    COMPLETING("COMPLETING"),
    DOWN("DOWN"),
    DRAINED("DRAINED"),
    DRAINING("DRAINING"),
    ERROR("ERROR"),
    FAIL("FAIL"),
    FAILING("FAILING"),
    FUTURE("FUTURE"),
    IDLE("IDLE"),
    MAINT("MAINT"),
    MIXED("MIXED"),
    PERFCTRS("PERFCTRS"),
    POWER_DOWN("POWER_DOWN"),
    POWER_UP("POWER_UP"),
    RESERVED("RESERVED"),
    UNKNOWN("UNKNOWN");

    private static final Logger LOGGER = LoggerFactory.getLogger(SlurmNodeStatus.class);
    private String displayName;

    SlurmNodeStatus(String displayName) {
        this.displayName = displayName;
    }

    public static SlurmNodeStatus toNodeStatus(String displayName) {
        if (displayName.endsWith("+") || displayName.endsWith("*")) {
            displayName = displayName.substring(0, displayName.length() - 1);
        }
        for (SlurmNodeStatus status : values()) {
            if (status.displayName.equalsIgnoreCase(displayName)) {
                return status;
            }
        }
        LOGGER.error(String.format("Unexpected node status: %s", displayName));
        return UNKNOWN;
    }


    public boolean isAlive() {
        return asList(ALLOCATED, COMPLETING, IDLE, MIXED).contains(this);
    }

    public boolean isBusy() {
        return asList(ALLOCATED, COMPLETING, MIXED).contains(this);
    }
}
