package ru.crp.lvms.tasks.space.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.cmlb.model.entities.ResourcePreparationException;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.CheckTaskStatusJob;
import ru.crp.lvms.tasks.misc.BaseTaskCommandCallback;
import ru.crp.lvms.tasks.space.exceptions.*;


@Component
public class SpaceStartTaskCommandCallback extends BaseTaskCommandCallback<Void> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SpaceStartTaskCommandCallback.class);

    @Autowired
    private CheckTaskStatusJob checkTaskStatusJob;


    @Override
    public void onFailure(TaskEntity task, Throwable error) {
        int taskId = task.getId();

        if (error instanceof NodeNotResponseException) {
            LOGGER.warn("Connection error while starting task " + taskId, error);
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Waiting);
            // unrecoverable errors
        } else if (error instanceof ResourcePreparationException) {
            LOGGER.error(error.getMessage(), error);
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
        } else if (error instanceof BusyNodeException) {
            LOGGER.error(error.getMessage(), error);
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
        } else if (error instanceof OutdatedTaskStateException) {
            LOGGER.warn("Outdated state for task " + taskId + " detected. Current task state: " + error.getMessage());
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.valueOf(error.getMessage()));
        } else if (error instanceof TaskStartingException) {
            LOGGER.error("Can't start task " + taskId, error);
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
        } else if (error instanceof SolverTypeNotFoundException) {
            LOGGER.error(error.getMessage(), error);
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
        } else {
            LOGGER.error("Unexpected error while starting task", error);
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Failed);
        }
    }


    @Override
    public void onSuccess(TaskEntity task, Object unused) {
        taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Starting);
    }
}
