package ru.crp.lvms.validators.task;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.lvms.cluster.model.ClusterType;
import ru.crp.lvms.managers.CompositeMaterialManager;
import ru.crp.lvms.managers.NodesManager;
import ru.crp.lvms.managers.solvers.SolverConfigManager;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntity;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.model.entities.tasks.TaskParent;
import ru.crp.lvms.model.entities.technology.TechnologyType;

import java.util.List;

abstract public class BaseTaskParamsValidator<T extends TaskParent> implements Validator {
    @Autowired
    private SolverConfigManager solverConfigManager;

    @Autowired
    private NodesManager nodesManager;

    @Autowired
    private CompositeMaterialManager compositeMaterialManager;

    @Override
    public boolean supports(Class<?> clazz) {
        return TaskEntity.class.equals(clazz);
    }

    @Override
    public void validate(Object target, Errors errors) {
        TaskEntity task = (TaskEntity) target;
        if (solverConfigManager.getSolverByName(task.getSolverName()) == null) {
            errors.reject("solverNotFound", new Object[]{task.getSolverName()}, null);
        }
        T taskParent = getTaskParent(task);
        if (taskParent == null) {
            errors.reject("solvingObjectNotFound");
            return;
        }
        if (!(task.getStatusId() == CalculationStatus.Draft.getId() || taskParent.getStatus().isRunnable())) {
            errors.reject("solvingAlreadyStarted");
            return;
        }
        if (task.getClusterId() == ClusterType.Space.getId() && task.getNodeId() != null) {
            List<String> availableSolvers = nodesManager.findNodeSolvers(task.getNodeId());
            if (!availableSolvers.contains(task.getSolverName())) {
                errors.reject("solverNotFoundOnNode", new Object[]{task.getSolverName()}, null);
                return;
            }

        }
        doValidate(task, taskParent, errors);
    }


    protected void validateParentCompositeMaterial(Integer compositeMaterialId, Errors errors) {
        CompositeMaterialEntity parentCompositeMaterial = compositeMaterialManager.findOne(compositeMaterialId, null);
        if (parentCompositeMaterial.getTechnologyType() == null) {
            errors.reject("compositeMaterialHasEmptyTechnology");
            return;
        } else if (parentCompositeMaterial.getTechnologyType() == TechnologyType.NOT_ADDITIVE) {
            errors.reject("compositeMaterialHasInappropriateTechnology", new Object[]{parentCompositeMaterial.getTechnology()}, null);
            return;
        }
    }

    abstract protected void doValidate(TaskEntity task, T taskParent, Errors errors);

    abstract protected T getTaskParent(TaskEntity task);
}
