package ru.crp.lvms.validators.task;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;
import ru.crp.cmls.api.TaskParentType;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.validators.task.optimization.CompositeOptimizationTaskParamsValidator;

import javax.annotation.PostConstruct;
import java.util.EnumMap;
import java.util.Map;

@Component
public class CompositeTaskParamsValidator implements Validator {
    @Autowired
    private ECTaskParamsValidator ecTaskParamsValidator;

    @Autowired
    private SimulationTaskParamsValidator simulationTaskParamsValidator;

    @Autowired
    private CompositeOptimizationTaskParamsValidator optimizationTaskParamsValidator;

    private Map<TaskParentType, Validator> taskParamsValidators;

    @PostConstruct
    public void init() {
        taskParamsValidators = new EnumMap<>(TaskParentType.class);
        taskParamsValidators.put(TaskParentType.effectiveCharacteristic, ecTaskParamsValidator);
        taskParamsValidators.put(TaskParentType.simulation, simulationTaskParamsValidator);
        taskParamsValidators.put(TaskParentType.optimization, optimizationTaskParamsValidator);
    }

    @Override
    public boolean supports(Class<?> clazz) {
        return TaskEntity.class.equals(clazz);
    }

    @Override
    public void validate(Object target, Errors errors) {
        TaskEntity task = (TaskEntity) target;
        Validator validator = taskParamsValidators.get(task.getParentType());
        if (validator == null) {
            errors.reject("unexpectedTaskParent", new Object[]{task.getParentType()}, null);
            return;
        }
        validator.validate(target, errors);
    }
}
