package ru.crp.lvms.validators.task.optimization;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.base.enums.OptimizationCae;
import ru.crp.lvms.optimizations.base.enums.OptimizationCao;
import ru.crp.lvms.optimizations.base.managers.OptimizationManager;
import ru.crp.lvms.validators.task.BaseTaskParamsValidator;

import static ru.crp.cmls.api.TaskParentType.optimization;

@Component
public class CompositeOptimizationTaskParamsValidator extends BaseTaskParamsValidator<OptimizationEntity> {
    @Autowired
    private OptimizationManager optimizationManager;

    @Autowired
    private MFTaskParamsValidator mfTaskParamsValidator;

    @Autowired
    private OptimusTaskParamsValidator optimusTaskParamsValidator;

    @Autowired
    private ToscaTaskParamsValidator toscaTaskParamsValidator;

    @Autowired
    private OptistructTaskParamsValidator optistructTaskParamsValidator;

    @Autowired
    private BaseOptimizationTaskParamsValidator baseOptimizationTaskParamsValidator;

    @Override
    protected void doValidate(TaskEntity task, OptimizationEntity taskParent, Errors errors) {
        if (task.getParentType() != optimization) {
            throw new IllegalArgumentException("Wrong validator is used for task with parent type " + task.getParentType());
        }

        if (taskParent.getSolverCaoId() != null) {
            switch (OptimizationCao.valueOf(taskParent.getSolverCaoId())) {
                case Tosca:
                    toscaTaskParamsValidator.doValidate(task, taskParent, errors);
                    break;
                case ModeFrontier:
                    mfTaskParamsValidator.doValidate(task, taskParent, errors);
                    break;
                case Optimus:
                    optimusTaskParamsValidator.doValidate(task, taskParent, errors);
                    break;
                case Optistruct:
                    optistructTaskParamsValidator.doValidate(task, taskParent, errors);
                    break;
                default:
                    throw new IllegalArgumentException("Optimizations with this solver type unsupported for now");
            }
        } else if (taskParent.getSolverCaeId() != null) {
            switch (OptimizationCae.valueOf(taskParent.getSolverCaeId())) {
                case ANSYS:
                case ABAQUS:
                case NASTRAN:
                case NXNASTRAN:
                case LSDYNA:
                    baseOptimizationTaskParamsValidator.doValidate(task, taskParent, errors);
                    break;
                default:
                    throw new IllegalArgumentException("Optimizations with this solver type unsupported for now");
            }
        } else {
            throw new IllegalArgumentException("Solver type not set");
        }
    }

    @Override
    protected OptimizationEntity getTaskParent(TaskEntity task) {
        return optimizationManager.findOne(task.getParentId(), null);
    }
}
