package ru.crp.lvms.web.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.managers.misc.PreferencesManager;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.ColumnsController;
import ru.crp.lvms.web.controllers.misc.FileHelper;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;

import static ru.crp.lvms.utils.URLs.FILES_URL;

@RestController
@RequestMapping(value = FILES_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class FilesController implements ColumnsController, FileHelper {
    @Autowired
    private PreferencesManager preferencesManager;

    @Autowired
    private FileManager fileManager;


    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public FileEntity getFile(@PathVariable("id") int fileId, @CurrentUser PortalUser currentUser) {
        return fileManager.checkEntityExists(fileId, currentUser.getUser());
    }


    @RequestMapping(value = "/{id}/content", method = RequestMethod.GET)
    public void getFileContent(HttpServletResponse response,
                               @PathVariable("id") int fileId, @CurrentUser PortalUser currentUser) throws IOException {
        FileEntity file = fileManager.checkEntityExists(fileId, currentUser.getUser());
        writeFileToResponse(file, response, currentUser.getUser());
    }


    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public FileEntity updateFile(@PathVariable("id") int fileId,
                                 @Valid @RequestBody FileEntity updatedFile,
                                 @CurrentUser PortalUser currentUser) {
        fileManager.checkEntityExists(fileId, currentUser.getUser());
        fileManager.updateFileEntity(updatedFile, currentUser.getUser());
        return getFile(fileId, currentUser);
    }


    @Override
    public Class getOutVoClass() {
        return FileEntity.class;
    }

    @Override
    public PreferencesManager getPreferencesManager() {
        return preferencesManager;
    }

    @Override
    public FileManager getFileManager() {
        return fileManager;
    }
}
