package ru.crp.lvms.web.controllers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.exceptions.NotFoundException;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.managers.HelpManager;
import ru.crp.lvms.managers.TechnologyManager;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.FileHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;

import static java.util.Collections.singletonMap;
import static ru.crp.lvms.utils.MessageUtils.WRONG_LOGIN;

@RestController
@RequestMapping(value = "/help", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
public class HelpController implements FileHelper {
    private static final Logger LOGGER = LoggerFactory.getLogger(HelpController.class);

    @Autowired
    private HelpManager helpManager;

    @Autowired
    private FileManager fileManager;


    @RequestMapping(value = "/content", method = RequestMethod.GET)
    public void getHelpFile(HttpServletResponse response,
                                   @CurrentUser PortalUser currentUser) throws IOException {

        Path pathToFile = getFileManager().resolvePathToFile("help/help.pdf",null);
        if (!Files.exists(pathToFile)) {
            throw new NotFoundException("Файл не найден");
        }
        response.setHeader("Content-Type", "application/pdf");
        Files.copy(pathToFile, response.getOutputStream());
    }

    @RequestMapping(value = "/getBookmark", method = RequestMethod.GET)
    public String getBookmark(@RequestParam("pageUrl") String pageUrl,
                              @CurrentUser PortalUser currentUser) throws IOException {
        return helpManager.getBookmarkName(pageUrl);
    }

    @Override
    public FileManager getFileManager() {
        return fileManager;
    }
}
