package ru.crp.lvms.web.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.managers.StandardManager;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.managers.misc.EventManager;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.StandardEntity;
import ru.crp.lvms.model.entities.misc.FileType;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.CrudControllerSuggest;
import ru.crp.lvms.web.controllers.misc.FileHelper;

import javax.annotation.Nonnull;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;

import static ru.crp.lvms.utils.URLs.STANDARDS_URL;

@RestController
@RequestMapping(value = STANDARDS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class StandardController extends CrudControllerSuggest<StandardEntity, StandardEntity, StandardEntity>
        implements FileHelper {
    @Autowired
    private StandardManager standardManager;

    @Autowired
    private EventManager eventManager;

    @Autowired
    private FileManager fileManager;

    @Nonnull
    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
    public StandardEntity createStandard(@Valid @RequestBody StandardEntity newStandard,
                                         @CurrentUser PortalUser currentUser) {
        return standardManager.createAndRead(newStandard, null, null, currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public StandardEntity createStandard(@Valid @RequestPart StandardEntity newStandard,
                                         @RequestPart(required = false) MultipartFile standardFile,
                                         @RequestPart(required = false) MultipartFile standardLoadFile,
                                         @CurrentUser PortalUser currentUser) {
        return standardManager.createAndRead(newStandard, standardFile, standardLoadFile, currentUser.getUser());
    }


    // working with standard file
    @RequestMapping(value = "/{id}/standardFile", method = RequestMethod.GET)
    public void getStandardFile(HttpServletResponse response,
                                @PathVariable("id") Integer standardId,
                                @CurrentUser PortalUser currentUser) throws IOException {
        FileEntity file = standardManager.checkAndGetFile(standardId, FileType.StandardFile, currentUser.getUser());
        writeFileToResponse(file, response, currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = "/{id}/standardFile", method = RequestMethod.POST)
    public StandardEntity updateStandardFile(HttpServletRequest request,
                                             @PathVariable("id") Integer standardId,
                                             @RequestParam("file") MultipartFile standardFile,
                                             @CurrentUser PortalUser currentUser) {
        standardManager.updateFile(standardId, FileType.StandardFile, standardFile, currentUser.getUser());
        return get(request, standardId, currentUser);
    }


    @Nonnull
    @RequestMapping(value = "/{id}/standardFile", method = RequestMethod.DELETE)
    public StandardEntity deleteStandardFile(HttpServletRequest request,
                                             @PathVariable("id") Integer standardId,
                                             @CurrentUser PortalUser currentUser) {
        standardManager.deleteFile(standardId, FileType.StandardFile, currentUser.getUser());
        return get(request, standardId, currentUser);
    }


    // working with load file
    @RequestMapping(value = "/{id}/loadFile", method = RequestMethod.GET)
    public void getLoadFile(HttpServletResponse response,
                            @PathVariable("id") Integer standardId,
                            @CurrentUser PortalUser currentUser) throws IOException {
        FileEntity file = standardManager.checkAndGetFile(standardId, FileType.StandardLoadFile, currentUser.getUser());
        writeFileToResponse(file, response, currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = "/{id}/loadFile", method = RequestMethod.POST)
    public StandardEntity updateLoadFile(HttpServletRequest request,
                                         @PathVariable("id") Integer standardId,
                                         @RequestParam("file") MultipartFile loadFile,
                                         @CurrentUser PortalUser currentUser) {
        standardManager.updateFile(standardId, FileType.StandardLoadFile, loadFile, currentUser.getUser());
        return get(request, standardId, currentUser);
    }


    @Nonnull
    @RequestMapping(value = "/{id}/loadFile", method = RequestMethod.DELETE)
    public StandardEntity deleteLoadFile(HttpServletRequest request,
                                         @PathVariable("id") Integer standardId,
                                         @CurrentUser PortalUser currentUser) {
        standardManager.deleteFile(standardId, FileType.StandardLoadFile, currentUser.getUser());
        return get(request, standardId, currentUser);
    }


    @Override
    protected BaseManagerSuggest<StandardEntity> getManager() {
        return standardManager;
    }

    @Override
    protected EventManager getEventsManager() {
        return eventManager;
    }

    @Override
    public FileManager getFileManager() {
        return fileManager;
    }
}
