package ru.crp.lvms.web.controllers.misc;

import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import ru.crp.lvms.exceptions.ImageNotFoundException;
import ru.crp.lvms.managers.image.ImageHandler;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;

import javax.annotation.Nonnull;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

public interface ControllerImageNew<E> extends FileHelper {
    ImageHandler imageHandler();

    E getEntity(HttpServletRequest request, Integer entityId, PortalUser currentUser);

    @RequestMapping(value = "/{id}/imageFile", method = RequestMethod.GET)
    default void getImageFile(HttpServletResponse response,
                              @PathVariable("id") Integer entityId,
                              @CurrentUser PortalUser currentUser) throws IOException {
        FileEntity file = imageHandler().getImageFile(entityId, currentUser.getUser());
        if (file == null) {
            throw new ImageNotFoundException();
        }
        writeFileToResponse(file, response, currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = "/{id}/imageFile", method = RequestMethod.POST)
    default E updateImageFile(HttpServletRequest request,
                              @PathVariable("id") Integer entityId,
                              @RequestParam("file") MultipartFile imageFile,
                              @CurrentUser PortalUser currentUser) {
        imageHandler().updateImageFile(entityId, imageFile, currentUser.getUser());
        return getEntity(request, entityId, currentUser);
    }


    @Nonnull
    @RequestMapping(value = "/{id}/imageFile", method = RequestMethod.DELETE)
    default E deleteImageFile(HttpServletRequest request,
                              @PathVariable("id") Integer entityId,
                              @CurrentUser PortalUser currentUser) {
        imageHandler().deleteImageFile(entityId, currentUser.getUser());
        return getEntity(request, entityId, currentUser);
    }
}
