package ru.crp.lvms.web.controllers.misc;

import ru.crp.lvms.exceptions.NotFoundException;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.UserEntity;

import javax.annotation.Nullable;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.nio.file.Path;

public interface FileHelper {
    FileManager getFileManager();

    static void setHeadersForDownloading(HttpServletResponse response, String fileName) {
        response.setHeader("Content-Type", "application/octet-stream");
        // see more http://stackoverflow.com/questions/93551/how-to-encode-the-filename-parameter-of-content-disposition-header-in-http
        try {
            String contentDisposition = "attachment; filename=\"" + fileName + "\"; filename*=UTF-8''" + URLEncoder.encode(fileName, "UTF-8").replace("+", "%20");
            response.setHeader("Content-Disposition", contentDisposition);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("UTF-8 is not supported");
        }
    }


    default void writeFileToResponse(@Nullable FileEntity file, HttpServletResponse response,
                                     UserEntity currentUser) throws IOException {
        checkFileExists(file);

        Path pathToFile = getFileManager().resolvePathToFile(file, currentUser);
        checkFileExists(pathToFile);

        setHeadersForDownloading(response, file.getName());
        Files.copy(pathToFile, response.getOutputStream());
    }

    static void writeFileToResponse(Path pathToFile, @Nullable String fileName, HttpServletResponse response) throws IOException {
        if (!Files.exists(pathToFile)) {
            throw new NotFoundException("Файл не найден");
        }
        if (fileName == null) {
            fileName = pathToFile.getFileName().toString();
        }

        setHeadersForDownloading(response, fileName);
        Files.copy(pathToFile, response.getOutputStream());
    }


    static void checkFileExists(@Nullable Object file) {
        if (file == null) {
            throw new NotFoundException("Файл не найден");
        }
    }
}
