package ru.crp.lvms.web.controllers.tasks;

import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.managers.misc.EventManager;
import ru.crp.lvms.managers.tasks.StopTaskManager;
import ru.crp.lvms.managers.tasks.TaskManager;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.model.entities.tasks.TaskIn;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.CrudControllerSuggest;
import ru.crp.lvms.web.dto.pagination.DataRequest;
import ru.crp.lvms.web.dto.pagination.PageingResponse;

import javax.annotation.Nonnull;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

import static ru.crp.lvms.utils.URLs.TASKS_URL;

@RestController
@RequestMapping(value = TASKS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class TaskController extends CrudControllerSuggest<TaskEntity, TaskEntity, TaskEntity> {
    @Autowired
    private TaskManager taskManager;

    @Autowired
    private EventManager eventManager;

    @Autowired
    private StopTaskManager stopTaskManager;

    @Nonnull
    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
    public TaskEntity createChainOfTasks(@Valid @RequestBody TaskIn taskIn,
                                         @CurrentUser PortalUser currentUser) {
        return taskManager.createChainOfTasks(taskIn, currentUser.getUser());
    }


    @NotNull
    @Override
    public PageingResponse<TaskEntity> page(HttpServletRequest request, @RequestBody DataRequest dataRequest, @CurrentUser PortalUser currentUser) {
        Map<String, Object> filters = dataRequest.getFilters();
        String ownedOnly = request.getParameter("ownedOnly");
        if (ownedOnly != null && Boolean.valueOf(ownedOnly)) {
            filters.put("ownerId", currentUser.getUser().getId());
        }
        PageingResponse<TaskEntity> page = super.page(request, dataRequest, currentUser);
        filters.remove("ownerId");
        return page;
    }


    @RequestMapping(value = "/running", method = RequestMethod.GET)
    public List<TaskEntity> getAllRunningTasks() {
        return taskManager.findRunning();
    }


    @NotNull
    @Override
    public TaskEntity update(HttpServletRequest request, @PathVariable("id") int id, @Valid @RequestBody TaskEntity updateVO,
                             @CurrentUser PortalUser user) {
        TaskEntity existedTask = getManager().findOne(id, user.getUser());
        checkEntityExists(id, existedTask);

        existedTask.setNodeId(updateVO.getNodeId());
        existedTask.setAdditionalStartupArguments(updateVO.getAdditionalStartupArguments());
        existedTask.setSolverName(updateVO.getSolverName());
        existedTask.setNumOfCores(updateVO.getNumOfCores());
        existedTask.setMemory(updateVO.getMemory());

        return getManager().updateAndRead(existedTask, user.getUser());
    }

    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{id}/stop", method = RequestMethod.POST)
    public Map<String, Object> stopTask(@PathVariable("id") int id, @CurrentUser PortalUser currentUser) throws Exception {
        return stopTaskManager.stopAndUpdateTask(id, currentUser.getUser());
    }

    @Override
    protected TaskManager getManager() {
        return taskManager;
    }

    @Override
    protected EventManager getEventsManager() {
        return eventManager;
    }
}
