package ru.crp.lvms.web.controllers.tasks;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.exceptions.NotFoundException;
import ru.crp.lvms.managers.UserManager;
import ru.crp.lvms.managers.tasks.StopTaskManager;
import ru.crp.lvms.managers.tasks.TaskManager;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.model.entities.tasks.TasksPositionSwapRequest;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.utils.AuthUtils;
import ru.crp.lvms.utils.URLs;

import javax.validation.Valid;

@RestController
@RequestMapping(value = URLs.TASKS_QUEUE_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class TaskQueueController {
    @Autowired
    private TaskManager taskManager;

    @Autowired
    private StopTaskManager stopTaskManager;

    @Autowired
    private UserManager userManager;


    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/swap", method = RequestMethod.POST)
    @ResponseStatus(HttpStatus.OK)
    public void swapTasks(@Valid @RequestBody TasksPositionSwapRequest swapRequest, @CurrentUser PortalUser currentUser) {
        UserEntity user = currentUser.getUser();
        TaskEntity firstTask = taskManager.findOne(swapRequest.getTask1Id(), user);
        TaskEntity secondTask = taskManager.findOne(swapRequest.getTask2Id(), user);
        if (firstTask == null || secondTask == null) {
            throw new NotFoundException("Задача не найдена");
        }
        taskManager.swapTasks(firstTask, secondTask, user);
    }


    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{id}/removeFromQueue", method = RequestMethod.POST)
    @ResponseStatus(HttpStatus.OK)
    public void removeFromQueue(@PathVariable("id") Integer taskId, @CurrentUser PortalUser portalUser) {
        UserEntity currentUser = portalUser.getUser();
        TaskEntity taskToRemove = taskManager.findOne(taskId, currentUser);
        if (taskToRemove == null) {
            throw new NotFoundException("Задача с номером " + taskId + " не найдена");
        }
        AuthUtils.ownerOrGreaterOrAdmin(userManager.findOne(taskToRemove.getOwnerId(), currentUser), currentUser);
        stopTaskManager.removeTaskFromQueue(taskToRemove, currentUser);
    }
}
