package ru.crp.lvms.web.dto.pagination;

import com.fasterxml.jackson.annotation.JsonIgnore;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static java.util.Collections.emptyList;
import static java.util.Collections.emptyMap;

public class DataRequest {
    private static final int DEFAULT_PAGE_SIZE = 1000;
    private Pagination pageable;
    private Map<String, Object> filters = emptyMap();
    private List<SortingOrder> sort = emptyList();

    @Nonnull
    public Pagination getPageable() {
        return pageable == null ? new Pagination(1, DEFAULT_PAGE_SIZE) : pageable;
    }

    public void setPageable(Pagination pageable) {
        this.pageable = pageable;
    }

    @Nonnull
    public Map<String, Object> getFilters() {
        return filters;
    }

    public void setFilters(Map<String, Object> filters) {
        this.filters = filters;
    }

    @Nonnull
    public List<SortingOrder> getSort() {
        return sort == null ? new ArrayList<>() : sort;
    }

    public void setSort(List<SortingOrder> sort) {
        this.sort = sort;
    }

    @JsonIgnore
    @Nullable
    private Sort getSpringSort() {
        if (sort.isEmpty()) {
            return null;
        }
        return new Sort(SortingOrder.toSpringSortabble(sort));
    }

    @JsonIgnore
    @Nonnull
    public Pageable getSpringPageable() {
        return pageable.toSpringPageable(getSpringSort());
    }
}
