ALTER TABLE tasks ADD COLUMN calculation_finished_with_exception BOOLEAN NOT NULL DEFAULT FALSE;
UPDATE tasks SET calculation_finished_with_exception = FALSE;

DROP VIEW tasks_join;
CREATE VIEW tasks_join AS (
  SELECT t.*,
    u.name AS owner,
    n.name AS node,

    CASE
    WHEN parent_type = 'effectiveCharacteristic' THEN ec.id
    WHEN parent_type = 'simulation' THEN vs.id
    WHEN parent_type = 'optimization' THEN o.id
    ELSE NULL
    END
           AS parent_id,

    CASE
    WHEN parent_type = 'effectiveCharacteristic' THEN ec.name
    WHEN parent_type = 'simulation' THEN vs.name
    WHEN parent_type = 'optimization' THEN o.name
    ELSE NULL
    END
           AS parent_name

  FROM tasks t
    INNER JOIN users u ON u.id = t.owner_id
    LEFT JOIN nodes n ON n.id = t.node_id
    LEFT JOIN effective_characteristics ec ON ec.id = t.ec_id
    LEFT JOIN virtual_simulations vs ON vs.id = t.sim_id
    LEFT JOIN optimizations o ON o.id = t.optimization_id
);

CREATE OR REPLACE RULE tasks_join_CREATE AS ON INSERT TO tasks_join DO INSTEAD (
  INSERT INTO tasks (type, status_id, queue_priority, solver_name, additional_startup_arguments, node_id,
                     num_cores, memory, progress, auto_select_node, cluster_id, owner_id, parent_type, ec_id, sim_id, optimization_id) VALUES
    (NEW.type, NEW.status_id, NEW.queue_priority, NEW.solver_name, NEW.additional_startup_arguments, NEW.node_id,
               NEW.num_cores, NEW.memory, NEW.progress, NEW.auto_select_node, NEW.cluster_id, NEW.owner_id, NEW.parent_type,
     CASE WHEN NEW.parent_type = 'effectiveCharacteristic' THEN NEW.parent_id ELSE NULL END,
     CASE WHEN NEW.parent_type = 'simulation' THEN NEW.parent_id ELSE NULL END,
     CASE WHEN NEW.parent_type = 'optimization' THEN NEW.parent_id ELSE NULL END)
  RETURNING
    tasks.*,
    NULL :: VARCHAR AS owner,
    NULL :: VARCHAR AS node,
    NULL :: INTEGER AS parent_id,
    NULL :: VARCHAR(255) AS parent_name
);

CREATE OR REPLACE RULE tasks_join_UPDATE AS ON UPDATE TO tasks_join DO INSTEAD (
  UPDATE tasks
  SET type = NEW.type,
    status_id = NEW.status_id,
    queue_priority = NEW.queue_priority,
    task_position = NEW.task_position,
    solver_name = NEW.solver_name,
    additional_startup_arguments = NEW.additional_startup_arguments,
    node_id = NEW.node_id,
    num_cores = NEW.num_cores,
    memory = NEW.memory,
    progress = NEW.progress,
    owner_id = NEW.owner_id,
    auto_select_node = NEW.auto_select_node,
    cluster_id = NEW.cluster_id
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tasks_join_DELETE AS ON DELETE TO tasks_join DO INSTEAD (
  DELETE FROM tasks WHERE id = OLD.id
);

