CREATE TABLE material_types (
  id   SERIAL,
  name VARCHAR UNIQUE,

  PRIMARY KEY (id)
);


CREATE TABLE materials (
  id            INTEGER DEFAULT nextval('tree_seq' :: REGCLASS),
  pid           INTEGER                                           NOT NULL,
  name          VARCHAR UNIQUE                                    NOT NULL,
  manufacture   VARCHAR,
  type_id       INTEGER DEFAULT 1                                 NOT NULL,
  source        VARCHAR,
  path          VARCHAR []                                        NOT NULL,
  path_names    VARCHAR                                           NOT NULL UNIQUE,
  description   TEXT,
  owner_id      INTEGER                                           NOT NULL,
  date_created  TIMESTAMP WITHOUT TIME ZONE DEFAULT now()         NOT NULL,
  date_modified TIMESTAMP WITHOUT TIME ZONE DEFAULT now()         NOT NULL,
  lock          BOOLEAN DEFAULT FALSE,

  PRIMARY KEY (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (pid) REFERENCES folders (id),
  FOREIGN KEY (type_id) REFERENCES material_types (id)
);


CREATE TRIGGER materials_path_creation
BEFORE INSERT ON materials
FOR EACH ROW
EXECUTE PROCEDURE create_path('Material');


CREATE TRIGGER materials_path_update
BEFORE UPDATE ON materials
FOR EACH ROW
WHEN (OLD.name IS DISTINCT FROM NEW.name OR OLD.pid IS DISTINCT FROM NEW.pid)
EXECUTE PROCEDURE update_path('Material');


CREATE TRIGGER materials_check_files
BEFORE DELETE ON materials
FOR EACH ROW
EXECUTE PROCEDURE check_files_parent_constraint();


CREATE TRIGGER materials_check_parameters
BEFORE DELETE ON materials
FOR EACH ROW
EXECUTE PROCEDURE check_parameters_parent_constraint();


CREATE TRIGGER materials_update
BEFORE UPDATE ON materials
FOR EACH ROW
WHEN (OLD.* IS DISTINCT FROM NEW.*)
EXECUTE PROCEDURE update_material_if_not_locked();


CREATE VIEW materials_join AS
  SELECT
    m.*,
    u.name  AS owner,
    mt.name AS type
  FROM materials m
    JOIN users u ON m.owner_id = u.id
    JOIN material_types mt ON m.type_id = mt.id;
