-- ModeFrontier base settings
DROP VIEW optimization_base_settings_join;
CREATE VIEW optimization_base_settings_join AS (
  SELECT
    obs.*,
    o.optimization_type_id AS optimization_type_id,
    obst.max_value         AS max_value,
    obst.min_value    AS min_value,
    obst.display_name      AS name

  FROM optimization_base_settings obs
    JOIN optimizations o ON obs.optimization_id = o.id
    JOIN optimization_base_setting_types obst ON obs.type_id = obst.id
);

CREATE RULE optimization_base_settings_join_CREATE AS ON INSERT TO optimization_base_settings_join DO INSTEAD (
  INSERT INTO optimization_base_settings (type_id, optimization_id, current_value)
  VALUES (NEW.type_id, NEW.optimization_id, NEW.current_value)
  RETURNING
    optimization_base_settings.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: INTEGER AS max_value,
    NULL :: INTEGER AS min_value,
    NULL :: VARCHAR(255) AS name
);

CREATE RULE optimization_base_settings_join_UPDATE AS ON UPDATE TO optimization_base_settings_join DO INSTEAD (
  UPDATE optimization_base_settings
  SET current_value = NEW.current_value
  WHERE id = NEW.id);

CREATE RULE optimization_base_settings_join_DELETE AS ON DELETE TO optimization_base_settings_join DO INSTEAD (
  DELETE FROM optimization_base_settings
  WHERE id = OLD.id
);

-- ModeFrontier CAO varying params
CREATE VIEW optimization_varying_params_join AS (
  SELECT
    ocvp.*,
    o.optimization_type_id AS optimization_type_id
  FROM optimization_varying_params ocvp
    JOIN optimizations o ON ocvp.optimization_id = o.id
);

CREATE RULE optimization_varying_params_join_CREATE AS ON INSERT TO optimization_varying_params_join DO INSTEAD (
  INSERT INTO optimization_varying_params (optimization_id, name, lower_bound, upper_bound, sampling, is_used_in_script)
  VALUES (NEW.optimization_id, NEW.name, NEW.lower_bound, NEW.upper_bound, NEW.sampling, FALSE)
  RETURNING
    optimization_varying_params.*,
    NULL :: INTEGER AS optimization_type_id
);


CREATE RULE optimization_varying_params_join_UPDATE AS ON UPDATE TO optimization_varying_params_join DO INSTEAD (
  UPDATE optimization_varying_params
  SET name            = NEW.name,
    lower_bound       = NEW.lower_bound,
    upper_bound       = NEW.upper_bound,
    sampling          = NEW.sampling,
    is_used_in_script = NEW.is_used_in_script
  WHERE id = NEW.id);

CREATE RULE optimization_varying_params_join_DELETE AS ON DELETE TO optimization_varying_params_join DO INSTEAD (
  DELETE FROM optimization_varying_params
  WHERE id = OLD.id
);

-- ModeFrontier CAD varying params
CREATE VIEW optimization_cad_varying_params_join AS (
  SELECT
    ocvp.*,
    o.optimization_type_id AS optimization_type_id
  FROM optimization_cad_varying_params ocvp
    JOIN optimizations o ON ocvp.optimization_id = o.id
);

CREATE RULE optimization_cad_varying_params_join_CREATE AS ON INSERT TO optimization_cad_varying_params_join DO INSTEAD (
  INSERT INTO optimization_cad_varying_params (optimization_id, name, lower_bound, upper_bound, sampling)
  VALUES (NEW.optimization_id, NEW.name, NEW.lower_bound, NEW.upper_bound, NEW.sampling)
  RETURNING
    optimization_cad_varying_params.*,
    NULL :: INTEGER AS optimization_type_id
);


CREATE RULE optimization_cad_varying_params_join_UPDATE AS ON UPDATE TO optimization_cad_varying_params_join DO INSTEAD (
  UPDATE optimization_cad_varying_params
  SET name      = NEW.name,
    lower_bound = NEW.lower_bound,
    upper_bound = NEW.upper_bound,
    sampling    = NEW.sampling
  WHERE id = NEW.id);

CREATE RULE optimization_cad_varying_params_join_DELETE AS ON DELETE TO optimization_cad_varying_params_join DO INSTEAD (
  DELETE FROM optimization_cad_varying_params
  WHERE id = OLD.id
);

-- ModeFrontier constraints
DROP VIEW optimization_constrains_join;
CREATE VIEW optimization_constrains_join AS (
  SELECT
    oc.*,
    o.optimization_type_id AS optimization_type_id,
    pd.name                AS func_name
  FROM optimization_constrains oc
    JOIN optimizations o ON oc.optimization_id = o.id
    LEFT JOIN parameters_dictionary pd ON pd.id = oc.func_id
);

CREATE RULE optimization_constrains_join_CREATE AS ON INSERT TO optimization_constrains_join DO INSTEAD (
  INSERT INTO optimization_constrains (optimization_id, name, func_id, val)
  VALUES (NEW.optimization_id, NEW.name, NEW.func_id, NEW.val)
  RETURNING
    optimization_constrains.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS func_name
);

CREATE RULE optimization_constrains_join_UPDATE AS ON UPDATE TO optimization_constrains_join DO INSTEAD (
  UPDATE optimization_constrains
  SET name  = NEW.name,
    func_id = NEW.func_id,
    val     = NEW.val
  WHERE id = NEW.id);

CREATE RULE optimization_constrains_join_DELETE AS ON DELETE TO optimization_constrains_join DO INSTEAD (
  DELETE FROM optimization_constrains
  WHERE id = OLD.id
);

-- ModeFrontier target functions
DROP VIEW optimization_target_funcs_join;
CREATE VIEW optimization_target_funcs_join AS (
  SELECT
    otf.*,
    o.optimization_type_id AS optimization_type_id,
    pd.name                AS func_name
  FROM optimization_target_funcs otf
    JOIN optimizations o ON otf.optimization_id = o.id
    LEFT JOIN parameters_dictionary pd ON pd.id = otf.func_id
);

CREATE RULE optimization_target_funcs_join_CREATE AS ON INSERT TO optimization_target_funcs_join DO INSTEAD (
  INSERT INTO optimization_target_funcs (optimization_id, name, func_id)
  VALUES (NEW.optimization_id, NEW.name, NEW.func_id)
  RETURNING
    optimization_target_funcs.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS func_name
);

CREATE RULE optimization_target_funcs_join_UPDATE AS ON UPDATE TO optimization_target_funcs_join DO INSTEAD (
  UPDATE optimization_target_funcs
  SET name  = NEW.name,
    func_id = NEW.func_id
  WHERE id = NEW.id);

CREATE RULE optimization_target_funcs_join_DELETE AS ON DELETE TO optimization_target_funcs_join DO INSTEAD (
  DELETE FROM optimization_target_funcs
  WHERE id = OLD.id
);

-- Tosca base settings
DROP VIEW toska_optimization_base_settings_join;

CREATE VIEW toska_optimization_base_settings_join AS (
  SELECT
    obs.*,
    o.optimization_type_id AS optimization_type_id,
    obst.max_value         AS max_value,
    obst.min_value         AS min_value,
    obst.display_name      AS name

  FROM toska_optimization_base_settings obs
    JOIN optimizations o ON obs.optimization_id = o.id
    JOIN toska_optimization_base_setting_types obst ON obs.type_id = obst.id
);

CREATE RULE toska_optimization_base_settings_join_CREATE AS ON INSERT TO toska_optimization_base_settings_join DO INSTEAD (
  INSERT INTO toska_optimization_base_settings (type_id, optimization_id, current_value)
  VALUES (NEW.type_id, NEW.optimization_id, NEW.current_value)
  RETURNING
    toska_optimization_base_settings.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: INTEGER AS max_value,
    NULL :: INTEGER AS min_value,
    NULL :: VARCHAR(255) AS name
);

CREATE RULE toska_optimization_base_settings_join_UPDATE AS ON UPDATE TO toska_optimization_base_settings_join DO INSTEAD (
  UPDATE toska_optimization_base_settings
  SET current_value = NEW.current_value
  WHERE id = NEW.id);

CREATE RULE toska_optimization_base_settings_join_DELETE AS ON DELETE TO toska_optimization_base_settings_join DO INSTEAD (
  DELETE FROM toska_optimization_base_settings
  WHERE id = OLD.id
);

-- Tosca constraints
DROP VIEW toska_constraints_join;

CREATE VIEW toska_constraints_join AS
  SELECT
    tc.*,
    o.optimization_type_id AS optimization_type_id,
    ttfp.name              AS param_name,
    ttfp.dresp_type,
    ttfp.node_or_element_id
  FROM toska_optimization_constraints tc
    JOIN optimizations o ON tc.optimization_id = o.id
    JOIN toska_target_function_parameters ttfp
      ON ttfp.id = tc.parameter_id;

CREATE RULE toska_constraints_join_CREATE AS ON INSERT TO toska_constraints_join DO INSTEAD (
  INSERT INTO toska_optimization_constraints (name, parameter_id, optimization_id, type_id, el_group, nd_group, scalar, value, isAbsolute)
  VALUES (NEW.name, NEW.parameter_id, NEW.optimization_id, NEW.type_id, NEW.el_group, NEW.nd_group, NEW.scalar, NEW.value, NEW.isAbsolute)
  RETURNING
    toska_optimization_constraints.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS param_name,
    NULL :: VARCHAR(255) AS dresp_type,
    NULL :: INTEGER AS node_or_element_id
);

CREATE RULE toska_constraints_join_UPDATE AS ON UPDATE TO toska_constraints_join DO INSTEAD (
  UPDATE toska_optimization_constraints
  SET name  = NEW.name, parameter_id = NEW.parameter_id, optimization_id = NEW.optimization_id,
    type_id = NEW.type_id, el_group = NEW.el_group, nd_group = NEW.nd_group, scalar = NEW.scalar,
    value   = NEW.value, isAbsolute = NEW.isAbsolute
  WHERE id = NEW.id);

CREATE RULE toska_constraints_join_DELETE AS ON DELETE TO toska_constraints_join DO INSTEAD (
  DELETE FROM toska_constraints_join
  WHERE id = OLD.id
);

-- Tosca target functions
DROP VIEW toska_target_functions_join;
CREATE VIEW toska_target_functions_join AS
  SELECT
    ttf.*,
    o.optimization_type_id AS optimization_type_id,
    ttfp.name              AS param_name,
    ttfp.dresp_type,
    ttfp.node_or_element_id
  FROM toska_optimization_target_functions ttf
    JOIN optimizations o ON ttf.optimization_id = o.id
    JOIN toska_target_function_parameters ttfp
      ON ttfp.id = ttf.parameter_id;

CREATE RULE toska_target_functions_join_CREATE AS ON INSERT TO toska_target_functions_join DO INSTEAD (
  INSERT INTO toska_optimization_target_functions (name, parameter_id, optimization_id, type_id, el_group, nd_group, scalar)
  VALUES (NEW.name, NEW.parameter_id, NEW.optimization_id, NEW.type_id, NEW.el_group, NEW.nd_group, NEW.scalar)
  RETURNING
    toska_optimization_target_functions.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS param_name,
    NULL :: VARCHAR(255) AS dresp_type,
    NULL :: INTEGER AS node_or_element_id
);

CREATE RULE toska_target_functions_join_UPDATE AS ON UPDATE TO toska_target_functions_join DO INSTEAD (
  UPDATE toska_optimization_target_functions
  SET name  = NEW.name, parameter_id = NEW.parameter_id, optimization_id = NEW.optimization_id,
    type_id = NEW.type_id, el_group = NEW.el_group, nd_group = NEW.nd_group, scalar = NEW.scalar
  WHERE id = NEW.id);

CREATE RULE toska_target_functions_join_DELETE AS ON DELETE TO toska_target_functions_join DO INSTEAD (
  DELETE FROM toska_optimization_target_functions
  WHERE id = OLD.id
);

-- Tosca technological constraints
DROP VIEW toska_technological_constraints_join;
CREATE VIEW toska_technological_constraints_join AS
  SELECT
    totc.*,
    o.optimization_type_id AS optimization_type_id,
    ttc.name               AS constraint_name,
    ttc.comand_name
  FROM toska_optimization_technological_constraints totc
    JOIN optimizations o ON totc.optimization_id = o.id
    JOIN toska_technological_constraints ttc
      ON ttc.id = totc.constraint_id;

CREATE RULE toska_technological_constraints_join_CREATE AS ON INSERT TO toska_technological_constraints_join DO INSTEAD (
  INSERT INTO toska_optimization_technological_constraints (name, constraint_id, optimization_id, set_id)
  VALUES (NEW.name, NEW.constraint_id, NEW.optimization_id, NEW.set_id)
  RETURNING
    toska_optimization_technological_constraints.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS constraint_name,
    NULL :: VARCHAR(255) AS comand_name
);

CREATE RULE toska_technological_constraints_join_UPDATE AS ON UPDATE TO toska_technological_constraints_join DO INSTEAD (
  UPDATE toska_optimization_technological_constraints
  SET name = NEW.name, constraint_id = NEW.constraint_id, set_id = NEW.set_id
  WHERE id = NEW.id);

CREATE RULE toska_technological_constraints_join_DELETE AS ON DELETE TO toska_technological_constraints_join DO INSTEAD (
  DELETE FROM toska_optimization_technological_constraints
  WHERE id = OLD.id
);

-- Tosca sets
DROP VIEW toska_sets_join;
CREATE VIEW toska_sets_join AS
  SELECT
    ts.*,
    o.optimization_type_id AS optimization_type_id,
    st.name                AS set_type_name
  FROM toska_sets ts
    JOIN optimizations o ON ts.optimization_id = o.id
    JOIN set_type st
      ON st.id = ts.set_type_id;

CREATE RULE toska_sets_join_CREATE AS ON INSERT TO toska_sets_join DO INSTEAD (
  INSERT INTO toska_sets (name, is_optimization_space, optimization_id, set_type_id, is_selected)
  VALUES (NEW.name, NEW.is_optimization_space, NEW.optimization_id, NEW.set_type_id, NEW.is_selected)
  RETURNING
    toska_sets.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS set_type_name
);

CREATE RULE toska_sets_join_UPDATE AS ON UPDATE TO toska_sets_join DO INSTEAD (
  UPDATE toska_sets
  SET name = NEW.name, is_optimization_space = NEW.is_optimization_space, set_type_id = NEW.set_type_id, is_selected = NEW.is_selected
  WHERE id = NEW.id);

CREATE RULE toska_sets_join_DELETE AS ON DELETE TO toska_sets_join DO INSTEAD (
  DELETE FROM toska_sets
  WHERE id = OLD.id
);

-- Optistruct base settings
DROP VIEW optistruct_optimization_base_settings_join;

CREATE VIEW optistruct_optimization_base_settings_join AS (
  SELECT
    obs.*,
    o.optimization_type_id AS optimization_type_id,
    obst.max_value         AS max_value,
    obst.min_value         AS min_value,
    obst.display_name      AS name

  FROM optistruct_optimization_base_settings obs
    JOIN optimizations o ON obs.optimization_id = o.id
    JOIN optistruct_optimization_base_setting_types obst ON obs.type_id = obst.id
);

CREATE RULE optistruct_optimization_base_settings_join_CREATE AS ON INSERT TO optistruct_optimization_base_settings_join DO INSTEAD (
  INSERT INTO optistruct_optimization_base_settings (type_id, optimization_id, current_value)
  VALUES (NEW.type_id, NEW.optimization_id, NEW.current_value)
  RETURNING
    optistruct_optimization_base_settings.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: INTEGER AS max_value,
    NULL :: INTEGER AS min_value,
    NULL :: VARCHAR(255) AS name
);

CREATE RULE optistruct_optimization_base_settings_join_UPDATE AS ON UPDATE TO optistruct_optimization_base_settings_join DO INSTEAD (
  UPDATE optistruct_optimization_base_settings
  SET current_value = NEW.current_value
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_base_settings_join_DELETE AS ON DELETE TO optistruct_optimization_base_settings_join DO INSTEAD (
  DELETE FROM optistruct_optimization_base_settings
  WHERE id = OLD.id
);

-- Optistruct constraints
DROP VIEW optistruct_optimization_constraint_join;
CREATE VIEW optistruct_optimization_constraint_join AS (
  SELECT
    base.id,
    base.optimization_id,
    base.tf_id,
    base.ptype_id,
    base.ptype_value,
    base.atta_id,
    base.atta_value,
    base.attb_id,
    base.attb_value,
    base.atti_id,
    base.atti_value,
    base.lower_bound,
    base.upper_bound,
    base.name,
    o.optimization_type_id   AS optimization_type_id,
    optistruct_tf.name       AS tf_name,
    optistruct_tf.response   AS tf_response,
    optistruct_tf_atta.name  AS atta_name,
    optistruct_tf_attb.name  AS attb_name,
    optistruct_tf_atti.name  AS atti_name,
    optistruct_tf_ptype.name AS ptype_name

  FROM optistruct_optimization_tf_and_constraint base
    JOIN optimizations o ON base.optimization_id = o.id
    LEFT JOIN optistruct_tf ON base.tf_id = optistruct_tf.id
    LEFT JOIN optistruct_tf_atta ON base.atta_id = optistruct_tf_atta.id
    LEFT JOIN optistruct_tf_attb ON base.attb_id = optistruct_tf_attb.id
    LEFT JOIN optistruct_tf_atti ON base.atti_id = optistruct_tf_atti.id
    LEFT JOIN optistruct_tf_ptype ON base.ptype_id = optistruct_tf_ptype.id

  WHERE base.is_constraint = TRUE
);

CREATE RULE optistruct_optimization_constraint_join_CREATE
AS ON INSERT TO optistruct_optimization_constraint_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tf_and_constraint (optimization_id, tf_id, ptype_id, ptype_value, atta_id, atta_value, attb_id, attb_value, atti_id, atti_value, is_constraint, lower_bound, upper_bound, name)
  VALUES
    (NEW.optimization_id, NEW.tf_id, NEW.ptype_id, NEW.ptype_value, NEW.atta_id, NEW.atta_value, NEW.attb_id, NEW.attb_value, NEW.atti_id, NEW.atti_value, TRUE, NEW.lower_bound,
     NEW.upper_bound, NEW.name)
  RETURNING
    optistruct_optimization_tf_and_constraint.id,
    optistruct_optimization_tf_and_constraint.optimization_id,
    optistruct_optimization_tf_and_constraint.tf_id,
    optistruct_optimization_tf_and_constraint.ptype_id,
    optistruct_optimization_tf_and_constraint.ptype_value,
    optistruct_optimization_tf_and_constraint.atta_id,
    optistruct_optimization_tf_and_constraint.atta_value,
    optistruct_optimization_tf_and_constraint.attb_id,
    optistruct_optimization_tf_and_constraint.attb_value,
    optistruct_optimization_tf_and_constraint.atti_id,
    optistruct_optimization_tf_and_constraint.atti_value,
    optistruct_optimization_tf_and_constraint.lower_bound,
    optistruct_optimization_tf_and_constraint.upper_bound,
    NULL :: VARCHAR(255) AS name,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS tf_response,
    NULL :: VARCHAR(255) AS tf_name,
    NULL :: VARCHAR(255) AS atta_name,
    NULL :: VARCHAR(255) AS attb_name,
    NULL :: VARCHAR(255) AS atti_name,
    NULL :: VARCHAR(255) AS ptype_name
);

CREATE RULE optistruct_optimization_constraint_join_UPDATE
AS ON UPDATE TO optistruct_optimization_constraint_join DO INSTEAD (
  UPDATE optistruct_optimization_tf_and_constraint
  SET
    optimization_id = NEW.optimization_id,
    tf_id           = NEW.tf_id,
    ptype_id        = NEW.ptype_id,
    ptype_value     = NEW.ptype_value,
    atta_id         = NEW.atta_id,
    atta_value      = NEW.atta_value,
    attb_id         = NEW.attb_id,
    attb_value      = NEW.attb_value,
    atti_id         = NEW.atti_id,
    atti_value      = NEW.atti_value,
    lower_bound     = NEW.lower_bound,
    upper_bound     = NEW.upper_bound,
    name            = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_constraint_join_DELETE
AS ON DELETE TO optistruct_optimization_constraint_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tf_and_constraint
  WHERE id = OLD.id
);

-- Optistruct tech constraints
DROP VIEW optistruct_optimization_tech_constraints_join;
CREATE VIEW optistruct_optimization_tech_constraints_join AS (
  SELECT
    base.*,
    o.optimization_type_id           AS optimization_type_id,
    optistruct_tech_constraints.placeholder AS tech_constraint_placeholder,
    optistruct_tech_constraints.name AS tech_constraint_name

  FROM optistruct_optimization_tech_constraints base
    JOIN optimizations o ON base.optimization_id = o.id
    LEFT JOIN optistruct_tech_constraints ON base.tech_constr_id = optistruct_tech_constraints.id
);

CREATE RULE optistruct_optimization_tech_constraints_join_CREATE
AS ON INSERT TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tech_constraints (optimization_id, tech_constr_id, value, name)
  VALUES (NEW.optimization_id, NEW.tech_constr_id, NEW.value, NEW.name)
  RETURNING
    optistruct_optimization_tech_constraints.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS tech_constraint_name,
    NULL :: VARCHAR(255) AS tech_constraint_placeholder
);

CREATE RULE optistruct_optimization_tech_constraints_join_UPDATE
AS ON UPDATE TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  UPDATE optistruct_optimization_tech_constraints
  SET tech_constr_id = NEW.tech_constr_id, value = NEW.value, name = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_tech_constraints_join_DELETE
AS ON DELETE TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tech_constraints
  WHERE id = OLD.id
);

-- Optistruct sets
DROP VIEW optistruct_sets_join;
CREATE VIEW optistruct_sets_join AS
  SELECT
    ts.*,
    o.optimization_type_id AS optimization_type_id,
    st.name                AS component_type_name
  FROM optistruct_sets ts
    JOIN optimizations o ON ts.optimization_id = o.id
    JOIN optistruct_component_type st
      ON st.id = ts.component_type_id;

CREATE RULE optistruct_sets_join_CREATE AS ON INSERT TO optistruct_sets_join DO INSTEAD (
  INSERT INTO optistruct_sets (name, is_optimization_space, optimization_id, component_type_id, is_selected)
  VALUES (NEW.name, NEW.is_optimization_space, NEW.optimization_id, NEW.component_type_id, NEW.is_selected)
  RETURNING
    optistruct_sets.*,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS component_type_name
);

CREATE RULE optistruct_sets_join_UPDATE AS ON UPDATE TO optistruct_sets_join DO INSTEAD (
  UPDATE optistruct_sets
  SET name = NEW.name, is_optimization_space = NEW.is_optimization_space, component_type_id = NEW.component_type_id, is_selected = NEW.is_selected
  WHERE id = NEW.id);

CREATE RULE optistruct_sets_join_DELETE AS ON DELETE TO optistruct_sets_join DO INSTEAD (
  DELETE FROM optistruct_sets
  WHERE id = OLD.id
);

-- We don't need to update Optistruct target functions because 'optimization_type_id' is already present in it

