CREATE TABLE composite_materials (
  id                  INTEGER DEFAULT nextval('tree_seq' :: REGCLASS),
  pid                 INTEGER                                                    NOT NULL,
  name                VARCHAR UNIQUE                                             NOT NULL,
  manufacture         VARCHAR,
  source              VARCHAR,
  path                VARCHAR []                                                 NOT NULL,
  path_names          VARCHAR                                                    NOT NULL UNIQUE,
  description         TEXT,
  owner_id            INTEGER                                                    NOT NULL,
  date_created        TIMESTAMP WITHOUT TIME ZONE DEFAULT now()                  NOT NULL,
  date_modified       TIMESTAMP WITHOUT TIME ZONE DEFAULT now()                  NOT NULL,
  lock                BOOLEAN DEFAULT FALSE,
  technology_id       INTEGER,
  equipment_id        INTEGER,
  matrix_id           INTEGER,
  fiber_id            INTEGER,
  fiber_concentration NUMERIC(20, 3),
  symmetrical_layers  BOOLEAN DEFAULT FALSE                                      NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (pid) REFERENCES folders (id),
  FOREIGN KEY (technology_id) REFERENCES technologies (id),
  FOREIGN KEY (equipment_id) REFERENCES equipments (id),
  FOREIGN KEY (matrix_id) REFERENCES materials (id),
  FOREIGN KEY (fiber_id) REFERENCES materials (id)
);


CREATE TRIGGER composite_materials_path_creation
BEFORE INSERT ON composite_materials
FOR EACH ROW
EXECUTE PROCEDURE create_path('CompositeMaterial');

CREATE TRIGGER composite_materials_check_files
BEFORE DELETE ON composite_materials
FOR EACH ROW
EXECUTE PROCEDURE check_files_parent_constraint();


CREATE TRIGGER composite_materials_check_parameters
BEFORE DELETE ON composite_materials
FOR EACH ROW
EXECUTE PROCEDURE check_parameters_parent_constraint();


CREATE TRIGGER composite_materials_update
BEFORE UPDATE ON composite_materials
FOR EACH ROW
WHEN (OLD.* IS DISTINCT FROM NEW.*)
EXECUTE PROCEDURE update_material_if_not_locked();


CREATE TRIGGER composite_materials_path_update
BEFORE UPDATE ON composite_materials
FOR EACH ROW
WHEN (OLD.name IS DISTINCT FROM NEW.name OR OLD.pid IS DISTINCT FROM NEW.pid)
EXECUTE PROCEDURE update_path('CompositeMaterial');


CREATE VIEW composite_materials_join AS
  SELECT
    cm.*,
    u.name    AS owner,
    tech.name AS technology_name,
    e.name    AS equipment_name,
    matr.name AS matrix_name,
    fib.name  AS fiber_name
  FROM composite_materials cm
    JOIN users u ON cm.owner_id = u.id
    LEFT JOIN technologies tech ON cm.technology_id = tech.id
    LEFT JOIN equipments e ON cm.equipment_id = e.id
    LEFT JOIN materials matr ON cm.matrix_id = matr.id
    LEFT JOIN materials fib ON cm.fiber_id = fib.id;


CREATE TABLE layers (
  id                    SERIAL,
  position              INTEGER        NOT NULL,
  thickness             NUMERIC(20, 3) NOT NULL,
  angle                 NUMERIC(20, 3) NOT NULL,
  composite_material_id INTEGER        NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (composite_material_id) REFERENCES composite_materials (id),
  CONSTRAINT unique_order UNIQUE (position, composite_material_id)
);
