CREATE TABLE optistruct_lnset_type (
  id SERIAL,
  name VARCHAR(255),

  PRIMARY KEY(id)
);

INSERT INTO optistruct_lnset_type(id, name) VALUES (1, 'nodes'), (2, 'elems');

CREATE TABLE optistruct_lnsets (
  id SERIAL,
  name VARCHAR(255),
  optimization_id INTEGER NOT NULL,
  lnset_type_id INTEGER NOT NULL,
  is_selected BOOLEAN NOT NULL DEFAULT FALSE,

  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(lnset_type_id) REFERENCES optistruct_lnset_type(id) ON DELETE CASCADE,
  PRIMARY KEY(id)
);

CREATE VIEW optistruct_lnsets_join AS
  SELECT olns.*, o.optimization_type_id,
    olnst.name as lnset_type_name
  FROM optistruct_lnsets olns
    JOIN optimizations o ON olns.optimization_id = o.id
    JOIN optistruct_lnset_type olnst ON olnst.id = olns.lnset_type_id;

CREATE RULE optistruct_lnsets_join_CREATE AS ON INSERT TO optistruct_lnsets_join DO INSTEAD (
  INSERT INTO optistruct_lnsets (name, optimization_id, lnset_type_id, is_selected)
  VALUES (NEW.name, NEW.optimization_id, NEW.lnset_type_id, NEW.is_selected)
  RETURNING
    optistruct_lnsets.*,
    NULL::integer AS optimization_type_id,
    NULL :: VARCHAR(255) AS lnset_type_name
);

CREATE RULE optistruct_lnsets_join_UPDATE AS ON UPDATE TO optistruct_lnsets_join DO INSTEAD (
  UPDATE optistruct_lnsets
  SET name = NEW.name, lnset_type_id = NEW.lnset_type_id, is_selected = NEW.is_selected
  WHERE id = NEW.id);

CREATE RULE optistruct_lnsets_join_DELETE AS ON DELETE TO optistruct_lnsets_join DO INSTEAD (
  DELETE FROM optistruct_lnsets WHERE id = OLD.id
);

DROP VIEW optistruct_optimization_constraint_join;
DROP VIEW optistruct_optimization_tf_join;

ALTER TABLE optistruct_optimization_tf_and_constraint ADD COLUMN lnset_id INTEGER REFERENCES optistruct_lnsets ON DELETE SET NULL;

-- Optistruct tf
CREATE VIEW optistruct_optimization_tf_join AS (
  SELECT base.id,
    base.optimization_id,
    base.tf_id,

    base.atta_id,
    base.atta_value,

    base.atti_id,
    base.atti_value,

    base.loadstep_id,

    base.lnset_id,

    base.optimization_type_id,
    base.name,

    optistruct_tf.name as tf_name,
    optistruct_tf.response as tf_response,
    optistruct_tf_atta.name as atta_name,
    optistruct_tf_atti.name as atti_name,
    loadstep.name as loadstep_name,
    lnset.name as lnset_name,
    optistruct_tf_optimization_type.name as optimization_type_name

  FROM optistruct_optimization_tf_and_constraint base
    LEFT JOIN optistruct_tf ON base.tf_id = optistruct_tf.id
    LEFT JOIN optistruct_tf_atta ON base.atta_id = optistruct_tf_atta.id
    LEFT JOIN optistruct_tf_atti ON base.atti_id = optistruct_tf_atti.id
    LEFT JOIN optistruct_optimization_load_steps loadstep ON base.loadstep_id = loadstep.id
    LEFT JOIN optistruct_lnsets lnset ON base.lnset_id = lnset.id
    LEFT JOIN optistruct_tf_optimization_type ON base.optimization_type_id = optistruct_tf_optimization_type.id

  WHERE base.is_constraint = FALSE
);

CREATE RULE optistruct_optimization_tf_join_CREATE
AS ON INSERT TO optistruct_optimization_tf_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tf_and_constraint (
    optimization_id,
    tf_id,
    atta_id,
    atta_value,
    atti_id,
    atti_value,
    loadstep_id,
    lnset_id,
    optimization_type_id,
    is_constraint,
    name
  )
  VALUES (
    NEW.optimization_id,
    NEW.tf_id,
    NEW.atta_id,
    NEW.atta_value,
    NEW.atti_id,
    NEW.atti_value,
    NEW.loadstep_id,
    NEW.lnset_id,
    NEW.optimization_type_id,
    FALSE,
    NEW.name
  )
  RETURNING
    optistruct_optimization_tf_and_constraint.id,
    optistruct_optimization_tf_and_constraint.optimization_id,
    optistruct_optimization_tf_and_constraint.tf_id,
    optistruct_optimization_tf_and_constraint.atta_id,
    optistruct_optimization_tf_and_constraint.atta_value,
    optistruct_optimization_tf_and_constraint.atti_id,
    optistruct_optimization_tf_and_constraint.atti_value,
    optistruct_optimization_tf_and_constraint.loadstep_id,
    optistruct_optimization_tf_and_constraint.lnset_id,
    optistruct_optimization_tf_and_constraint.optimization_type_id,
    NULL :: VARCHAR(255) AS tf_response,
    NULL :: VARCHAR(255) AS tf_name,
    NULL :: VARCHAR(255) AS name,
    NULL :: VARCHAR(255) AS atta_name,
    NULL :: VARCHAR(255) AS atti_name,
    NULL :: VARCHAR(255) AS loadstep_name,
    NULL :: VARCHAR(255) AS lnset_name,
    NULL :: VARCHAR(255) AS optimization_type_name
);

CREATE RULE optistruct_optimization_tf_join_UPDATE
AS ON UPDATE TO optistruct_optimization_tf_join DO INSTEAD (
  UPDATE optistruct_optimization_tf_and_constraint
  SET
    optimization_id = NEW.optimization_id,
    tf_id = NEW.tf_id,
    atta_id = NEW.atta_id,
    atta_value = NEW.atta_value,
    atti_id = NEW.atti_id,
    loadstep_id = NEW.loadstep_id,
    lnset_id = NEW.lnset_id,
    atti_value = NEW.atti_value,
    optimization_type_id = NEW.optimization_type_id,
    name = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_tf_join_DELETE
AS ON DELETE TO optistruct_optimization_tf_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tf_and_constraint WHERE id = OLD.id
);

-- Optistruct constraints
CREATE VIEW optistruct_optimization_constraint_join AS (
  SELECT
    base.id,
    base.optimization_id,
    base.tf_id,
    base.atta_id,
    base.atta_value,
    base.atti_id,
    base.atti_value,
    base.loadstep_id,
    base.lnset_id,
    base.lower_bound,
    base.upper_bound,
    base.name,
    o.optimization_type_id   AS optimization_type_id,
    optistruct_tf.name       AS tf_name,
    optistruct_tf.response   AS tf_response,
    optistruct_tf_atta.name  AS atta_name,
    optistruct_tf_atti.name  AS atti_name,
    loadstep.name AS loadstep_name,
    lnset.name AS lnset_name

  FROM optistruct_optimization_tf_and_constraint base
    JOIN optimizations o ON base.optimization_id = o.id
    LEFT JOIN optistruct_tf ON base.tf_id = optistruct_tf.id
    LEFT JOIN optistruct_tf_atta ON base.atta_id = optistruct_tf_atta.id
    LEFT JOIN optistruct_optimization_load_steps loadstep ON base.loadstep_id = loadstep.id
    LEFT JOIN optistruct_lnsets lnset ON base.lnset_id = lnset.id
    LEFT JOIN optistruct_tf_atti ON base.atti_id = optistruct_tf_atti.id

  WHERE base.is_constraint = TRUE
);

CREATE RULE optistruct_optimization_constraint_join_CREATE
AS ON INSERT TO optistruct_optimization_constraint_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tf_and_constraint (
    optimization_id,
    tf_id,
    atta_id,
    atta_value,
    atti_id,
    atti_value,
    loadstep_id,
    lnset_id,
    is_constraint,
    lower_bound,
    upper_bound,
    name)
  VALUES(
    NEW.optimization_id,
    NEW.tf_id,
    NEW.atta_id,
    NEW.atta_value,
    NEW.atti_id,
    NEW.atti_value,
    NEW.loadstep_id,
    NEW.lnset_id,
    TRUE,
    NEW.lower_bound,
    NEW.upper_bound,
    NEW.name)
  RETURNING
    optistruct_optimization_tf_and_constraint.id,
    optistruct_optimization_tf_and_constraint.optimization_id,
    optistruct_optimization_tf_and_constraint.tf_id,
    optistruct_optimization_tf_and_constraint.atta_id,
    optistruct_optimization_tf_and_constraint.atta_value,
    optistruct_optimization_tf_and_constraint.atti_id,
    optistruct_optimization_tf_and_constraint.atti_value,
    optistruct_optimization_tf_and_constraint.loadstep_id,
    optistruct_optimization_tf_and_constraint.lnset_id,
    optistruct_optimization_tf_and_constraint.lower_bound,
    optistruct_optimization_tf_and_constraint.upper_bound,
    NULL :: VARCHAR(255) AS name,
    NULL :: INTEGER AS optimization_type_id,
    NULL :: VARCHAR(255) AS tf_response,
    NULL :: VARCHAR(255) AS tf_name,
    NULL :: VARCHAR(255) AS atta_name,
    NULL :: VARCHAR(255) AS atti_name,
    NULL :: VARCHAR(255) AS loadstep_name,
    NULL :: VARCHAR(255) AS lnset_name
);

CREATE RULE optistruct_optimization_constraint_join_UPDATE
AS ON UPDATE TO optistruct_optimization_constraint_join DO INSTEAD (
  UPDATE optistruct_optimization_tf_and_constraint
  SET
    optimization_id = NEW.optimization_id,
    tf_id           = NEW.tf_id,
    atta_id         = NEW.atta_id,
    atta_value      = NEW.atta_value,
    atti_id         = NEW.atti_id,
    atti_value      = NEW.atti_value,
    loadstep_id     = NEW.loadstep_id,
    lnset_id        = NEW.lnset_id,
    lower_bound     = NEW.lower_bound,
    upper_bound     = NEW.upper_bound,
    name            = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_constraint_join_DELETE
AS ON DELETE TO optistruct_optimization_constraint_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tf_and_constraint
  WHERE id = OLD.id
);

INSERT INTO optistruct_tf(id, name, dresp, response, lc_default, integers_amount) VALUES
  (10, 'Эквивалентные (по Мизесу) напряжения', 'STRESS', '9', '0', '0');

INSERT INTO optistruct_tf_atta(id, name, prop_id) VALUES
  (19, 'Норма Мизеса', '13');

INSERT INTO optistruct_tf_allowed_atta(id, tf_id, atta_id) VALUES
  (31, 10, 19);

DELETE FROM optistruct_tf_allowed_ptype;
INSERT INTO optistruct_tf_allowed_ptype(id, tf_id, ptype_id) VALUES
  (1, 1, 0),
  (2, 2, 0),
  (3, 3, 0),
  (4, 4, 0),
  (5, 5, 0),
  (6, 6, 0),
  (7, 7, 0),
  (8, 8, 8),
  (9, 9, 0),
  (10, 10, 8);

DELETE FROM optistruct_tf_allowed_atti;
INSERT INTO optistruct_tf_allowed_atti(id, tf_id, atti_id) VALUES
  (1, 6, 1),
  (2, 8, 1),
  (3, 10, 1);


ALTER TABLE optistruct_tf ADD COLUMN node_or_element INTEGER REFERENCES optistruct_lnset_type ON DELETE SET NULL;

UPDATE optistruct_tf SET node_or_element=2 WHERE id=1;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=2;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=3;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=4;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=5;
UPDATE optistruct_tf SET node_or_element=1 WHERE id=6;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=7;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=8;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=9;
UPDATE optistruct_tf SET node_or_element=2 WHERE id=10;

UPDATE optistruct_tf SET integers_amount = 1 WHERE id = 10;

update virtual_simulation_types set display_name = 'растяжение по ГОСТ 25.601-80' where id=1;
update virtual_simulation_types set display_name = 'изгиб по ГОСТ 25.604-82' where id=2;
update virtual_simulation_types set display_name = 'сжатие по ГОСТ 25.602-80' where id=3;

