CREATE TABLE parameter_files_materials (
  id                    SERIAL,
  parameter_id          INTEGER NOT NULL,
  file_id               INTEGER NOT NULL,
  composite_material_id INTEGER NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (parameter_id) REFERENCES parameters (id),
  FOREIGN KEY (file_id) REFERENCES files (id),
  FOREIGN KEY (composite_material_id) REFERENCES composite_materials (id),
  CONSTRAINT one_parameter_file_for_material UNIQUE (composite_material_id, parameter_id)
);


CREATE TABLE parameters_materials (
  id           SERIAL,
  parameter_id INTEGER        NOT NULL,
  material_id  INTEGER        NOT NULL,
  temp         NUMERIC(20, 3) NOT NULL,
  value        NUMERIC(20, 3) NOT NULL,
  func_value   NUMERIC(20, 3),
  sim_value    NUMERIC(20, 3),

  PRIMARY KEY (id),
  FOREIGN KEY (parameter_id) REFERENCES parameters (id)
);


CREATE VIEW parameters_materials_join AS
  SELECT
    pa.id,
    pa.type,
    pa.name,
    pm.material_id                            AS material_id,
    pfm.file_id                               AS parameter_file_id,
    array_agg(pm.id ORDER BY pm.temp)         AS id_list,
    array_agg(pm.value ORDER BY pm.temp)      AS values_list,
    array_agg(pm.func_value ORDER BY pm.temp) AS func_values_list,
    array_agg(pm.sim_value ORDER BY pm.temp)  AS sim_values_list,
    array_agg(pm.temp ORDER BY pm.temp)       AS temp_list
  FROM parameters pa
    INNER JOIN parameters_materials pm ON pa.id = pm.parameter_id
    LEFT JOIN parameter_files_materials pfm ON pm.material_id = pfm.composite_material_id AND pa.id = pfm.parameter_id
  GROUP BY pa.id, pm.material_id, pfm.file_id;
