CREATE TABLE optimization_types (
  id   INTEGER,
  name VARCHAR,

  PRIMARY KEY (id)
);

INSERT INTO optimization_types (id, name) VALUES
  (1, 'Простой расчет'),
  (2, 'Полная топологическая оптимизация'),
  (3, 'Топографическая оптимизация формы'),
  (4, 'Параметрическая оптимизация формы'),
  (5, 'Топологическая оптимизация микроструктуры'),
  (6, 'Параметрическая оптимизация микроструктуры');


CREATE TABLE optimization_folders (
  id                      INTEGER DEFAULT nextval('tree_seq' :: REGCLASS),
  construction_element_id INTEGER                                           NOT NULL,
  parent_folder_id        INTEGER,
  name                    VARCHAR UNIQUE                                    NOT NULL,
  optimization_type_id    INTEGER                                           NOT NULL,
  path                    VARCHAR []                                        NOT NULL,
  path_names              VARCHAR UNIQUE                                    NOT NULL,
  owner_id                INTEGER                                           NOT NULL,
  date_created            TIMESTAMP WITHOUT TIME ZONE DEFAULT now()         NOT NULL,
  date_modified           TIMESTAMP WITHOUT TIME ZONE DEFAULT now()         NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (construction_element_id) REFERENCES construction_elements (id),
  FOREIGN KEY (parent_folder_id) REFERENCES optimization_folders (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (optimization_type_id) REFERENCES optimization_types (id)
);

CREATE TRIGGER optimization_folders_path_create
BEFORE INSERT ON optimization_folders
FOR EACH ROW
EXECUTE PROCEDURE create_path_for_optimization_folder();

CREATE TRIGGER optimization_folders_path_update
BEFORE UPDATE ON optimization_folders
FOR EACH ROW
WHEN (OLD.name IS DISTINCT FROM NEW.name
      OR OLD.parent_folder_id IS DISTINCT FROM NEW.parent_folder_id
      OR OLD.construction_element_id IS DISTINCT FROM NEW.construction_element_id)
EXECUTE PROCEDURE update_path_for_optimization_folder();

CREATE TABLE optimizations (
  id                   SERIAL,
  pid                  INTEGER                                           NOT NULL,
  name                 VARCHAR UNIQUE                                    NOT NULL,
  optimization_type_id INTEGER                                           NOT NULL,
  owner_id             INTEGER                                           NOT NULL,
  date_created         TIMESTAMP WITHOUT TIME ZONE DEFAULT now()         NOT NULL,
  date_modified        TIMESTAMP WITHOUT TIME ZONE DEFAULT now()         NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (pid) REFERENCES optimization_folders (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (optimization_type_id) REFERENCES optimization_types (id)
);