ALTER TABLE materials             ADD COLUMN reference_id INTEGER REFERENCES materials(id);
ALTER TABLE composite_materials   ADD COLUMN reference_id INTEGER REFERENCES composite_materials(id);
ALTER TABLE construction_elements ADD COLUMN reference_id INTEGER REFERENCES construction_elements(id);
ALTER TABLE optimizations         ADD COLUMN reference_id INTEGER REFERENCES optimizations(id);


DROP VIEW materials_join;
CREATE VIEW materials_join AS
  SELECT
    m.*,
    u.name  AS owner,
    mt.name AS type
  FROM materials m
    JOIN users u ON m.owner_id = u.id
    JOIN material_types mt ON m.type_id = mt.id;


DROP VIEW composite_materials_join;
CREATE VIEW composite_materials_join AS
  SELECT
    cm.*,
    u.name    AS owner,
    tech.name AS technology_name,
    e.name    AS equipment_name,
    matr.name AS matrix_name,
    fib.name  AS fiber_name
  FROM composite_materials cm
    JOIN users u ON cm.owner_id = u.id
    LEFT JOIN technologies tech ON cm.technology_id = tech.id
    LEFT JOIN equipments e ON cm.equipment_id = e.id
    LEFT JOIN materials matr ON cm.matrix_id = matr.id
    LEFT JOIN materials fib ON cm.fiber_id = fib.id;


DROP VIEW construction_elements_join;
CREATE VIEW construction_elements_join AS
  SELECT
    ce.*,
    image.name AS image_name,
    cad.name   AS cad_name
  FROM construction_elements ce
    LEFT OUTER JOIN files image
      ON ce.image_file_id = image.id
    LEFT OUTER JOIN files cad
      ON ce.cad_model_id = cad.id;


