INSERT INTO file_types (id, name) VALUES (13, 'DefectActFile');

ALTER TABLE files DROP CONSTRAINT files_path_names_key;
CREATE UNIQUE INDEX ON files(path_names) WHERE file_type_id NOT IN (4, 5, 13);

ALTER TABLE possible_defects ADD COLUMN defect_file_id INTEGER REFERENCES files(id);

DROP VIEW possible_defects_join;
CREATE VIEW possible_defects_join AS
  SELECT
    pd.*,
    pdn.name AS defect_name
  FROM possible_defects pd
    JOIN possible_defects_names pdn ON pdn.id = pd.defect_name_id;

CREATE OR REPLACE RULE possible_defects_create AS ON INSERT TO possible_defects_join DO INSTEAD (
  INSERT INTO possible_defects (technology_id, defect_name_id, defect_value, act_name, defect_file_id) VALUES
    (NEW.technology_id, NEW.defect_name_id, NEW.defect_value, NEW.act_name, NEW.defect_file_id)
  RETURNING
    possible_defects.*, '' :: VARCHAR AS defect_name);


CREATE OR REPLACE RULE possible_defects_update AS ON UPDATE TO possible_defects_join DO INSTEAD (
  UPDATE possible_defects
  SET technology_id = NEW.technology_id, defect_name_id = NEW.defect_name_id, defect_value = NEW.defect_value,
    act_name        = NEW.act_name, defect_file_id = NEW.defect_file_id
  WHERE id = NEW.id);

CREATE OR REPLACE RULE possible_defects_delete AS ON DELETE TO possible_defects_join DO INSTEAD (
  DELETE FROM possible_defects
  WHERE id = OLD.id
);
