CREATE TABLE virtual_simulation_statuses (
  id SERIAL,
  display_name VARCHAR(255) NOT NULL,

  PRIMARY KEY (id)
);

INSERT INTO virtual_simulation_statuses(display_name) VALUES
('Created'),
('Processing'),
('Failed'),
('Completed');

CREATE TABLE virtual_simulation_types (
  id SERIAL,
  display_name VARCHAR(255) NOT NULL,

  PRIMARY KEY (id)
);
INSERT INTO virtual_simulation_types(display_name) VALUES
('растяжение'),
('изгиб'),
('сжатие'),
('определение эффективных коэффициентов линейного температурного расширения'),
('определение эффективных коэффициентов теплопроводности');

DROP VIEW IF EXISTS virtual_simulations_join;
DROP TABLE simulation_standards;
DROP TABLE simulation_sample_files;
ALTER TABLE virtual_simulations DROP CONSTRAINT single_sample_file_standard_constrain;

ALTER TABLE virtual_simulations DROP COLUMN is_finished;
ALTER TABLE virtual_simulations ALTER COLUMN standard_id DROP NOT NULL;
ALTER TABLE virtual_simulations ALTER COLUMN sample_file_id DROP NOT NULL;
ALTER TABLE virtual_simulations ADD COLUMN status_id INTEGER NOT NULL REFERENCES virtual_simulation_statuses(id) DEFAULT 1;

ALTER TABLE virtual_simulations ADD COLUMN name VARCHAR(255) NOT NULL;
ALTER TABLE virtual_simulations ADD COLUMN description VARCHAR;
ALTER TABLE virtual_simulations ADD COLUMN owner_id INTEGER NOT NULL REFERENCES users(id);
ALTER TABLE virtual_simulations ADD COLUMN date_created TIMESTAMP WITHOUT TIME ZONE DEFAULT now() NOT NULL;
ALTER TABLE virtual_simulations ADD COLUMN type_id INTEGER NOT NULL REFERENCES virtual_simulation_types (id);
ALTER TABLE virtual_simulations ADD COLUMN experiment_results_file_id INTEGER REFERENCES files(id);
ALTER TABLE virtual_simulations ADD COLUMN ec_id INTEGER NOT NULL REFERENCES effective_characteristics(id);

ALTER TABLE virtual_simulations ADD COLUMN manual_size_h DOUBLE PRECISION;
ALTER TABLE virtual_simulations ADD COLUMN manual_size_b DOUBLE PRECISION;
ALTER TABLE virtual_simulations ADD COLUMN manual_size_l DOUBLE PRECISION;

CREATE VIEW virtual_simulations_join AS (
  SELECT
    vs.*,
    cm.name AS composite_material_name,
    f.name  AS sample_file_name,
    s.name  AS standard_name,
    u.name AS owner,
    f2.name AS experiment_results_file_name,
    ec.name as ec_name
  FROM virtual_simulations vs
    INNER JOIN composite_materials cm ON vs.composite_material_id = cm.id
    INNER JOIN files f ON vs.sample_file_id = f.id
    INNER JOIN users u ON u.id = vs.owner_id
    INNER JOIN effective_characteristics ec ON ec.id = vs.ec_id
    LEFT JOIN standards s ON vs.standard_id = s.id
    LEFT JOIN files f2 ON vs.experiment_results_file_id = f2.id
);
