CREATE TABLE optimization_base_setting_types (
  id SERIAL,
  max_value INTEGER NOT NULL,
  display_name VARCHAR(255),

  PRIMARY KEY (id)
);

INSERT INTO optimization_base_setting_types(max_value, display_name) VALUES
 (2, 'Количество целевых функций'),
 (8, 'Количество варьируемых параметров'),
 (8, 'Количество выходных параметров'),
 (4, 'Количество ограничений');

CREATE TABLE optimization_base_settings (
  id SERIAL,
  type_id INTEGER NOT NULL,
  optimization_id INTEGER NOT NULL,
  current_value INTEGER,

  PRIMARY KEY(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(type_id) REFERENCES optimization_base_setting_types(id)
);
CREATE INDEX ON optimization_base_settings(optimization_id);

CREATE VIEW optimization_base_settings_join AS (
  SELECT obs.*,
  obst.max_value AS max_value,
  obst.display_name AS name

  FROM optimization_base_settings obs
  JOIN optimization_base_setting_types obst ON obs.type_id = obst.id
);

CREATE RULE optimization_base_settings_join_CREATE AS ON INSERT TO optimization_base_settings_join DO INSTEAD (
    INSERT INTO optimization_base_settings (type_id, optimization_id, current_value)
     VALUES (NEW.type_id, NEW.optimization_id, NEW.current_value)
    RETURNING
      optimization_base_settings.*,
      NULL :: INTEGER AS max_value,
      NULL :: VARCHAR(255) AS name
);

CREATE RULE optimization_base_settings_join_UPDATE AS ON UPDATE TO optimization_base_settings_join DO INSTEAD (
  UPDATE optimization_base_settings
  SET current_value = NEW.current_value
  WHERE id = NEW.id);

CREATE RULE optimization_base_settings_join_DELETE AS ON DELETE TO optimization_base_settings_join DO INSTEAD (
  DELETE FROM optimization_base_settings WHERE id = OLD.id
);



CREATE TABLE optimization_varying_params (
  id SERIAL,
  optimization_id INTEGER NOT NULL,
  is_used_in_script BOOLEAN NOT NULL DEFAULT FALSE,
  name VARCHAR(255),
  lower_bound DOUBLE PRECISION,
  upper_bound DOUBLE PRECISION,
  sampling INTEGER,

  PRIMARY KEY(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE
);
CREATE INDEX ON optimization_varying_params(optimization_id);


CREATE TABLE optimization_target_funcs (
  id SERIAL,
  optimization_id INTEGER NOT NULL,
  func_id INTEGER NOT NULL DEFAULT 1,
  name VARCHAR(255),

  PRIMARY KEY(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE
);
CREATE INDEX ON optimization_target_funcs(optimization_id);

CREATE VIEW optimization_target_funcs_join AS (
  SELECT otf.*,
  pd.name AS func_name
  FROM optimization_target_funcs otf
  LEFT JOIN parameters_dictionary pd ON pd.id = otf.func_id
);

CREATE RULE optimization_target_funcs_join_CREATE AS ON INSERT TO optimization_target_funcs_join DO INSTEAD (
    INSERT INTO optimization_target_funcs (optimization_id, name, func_id)
     VALUES (NEW.optimization_id, NEW.name, NEW.func_id)
    RETURNING
      optimization_target_funcs.*,
      NULL :: VARCHAR(255) AS func_name
);

CREATE RULE optimization_target_funcs_join_UPDATE AS ON UPDATE TO optimization_target_funcs_join DO INSTEAD (
  UPDATE optimization_target_funcs
  SET name = NEW.name,
      func_id = NEW.func_id
  WHERE id = NEW.id);

CREATE RULE optimization_target_funcs_join_DELETE AS ON DELETE TO optimization_target_funcs_join DO INSTEAD (
  DELETE FROM optimization_target_funcs WHERE id = OLD.id
);


CREATE TABLE optimization_constrains (
  id SERIAL,
  optimization_id INTEGER NOT NULL,
  name VARCHAR(255),
  func_id INTEGER NOT NULL DEFAULT 4,
  val DOUBLE PRECISION,

  PRIMARY KEY(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE
);
CREATE INDEX ON optimization_constrains(optimization_id);

CREATE VIEW optimization_constrains_join AS (
  SELECT oc.*,
  pd.name AS func_name
  FROM optimization_constrains oc
  LEFT JOIN parameters_dictionary pd ON pd.id = oc.func_id
);

CREATE RULE optimization_constrains_join_CREATE AS ON INSERT TO optimization_constrains_join DO INSTEAD (
    INSERT INTO optimization_constrains (optimization_id, name, func_id, val)
     VALUES (NEW.optimization_id, NEW.name, NEW.func_id, NEW.val)
    RETURNING
      optimization_constrains.*,
      NULL :: VARCHAR(255) AS func_name
);

CREATE RULE optimization_constrains_join_UPDATE AS ON UPDATE TO optimization_constrains_join DO INSTEAD (
  UPDATE optimization_constrains
  SET name = NEW.name,
      func_id = NEW.func_id,
      val = NEW.val
  WHERE id = NEW.id);

CREATE RULE optimization_constrains_join_DELETE AS ON DELETE TO optimization_constrains_join DO INSTEAD (
  DELETE FROM optimization_constrains WHERE id = OLD.id
);
