CREATE TABLE optistruct_optimization_base_setting_types (
  id SERIAL,
  max_value INTEGER NOT NULL,
  display_name VARCHAR(255),

  PRIMARY KEY (id)
);

INSERT INTO optistruct_optimization_base_setting_types (max_value, display_name) VALUES
 (1, 'Количество целевых функций'),
 (3, 'Количество технологических ограничений'),
 (5, 'Количество ограничений');

CREATE TABLE optistruct_optimization_base_settings (
  id SERIAL,
  type_id INTEGER NOT NULL,
  optimization_id INTEGER NOT NULL,
  current_value INTEGER,

  PRIMARY KEY(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(type_id) REFERENCES optistruct_optimization_base_setting_types(id),
  UNIQUE(optimization_id, type_id)
);
CREATE INDEX ON optistruct_optimization_base_settings(optimization_id);

CREATE VIEW optistruct_optimization_base_settings_join AS (
  SELECT obs.*,
  obst.max_value AS max_value,
  obst.display_name AS name

  FROM optistruct_optimization_base_settings obs
  JOIN optistruct_optimization_base_setting_types obst ON obs.type_id = obst.id
);

CREATE RULE optistruct_optimization_base_settings_join_CREATE AS ON INSERT TO optistruct_optimization_base_settings_join DO INSTEAD (
    INSERT INTO optistruct_optimization_base_settings (type_id, optimization_id, current_value)
     VALUES (NEW.type_id, NEW.optimization_id, NEW.current_value)
    RETURNING
      optistruct_optimization_base_settings.*,
      NULL :: INTEGER AS max_value,
      NULL :: VARCHAR(255) AS name
);

CREATE RULE optistruct_optimization_base_settings_join_UPDATE AS ON UPDATE TO optistruct_optimization_base_settings_join DO INSTEAD (
  UPDATE optistruct_optimization_base_settings
  SET current_value = NEW.current_value
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_base_settings_join_DELETE AS ON DELETE TO optistruct_optimization_base_settings_join DO INSTEAD (
  DELETE FROM optistruct_optimization_base_settings WHERE id = OLD.id
);
