-- Following tables organized according to rules:
-- 1) There is a target function PROPERTY (e.g., atta);
-- 2) Not all combinations of PROPERTY and target function allowed.
--    Table with white list declared with name allowed_PROPERTY (e.g., allowed_atta);
-- 3) Chosen combination of (target function, parameter and value) stored in according field of optimization_tf.
--
--	       +----+tf+-----+
--	       |             |
--	       +             +
--	allowed_atta    optimization_tf+--+optimizations
--	       +             +
--	       |             |
--	       +---+atta+----+


CREATE TABLE optistruct_tf (
  id SERIAL,
  name VARCHAR(255) UNIQUE NOT NULL,
  dresp VARCHAR(255) NOT NULL,
  response VARCHAR(255) NOT NULL,
  lc_default VARCHAR(255) NOT NULL,
  integers_amount VARCHAR(255) NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_tf(id, name, dresp, response, lc_default, integers_amount) VALUES
  (1, 'Масса', 'MASS', '29', '0', '0'),
  (2, 'Доля массы', 'MASSFRAC', '30', '0', '0'),
  (3, 'Объем', 'VOLUME', '2', '0', '0'),
  (4, 'Доля проектируемого объема', 'VOLFRAC', '3', '0', '0'),
  (5, 'Податливость', 'COMP', '31', '1', '1'),
  (6, 'Вектор перемещений в узле', 'DISP', '7', '1', '1'),
  (7, 'Собственная частота колебаний', 'FREQ', '5', '1', '0'),
  (8, 'Сила/Момент', 'FORCE', '10', '1', '1'),
  (9, 'Взвешенная податливость', 'WCOMP', '32', '0', '0');

---------------------------------  PTYPE  ---------------------------------

CREATE TABLE optistruct_tf_ptype (
  id SERIAL,
  name VARCHAR(255) UNIQUE NOT NULL,
  prop_id VARCHAR(255) NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_tf_ptype VALUES
  (0, 'Не выбрано', '0'),
  (1, 'Произвольно', '0'),
  (2, 'PROP_TOTAL', '0'),
  (3, 'PSHELL', '1'),
  (4, 'PSOLID', '2'),
  (5, 'PSHEAR', '3'),
  (6, 'PCOMP', '4'),
  (7, 'PBAR', '5'),
  (8, 'ELEM', '14');

CREATE TABLE optistruct_tf_allowed_ptype (
  id SERIAL,
  tf_id INTEGER NOT NULL,
  ptype_id INTEGER NOT NULL,

  FOREIGN KEY(tf_id) REFERENCES optistruct_tf(id),
  FOREIGN KEY(ptype_id) REFERENCES optistruct_tf_ptype(id)
);

INSERT INTO optistruct_tf_allowed_ptype(tf_id, ptype_id) VALUES
--   (1, 0), (1, 1), (1, 2), (1, 3), (1, 4), (1, 5), (1, 6), (1, 7),
--   (2, 0), (2, 1), (2, 2), (2, 3), (2, 4), (2, 5), (2, 6), (2, 7),
--   (3, 0), (3, 1), (3, 2), (3, 3), (3, 4), (3, 5), (3, 6), (3, 7),
--   (4, 0), (4, 1), (4, 2), (4, 3), (4, 4), (4, 5), (4, 6), (4, 7),
--   (5, 0), (5, 1), (5, 2), (5, 3), (5, 4), (5, 5), (5, 6), (5, 7),
--   (6, 0), (6, 1),
--   (7, 0), (7, 1),
  (8, 1), (8, 2), (8, 3), (8, 4), (8, 5), (8, 6), (8, 7), (8, 8);
--   (9, 0), (9, 1);

---------------------------------  ATTA  ---------------------------------

CREATE TABLE optistruct_tf_atta (
  id SERIAL,
  name VARCHAR(255) UNIQUE NOT NULL,
  prop_id VARCHAR(255) NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_tf_atta VALUES
  (0, 'Не выбрано', '0'),
  (1, 'Произвольно', '0'),
  (3, 'Перемещение вдоль оси X', '1'),
  (4, 'Перемещение вдоль оси Y', '2'),
  (5, 'Перемещение вдоль оси Z', '3'),
  (6, 'Суммарный вектор перемещений', '7'),
  (7, 'Поворот вокруг оси X', '4'),
  (8, 'Поворот вокруг оси Y', '5'),
  (9, 'Поворот вокруг оси Z', '6'),
  (10, 'Суммарный вектор поворота', '8'),
  (11, 'Сила вдоль оси X', '2'),
  (12, 'Сила вдоль оси Y', '3'),
  (13, 'Сдвиговая сила в плоскости XY', '4'),
  (14, 'Изгибающий момент вдоль оси X', '5'),
  (15, 'Изгибающий момент вдоль оси Y', '6'),
  (16, 'Крутящий момент в плоскости XY', '7'),
  (17, 'Перерезывающая сила в плоскости YZ', '9'),
  (18, 'Перерезывающая сила в плоскости XZ', '8');

CREATE TABLE optistruct_tf_allowed_atta (
  id SERIAL,
  tf_id INTEGER NOT NULL,
  atta_id INTEGER NOT NULL,

  FOREIGN KEY(tf_id) REFERENCES optistruct_tf(id),
  FOREIGN KEY(atta_id) REFERENCES optistruct_tf_atta(id)
);

INSERT INTO optistruct_tf_allowed_atta(tf_id, atta_id) VALUES
  (2, 0),  (2, 1),
  (3, 0),  (3, 1),
  (4, 0),  (4, 1),
  (5, 0),  (5, 1),
  (6, 1),  (6, 3),  (6, 4),  (6, 5),  (6, 6),  (6, 7),  (6, 8),  (6, 9),  (6, 10),
  (7, 0),  (7, 1),
  (8, 1),  (8, 11), (8, 12), (8, 13), (8, 14), (8, 15), (8, 16), (8, 17), (8, 18),
  (9, 0),  (9, 1);

---------------------------------  ATTB  ---------------------------------

CREATE TABLE optistruct_tf_attb (
  id SERIAL,
  name VARCHAR(255) UNIQUE NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_tf_attb VALUES
  (0, 'Не выбрано'),
  (1, 'Произвольно'),
  (3, 'SUM'),
  (4, 'COMB'),
  (5, 'ALL'),
  (6, '№ PLY');

CREATE TABLE optistruct_tf_allowed_attb (
  id SERIAL,
  tf_id INTEGER NOT NULL,
  attb_id INTEGER NOT NULL,

  FOREIGN KEY(tf_id) REFERENCES optistruct_tf(id),
  FOREIGN KEY(attb_id) REFERENCES optistruct_tf_attb(id)
);

-- INSERT INTO optistruct_tf_allowed_attb(tf_id, attb_id) VALUES
--   (1, 1), (1, 0), (1, 3), (1, 4),
--   (2, 1), (2, 0), (2, 3), (2, 4),
--   (3, 1), (3, 0), (3, 3), (3, 4),
--   (4, 1), (4, 0), (4, 3), (4, 4),
--   (5, 1), (5, 0), (5, 3), (5, 4),
--   (6, 1), (6, 0),
--   (7, 1), (7, 0),
--   (8, 1), (8, 0),
--   (9, 1), (9, 0);

---------------------------------  ATTi  ---------------------------------

CREATE TABLE optistruct_tf_atti (
  id SERIAL,
  name VARCHAR(255) UNIQUE NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_tf_atti VALUES
  (0, 'Не выбрано'),
  (1, 'Произвольно'),
  (3, 'PID'),
  (4, 'MID'),
  (5, 'Ply ID'),
  (6, 'Grid ID'),
  (7, 'PFBODY ID'),
  (8, 'EID');

CREATE TABLE optistruct_tf_allowed_atti (
  id SERIAL,
  tf_id INTEGER NOT NULL,
  atti_id INTEGER NOT NULL,

  FOREIGN KEY(tf_id) REFERENCES optistruct_tf(id),
  FOREIGN KEY(atti_id) REFERENCES optistruct_tf_atti(id)
);

INSERT INTO optistruct_tf_allowed_atti(tf_id, atti_id) VALUES
--   (1, 1), (1, 0), (1, 3), (1, 4),
--   (2, 1), (2, 0), (2, 3), (2, 4),
--   (3, 1), (3, 0), (3, 3), (3, 4),
--   (4, 1), (4, 0), (4, 3), (4, 4),
--   (5, 1), (5, 0), (5, 3), (5, 4),
  (6, 1), --(6, 6),
--   (7, 0),
  (8, 1); --(8, 1), (8, 3), (8, 8);
--   (9, 0);

---------------------------------  OPTIMIZATION TYPE -----------------------------

CREATE TABLE optistruct_tf_optimization_type (
  id SERIAL,
  name VARCHAR(255) UNIQUE NOT NULL,

  PRIMARY KEY(id)
);

INSERT INTO optistruct_tf_optimization_type VALUES
  (0, 'MIN'),
  (1, 'MAX');

---------------------------------  SUMMARY TABLE ---------------------------------

CREATE TABLE optistruct_optimization_tf_and_constraint (
  id SERIAL,
  optimization_id INTEGER NOT NULL,
  tf_id INTEGER,
  ptype_id INTEGER,
  ptype_value VARCHAR(255),
  atta_id INTEGER,
  atta_value VARCHAR(255),
  attb_id INTEGER,
  attb_value VARCHAR(255),
  atti_id INTEGER,
  atti_value VARCHAR(255),
  optimization_type_id INTEGER,
  name VARCHAR(255),

  is_constraint BOOLEAN,
  lower_bound VARCHAR(255),
  upper_bound VARCHAR(255),

  PRIMARY KEY(id),
  FOREIGN KEY(tf_id) REFERENCES optistruct_tf(id),
  FOREIGN KEY(ptype_id) REFERENCES optistruct_tf_ptype(id),
  FOREIGN KEY(atta_id)  REFERENCES optistruct_tf_atta(id),
  FOREIGN KEY(attb_id)  REFERENCES optistruct_tf_attb(id),
  FOREIGN KEY(atti_id)  REFERENCES optistruct_tf_atti(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(optimization_type_id) REFERENCES optistruct_tf_optimization_type(id)
);

CREATE FUNCTION check_optistruct_tf_parameter_allowed()
  RETURNS TRIGGER AS $body$
BEGIN

  IF NEW.ptype_id IS NOT NULL AND NOT EXISTS(SELECT *
                                         FROM optistruct_tf_allowed_ptype
                                         WHERE tf_id = NEW.tf_id AND ptype_id = NEW.ptype_id)
  THEN
    RAISE EXCEPTION 'That tf and ptype combination is not allowed (tf: %, ptype: %)', NEW.tf_id, NEW.ptype_id;
  END IF;

  IF NEW.atta_id IS NOT NULL AND NOT EXISTS(SELECT *
                                        FROM optistruct_tf_allowed_atta
                                        WHERE tf_id = NEW.tf_id AND atta_id = NEW.atta_id)
  THEN
    RAISE EXCEPTION 'That tf and atta combination is not allowed (tf: %, atta: %)', NEW.tf_id, NEW.atta_id;
  END IF;

  IF NEW.attb_id IS NOT NULL AND NOT EXISTS(SELECT *
                                        FROM optistruct_tf_allowed_attb
                                        WHERE tf_id = NEW.tf_id AND attb_id = NEW.attb_id)
  THEN
    RAISE EXCEPTION 'That tf and attb combination is not allowed (tf: %, attb: %)', NEW.tf_id, NEW.attb_id;
  END IF;

  IF NEW.atti_id IS NOT NULL AND NOT EXISTS(SELECT *
                                        FROM optistruct_tf_allowed_atti
                                        WHERE tf_id = NEW.tf_id AND atti_id = NEW.atti_id)
  THEN
    RAISE EXCEPTION 'That tf and atti combination is not allowed (tf: %, atti: %)', NEW.tf_id, NEW.atti_id;
  END IF;

  RETURN NEW;
END;
$body$ LANGUAGE plpgsql;

CREATE TRIGGER optistruct_optimization_tf_check_parameter_allowed
BEFORE INSERT OR UPDATE ON optistruct_optimization_tf_and_constraint
FOR EACH ROW
EXECUTE PROCEDURE check_optistruct_tf_parameter_allowed();

---------------------------------  TF VIEW  ---------------------------------

CREATE VIEW optistruct_optimization_tf_join AS (
  SELECT base.id,
    base.optimization_id,
    base.tf_id,
    base.ptype_id,
    base.ptype_value,
    base.atta_id,
    base.atta_value,
    base.attb_id,
    base.attb_value,
    base.atti_id,
    base.atti_value,
    base.optimization_type_id,
    base.name,
    optistruct_tf.name as tf_name,
    optistruct_tf.response as tf_response,
    optistruct_tf_atta.name as atta_name,
    optistruct_tf_attb.name as attb_name,
    optistruct_tf_atti.name as atti_name,
    optistruct_tf_ptype.name as ptype_name,
    optistruct_tf_optimization_type.name as optimization_type_name

  FROM optistruct_optimization_tf_and_constraint base
    LEFT JOIN optistruct_tf ON base.tf_id = optistruct_tf.id
    LEFT JOIN optistruct_tf_atta ON base.atta_id = optistruct_tf_atta.id
    LEFT JOIN optistruct_tf_attb ON base.attb_id = optistruct_tf_attb.id
    LEFT JOIN optistruct_tf_atti ON base.atti_id = optistruct_tf_atti.id
    LEFT JOIN optistruct_tf_ptype ON base.ptype_id = optistruct_tf_ptype.id
    LEFT JOIN optistruct_tf_optimization_type ON base.optimization_type_id = optistruct_tf_optimization_type.id

  WHERE base.is_constraint = FALSE
);

CREATE RULE optistruct_optimization_tf_join_CREATE
AS ON INSERT TO optistruct_optimization_tf_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tf_and_constraint (
    optimization_id,
    tf_id,
    ptype_id,
    ptype_value,
    atta_id,
    atta_value,
    attb_id,
    attb_value,
    atti_id,
    atti_value,
    optimization_type_id,
    is_constraint,
    name
  )
  VALUES (
    NEW.optimization_id,
    NEW.tf_id,
    NEW.ptype_id,
    NEW.ptype_value,
    NEW.atta_id,
    NEW.atta_value,
    NEW.attb_id,
    NEW.attb_value,
    NEW.atti_id,
    NEW.atti_value,
    NEW.optimization_type_id,
    FALSE,
    NEW.name
  )
  RETURNING
    optistruct_optimization_tf_and_constraint.id,
    optistruct_optimization_tf_and_constraint.optimization_id,
    optistruct_optimization_tf_and_constraint.tf_id,
    optistruct_optimization_tf_and_constraint.ptype_id,
    optistruct_optimization_tf_and_constraint.ptype_value,
    optistruct_optimization_tf_and_constraint.atta_id,
    optistruct_optimization_tf_and_constraint.atta_value,
    optistruct_optimization_tf_and_constraint.attb_id,
    optistruct_optimization_tf_and_constraint.attb_value,
    optistruct_optimization_tf_and_constraint.atti_id,
    optistruct_optimization_tf_and_constraint.atti_value,
    optistruct_optimization_tf_and_constraint.optimization_type_id,
    NULL :: VARCHAR(255) AS tf_response,
    NULL :: VARCHAR(255) AS tf_name,
    NULL :: VARCHAR(255) AS name,
    NULL :: VARCHAR(255) AS atta_name,
    NULL :: VARCHAR(255) AS attb_name,
    NULL :: VARCHAR(255) AS atti_name,
    NULL :: VARCHAR(255) AS ptype_name,
    NULL :: VARCHAR(255) AS optimization_type_name
);

CREATE RULE optistruct_optimization_tf_join_UPDATE
AS ON UPDATE TO optistruct_optimization_tf_join DO INSTEAD (
  UPDATE optistruct_optimization_tf_and_constraint
  SET
    optimization_id = NEW.optimization_id,
    tf_id = NEW.tf_id,
    ptype_id = NEW.ptype_id,
    ptype_value = NEW.ptype_value,
    atta_id = NEW.atta_id,
    atta_value = NEW.atta_value,
    attb_id = NEW.attb_id,
    attb_value = NEW.attb_value,
    atti_id = NEW.atti_id,
    atti_value = NEW.atti_value,
    optimization_type_id = NEW.optimization_type_id,
    name = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_tf_join_DELETE
AS ON DELETE TO optistruct_optimization_tf_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tf_and_constraint WHERE id = OLD.id
);


---------------------------------  CONSTRAINTS VIEW ---------------------------------


CREATE VIEW optistruct_optimization_constraint_join AS (
  SELECT base.id,
    base.optimization_id,
    base.tf_id,
    base.ptype_id,
    base.ptype_value,
    base.atta_id,
    base.atta_value,
    base.attb_id,
    base.attb_value,
    base.atti_id,
    base.atti_value,
    base.lower_bound,
    base.upper_bound,
    base.name,
    optistruct_tf.name as tf_name,
    optistruct_tf.response as tf_response,
    optistruct_tf_atta.name as atta_name,
    optistruct_tf_attb.name as attb_name,
    optistruct_tf_atti.name as atti_name,
    optistruct_tf_ptype.name as ptype_name

  FROM optistruct_optimization_tf_and_constraint base
    LEFT JOIN optistruct_tf ON base.tf_id = optistruct_tf.id
    LEFT JOIN optistruct_tf_atta ON base.atta_id = optistruct_tf_atta.id
    LEFT JOIN optistruct_tf_attb ON base.attb_id = optistruct_tf_attb.id
    LEFT JOIN optistruct_tf_atti ON base.atti_id = optistruct_tf_atti.id
    LEFT JOIN optistruct_tf_ptype ON base.ptype_id = optistruct_tf_ptype.id

  WHERE base.is_constraint = TRUE
);

CREATE RULE optistruct_optimization_constraint_join_CREATE
AS ON INSERT TO optistruct_optimization_constraint_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tf_and_constraint (
    optimization_id,
    tf_id,
    ptype_id,
    ptype_value,
    atta_id,
    atta_value,
    attb_id,
    attb_value,
    atti_id,
    atti_value,
    is_constraint,
    lower_bound,
    upper_bound,
    name
  )
  VALUES (
    NEW.optimization_id,
    NEW.tf_id,
    NEW.ptype_id,
    NEW.ptype_value,
    NEW.atta_id,
    NEW.atta_value,
    NEW.attb_id,
    NEW.attb_value,
    NEW.atti_id,
    NEW.atti_value,
    TRUE,
    NEW.lower_bound,
    NEW.upper_bound,
    NEW.name
  )
  RETURNING
    optistruct_optimization_tf_and_constraint.id,
    optistruct_optimization_tf_and_constraint.optimization_id,
    optistruct_optimization_tf_and_constraint.tf_id,
    optistruct_optimization_tf_and_constraint.ptype_id,
    optistruct_optimization_tf_and_constraint.ptype_value,
    optistruct_optimization_tf_and_constraint.atta_id,
    optistruct_optimization_tf_and_constraint.atta_value,
    optistruct_optimization_tf_and_constraint.attb_id,
    optistruct_optimization_tf_and_constraint.attb_value,
    optistruct_optimization_tf_and_constraint.atti_id,
    optistruct_optimization_tf_and_constraint.atti_value,
    optistruct_optimization_tf_and_constraint.lower_bound,
    optistruct_optimization_tf_and_constraint.upper_bound,
    NULL :: VARCHAR(255) AS tf_response,
    NULL :: VARCHAR(255) AS tf_name,
    NULL :: VARCHAR(255) AS name,
    NULL :: VARCHAR(255) AS atta_name,
    NULL :: VARCHAR(255) AS attb_name,
    NULL :: VARCHAR(255) AS atti_name,
    NULL :: VARCHAR(255) AS ptype_name
);

CREATE RULE optistruct_optimization_constraint_join_UPDATE
AS ON UPDATE TO optistruct_optimization_constraint_join DO INSTEAD (
  UPDATE optistruct_optimization_tf_and_constraint
  SET
    optimization_id = NEW.optimization_id,
    tf_id = NEW.tf_id,
    ptype_id = NEW.ptype_id,
    ptype_value = NEW.ptype_value,
    atta_id = NEW.atta_id,
    atta_value = NEW.atta_value,
    attb_id = NEW.attb_id,
    attb_value = NEW.attb_value,
    atti_id = NEW.atti_id,
    atti_value = NEW.atti_value,
    lower_bound = NEW.lower_bound,
    upper_bound = NEW.upper_bound,
    name = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_constraint_join_DELETE
AS ON DELETE TO optistruct_optimization_constraint_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tf_and_constraint WHERE id = OLD.id
);
