CREATE TABLE optistruct_tech_constraints (
  id SERIAL,
  name VARCHAR(255),

  PRIMARY KEY (id)
);

INSERT INTO optistruct_tech_constraints (id, name) VALUES
  (1, 'Минимальный диаметр оптимизированного элемента'),
  (2, 'Максимальный диаметр оптимизированного элемента'),
  (3, 'Минимальное расстояние между оптимизированными структурными элементами');

CREATE TABLE optistruct_optimization_tech_constraints (
  id SERIAL,
  optimization_id INTEGER NOT NULL,
  tech_constr_id INTEGER NOT NULL,
  name VARCHAR(255),
  value VARCHAR(255),

  PRIMARY KEY(id),
  FOREIGN KEY(tech_constr_id) REFERENCES optistruct_tech_constraints(id) ON DELETE CASCADE,
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE
);

CREATE VIEW optistruct_optimization_tech_constraints_join AS (
  SELECT base.*,
    optistruct_tech_constraints.name AS tech_constraint_name

  FROM optistruct_optimization_tech_constraints base
    LEFT JOIN optistruct_tech_constraints ON base.tech_constr_id = optistruct_tech_constraints.id
);

CREATE RULE optistruct_optimization_tech_constraints_join_CREATE
AS ON INSERT TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  INSERT INTO optistruct_optimization_tech_constraints (optimization_id, tech_constr_id, value, name)
  VALUES (NEW.optimization_id, NEW.tech_constr_id, NEW.value, NEW.name)
  RETURNING
    optistruct_optimization_tech_constraints.*,
    NULL :: VARCHAR(255) AS tech_constraint_name
);

CREATE RULE optistruct_optimization_tech_constraints_join_UPDATE
AS ON UPDATE TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  UPDATE optistruct_optimization_tech_constraints
  SET tech_constr_id = NEW.tech_constr_id, value = NEW.value, name = NEW.name
  WHERE id = NEW.id);

CREATE RULE optistruct_optimization_tech_constraints_join_DELETE
AS ON DELETE TO optistruct_optimization_tech_constraints_join DO INSTEAD (
  DELETE FROM optistruct_optimization_tech_constraints WHERE id = OLD.id
);
