ALTER TABLE optimization_types ADD COLUMN unique_name VARCHAR(255) UNIQUE;

UPDATE optimization_types SET name = 'Инженерный расчет', unique_name = 'ENGINEERING_CALCULATION' WHERE name = 'Простой расчет';
UPDATE optimization_types SET name = 'Топологическая оптимизация', unique_name = 'TOPOLOGICAL_OPTIMIZATION' WHERE name = 'Полная топологическая оптимизация';
UPDATE optimization_types SET name = 'Топографическая оптимизация', unique_name = 'TOPOGRAPHIC_OPTIMIZATION' WHERE name = 'Топографическая оптимизация формы';
UPDATE optimization_types SET unique_name = 'PARAMETRIC_SHAPE_OPTIMIZATION' WHERE name = 'Параметрическая оптимизация формы';
UPDATE optimization_types SET unique_name = 'TOPOLOGICAL OPTIMIZATION OF MICROSTRUCTURE' WHERE name = 'Топологическая оптимизация микроструктуры';
UPDATE optimization_types SET unique_name = 'PARAMETRIC OPTIMIZATION OF MICROSTRUCTURE' WHERE name = 'Параметрическая оптимизация микроструктуры';
INSERT INTO optimization_types(id, name, unique_name) VALUES (7, 'Оптимизация формы', 'SHAPE_OPTIMIZATION');

---------------------------------------------------------------------------------------------

-- Three tables instead of one with solver type are made because we need to fill enums with
-- content of tables by type, but at class post-create time we don't know id's of types
CREATE TABLE cads (
  id SERIAL,
  name VARCHAR NOT NULL,
  unique_name VARCHAR(255) UNIQUE NOT NULL,

  PRIMARY KEY (id)
);

INSERT INTO cads (id, name, unique_name) VALUES
  (1, 'SolidWorks', 'SOLIDWORKS');

CREATE TABLE caes (
  id SERIAL,
  name VARCHAR NOT NULL,
  unique_name VARCHAR(255) UNIQUE NOT NULL,

  PRIMARY KEY (id)
);

INSERT INTO caes (id, name, unique_name) VALUES
(1, 'ANSYS', 'ANSYS'),
(2, 'ABAQUS', 'ABAQUS'),
(3, 'NASTRAN', 'NASTRAN'),
(4, 'LS-DYNA', 'LS-DYNA');

CREATE TABLE caos (
  id SERIAL,
  name VARCHAR NOT NULL,
  unique_name VARCHAR(255) UNIQUE NOT NULL,

  PRIMARY KEY (id)
);

INSERT INTO caos (id, name, unique_name) VALUES
(1, 'Tosca', 'TOSCA'),
(2, 'ModeFrontier', 'MODEFRONTIER'),
(3, 'Optistruct', 'OPTISTRUCT');

---------------------------------------------------------------------------------------------

CREATE TABLE optimization_type_solver_combinations (
  id SERIAL,
  cao_id INTEGER,
  cae_id INTEGER,
  cad_id INTEGER,
  optimization_type_id INTEGER NOT NULL,
  available BOOL NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (cao_id) REFERENCES caos(id),
  FOREIGN KEY (cae_id) REFERENCES caes(id),
  FOREIGN KEY (cad_id) REFERENCES cads(id),
  FOREIGN KEY (optimization_type_id) REFERENCES optimization_types(id)
);

INSERT INTO optimization_type_solver_combinations(optimization_type_id, cao_id, cae_id, cad_id, available) VALUES
-- Инженерный расчет
  (1, NULL, 1, NULL, FALSE), (1, NULL, 2, NULL, FALSE), (1, NULL, 3, NULL, FALSE), (1, NULL, 4, NULL, FALSE),
-- Топологическая оптимизация
  (2, 1, NULL, NULL, FALSE), (2, 3, NULL, NULL, FALSE),
  (2, NULL, 1, NULL, FALSE), (2, NULL, 2, NULL, FALSE), (2, NULL, 3, NULL, FALSE),
  (2, 1, 1, NULL, TRUE), (2, 1, 2, NULL, TRUE), (2, 1, 3, NULL, FALSE),
  (2, 3, 1, NULL, TRUE), (2, 3, 2, NULL, TRUE), (2, 3, 3, NULL, FALSE),
-- Топографическая оптимизация
  (3, 1, NULL, NULL, FALSE), (3, 3, NULL, NULL, FALSE),
  (3, NULL, 1, NULL, FALSE), (3, NULL, 2, NULL, FALSE), (3, NULL, 3, NULL, FALSE),
  (3, 1, 1, NULL, TRUE), (3, 1, 2, NULL, TRUE), (3, 1, 3, NULL, FALSE),
  (3, 3, 1, NULL, TRUE), (3, 3, 2, NULL, TRUE), (3, 3, 3, NULL, FALSE),
-- Параметрическая оптимизации формы
  (4, 2, NULL, NULL, FALSE),
  (4, NULL, 1, NULL, FALSE), (4, NULL, 3, NULL, FALSE),
  (4, NULL, NULL, 1, FALSE),
  (4, 2, 1, NULL, TRUE), (4, 2, 3, NULL, FALSE),
  (4, 2, NULL, 1, FALSE),
  (4, 2, 1, 1, FALSE), (4, 2, 3, 1, FALSE),
  (4, NULL, 1, 1, FALSE), (4, NULL, 3, 1, FALSE),
-- Топологическая оптимизация микроструктуры
  (5, 2, NULL, NULL, FALSE),
  (5, NULL, 1, NULL, FALSE), (5, NULL, 2, NULL, FALSE), (5, NULL, 3, NULL, FALSE),
  (5, 2, 1, NULL, TRUE), (5, 2, 2, NULL, FALSE), (5, 2, 3, NULL, FALSE),
-- Параметрическая оптимизация микроструктуры
  (6, 2, NULL, NULL, FALSE),
  (6, NULL, 1, NULL, FALSE), (6, NULL, 2, NULL, FALSE), (6, NULL, 3, NULL, FALSE),
  (6, 2, 1, NULL, TRUE), (6, 2, 2, NULL, TRUE), (6, 2, 3, NULL, FALSE);

CREATE OR REPLACE VIEW optimization_type_solver_combinations_join AS
SELECT combs.*,
caos.name as cao_name, caos.unique_name as cao_unique,
cads.name as cad_name, cads.unique_name as cad_unique,
caes.name as cae_name, caes.unique_name as cae_unique,
ot.name as optimization_type_name
FROM optimization_type_solver_combinations AS combs
JOIN optimization_types AS ot ON combs.optimization_type_id = ot.id
LEFT JOIN caos ON combs.cao_id = caos.id
LEFT JOIN cads ON combs.cad_id = cads.id
LEFT JOIN caes ON combs.cae_id = caes.id;

---------------------------------------------------------------------------------------------

ALTER TABLE optimizations DROP CONSTRAINT optimizations_solver_id_fkey;
ALTER TABLE optimizations ALTER COLUMN solver_id DROP NOT NULL;
ALTER TABLE optimizations RENAME solver_id TO solver_cao_id;
ALTER TABLE optimizations ADD CONSTRAINT optimizations_solver_cao_id_fkey FOREIGN KEY (solver_cao_id) REFERENCES caos(id);

ALTER TABLE optimizations ADD COLUMN solver_cae_id INTEGER DEFAULT NULL REFERENCES caes;
ALTER TABLE optimizations ADD COLUMN solver_cad_id INTEGER DEFAULT NULL REFERENCES cads;

DROP VIEW optimizations_join;

CREATE VIEW optimizations_join AS
  SELECT
    o.*,
    caos.name                                                                                  AS cao_name,
    u.name                                                                                     AS owner,
    vss.display_name                                                                           AS status,
    ot.name                                                                                    AS optimization_type,
    opf.path_names || '/' :: VARCHAR || o.name :: VARCHAR                                      AS path_names,
    opf.path || concat('{', '"name":"', o.name, '", "id":', o.id, ', "type":null}') :: VARCHAR AS path,
    o2.name                                                                                    AS reference,
    fi.name                                                                                    AS image_name,
    caes.name                                                                                  AS cae_name,
    cads.name                                                                                  AS cad_name
  FROM optimizations o
    LEFT JOIN caos ON caos.id = o.solver_cao_id
    JOIN users u ON u.id = o.owner_id
    JOIN virtual_simulation_statuses vss ON vss.id = o.status_id
    JOIN optimization_types ot ON ot.id = o.optimization_type_id
    JOIN optimization_folders opf ON opf.id = o.pid
    LEFT JOIN optimizations o2 ON o2.id = o.reference_id
    LEFT JOIN files fi ON fi.id = o.image_file_id
    LEFT JOIN caes ON caes.id = o.solver_cae_id
    LEFT JOIN cads ON cads.id = o.solver_cad_id;

CREATE OR REPLACE FUNCTION check_optimizations_cae()
  RETURNS TRIGGER AS $body$
BEGIN

  IF NOT EXISTS(SELECT *
                FROM optimization_type_solver_combinations
                WHERE ((cao_id IS NULL AND NEW.solver_cao_id IS NULL) OR (cao_id = NEW.solver_cao_id))
                AND ((cae_id IS NULL AND NEW.solver_cae_id IS NULL) OR (cae_id = NEW.solver_cae_id))
                AND ((cad_id IS NULL AND NEW.solver_cad_id IS NULL) OR (cad_id = NEW.solver_cad_id))
                AND available = TRUE)
  THEN
    RAISE EXCEPTION 'That CAD/CAE/CAO combination is not available: %, %, %', NEW.solver_cad_id, NEW.solver_cae_id, NEW.solver_cao_id;
  END IF;

  RETURN NEW;
END;
$body$ LANGUAGE plpgsql;

DROP TABLE optimization_solvers;