ALTER TABLE technologies
  ADD COLUMN technology_type VARCHAR(64) NOT NULL DEFAULT 'NOT_ADDITIVE';

DROP VIEW technologies_join;

CREATE VIEW technologies_join AS
  SELECT
    tech.*,
    descr.name         AS descr_name,
    descr.path_to_file AS descr_source,
    image.name         AS image_name,
    image.path_to_file AS image_source
  FROM technologies tech
    LEFT OUTER JOIN files descr
      ON tech.description_file_id = descr.id
    LEFT OUTER JOIN files image
      ON tech.image_file_id = image.id;

ALTER TABLE composite_materials
  ADD COLUMN epoxide_id INTEGER REFERENCES materials (id);


CREATE FUNCTION check_epoxide_references()
  RETURNS TRIGGER AS $body$
BEGIN
  --  epoxide is used only for technologies with 'ADDITIVE_3CM' type
  IF NOT EXISTS(SELECT *
                FROM technologies t
                WHERE t.id = NEW.technology_id AND t.technology_type = 'ADDITIVE_3CM')
  THEN
    NEW.epoxide_id := NULL;
  END IF;
  RETURN NEW;
END;
$body$ LANGUAGE plpgsql;


CREATE TRIGGER composite_materials_check_epoxide_references
BEFORE INSERT ON composite_materials
FOR EACH ROW
EXECUTE PROCEDURE check_epoxide_references();


DROP VIEW virtual_simulations_join;
CREATE VIEW virtual_simulations_join AS (
  SELECT
    vs.*,
    cm.name                AS composite_material_name,
    f.name                 AS sample_file_name,
    s.name                 AS standard_name,
    u.name                 AS owner,
    f2.name                AS experiment_results_file_name,
    ec.name                AS ec_name,
    (SELECT COUNT(*)
     FROM results
     WHERE sim_id = vs.id) AS results_count
  FROM virtual_simulations vs
    INNER JOIN composite_materials cm ON vs.composite_material_id = cm.id
    INNER JOIN users u ON u.id = vs.owner_id
    INNER JOIN effective_characteristics ec ON ec.id = vs.ec_id

    LEFT JOIN files f ON vs.sample_file_id = f.id
    LEFT JOIN standards s ON vs.standard_id = s.id
    LEFT JOIN files f2 ON vs.experiment_results_file_id = f2.id
);


DROP VIEW effective_characteristics_join;
CREATE VIEW effective_characteristics_join AS (
  SELECT
    ec.*,
    u.name                AS owner,
    cm.name               AS composite_material_name,
    mf.name               AS microstructure_file_name,
    (SELECT COUNT(*)
     FROM results
     WHERE ec_id = ec.id) AS results_count
  FROM effective_characteristics ec
    JOIN users u ON u.id = ec.owner_id
    JOIN composite_materials cm ON cm.id = ec.composite_material_id
    LEFT JOIN files mf ON mf.id = ec.microstructure_file_id
);


DROP VIEW composite_materials_join;
CREATE VIEW composite_materials_join AS
  SELECT
    cm.*,
    u.name               AS owner,
    tech.name            AS technology_name,
    e.name               AS equipment_name,
    matr.name            AS matrix_name,
    epo.name             AS epoxide_name,
    fib.name             AS fiber_name,

    tech.technology_type AS technology_type
  FROM composite_materials cm
    JOIN users u ON cm.owner_id = u.id
    LEFT JOIN technologies tech ON cm.technology_id = tech.id
    LEFT JOIN equipments e ON cm.equipment_id = e.id
    LEFT JOIN materials matr ON cm.matrix_id = matr.id
    LEFT JOIN materials fib ON cm.fiber_id = fib.id
    LEFT JOIN materials epo ON cm.epoxide_id = epo.id;
