/*
      +------------+        +---------------+
      |tosca_tc    |        |tosca_opt_tc   |
      +------------+        +---------------+
 +---->id          <----+   |id             <-----+
 |    |name        |    +---+constraint_id  |     |
 |    |command_name|        |optimization_id|     |
 |    +------------+        |name           |     |
 |                          |set_id         |     |
 |                          +---------------+     |
 |    +-----------------+                         |
 |    |tosca_tc_tree    |                         |
 |    +-----------------+   +-------------------+ |
 | +-->id               <-+ |tosca_tc_tree_value| |
 | |  |name             | | +-------------------+ |
 | |  |display_name     | | |id                 | |
 | +--+parent_id        | +-+tc_tree_id         | |
 |    |is_value_editable|   |tc_id              +-+
 +----+tc_id            |   |value              |
      +-----------------+   +-------------------+

 */

DELETE FROM toska_technological_constraints WHERE id = 4;

ALTER TABLE toska_technological_constraints DROP COLUMN type;
ALTER TABLE toska_technological_constraints DROP COLUMN group_name;

INSERT INTO toska_technological_constraints(id, name, comand_name) VALUES
  (5, 'Литье', 'CAST'),
  (6, 'Условия симметрии', 'LINK_TOPO');

-- name was marked UNIQUE
ALTER TABLE toska_technological_constraints DROP CONSTRAINT toska_technological_constraints_name_key;
ALTER TABLE toska_technological_constraints ADD COLUMN opt_type_id INTEGER REFERENCES optimization_types(id);
UPDATE toska_technological_constraints SET opt_type_id = 2; -- topological optimizations

-- TOSKA TECHNOLOGICAL CONSTRAINT TREE

CREATE TABLE toska_technological_constraints_tree (
  id SERIAL,
  name VARCHAR(255),
  display_name VARCHAR(255),
  is_value_editable BOOLEAN,
  parent_id INTEGER,
--   pointer to 'common' tc
  tc_id INTEGER,
  type VARCHAR(255),
  tooltip VARCHAR(255),
  group_name VARCHAR(255),

  FOREIGN KEY(tc_id) REFERENCES toska_technological_constraints(id) ON DELETE CASCADE,
  FOREIGN KEY(parent_id) REFERENCES toska_technological_constraints_tree(id) ON DELETE CASCADE,
  PRIMARY KEY(id)
);

CREATE TABLE toska_technological_constraints_tree_values (
  id SERIAL,
  tc_tree_id INTEGER,
--   pointer to optimization tc
  tc_id INTEGER,
  value VARCHAR(255),

  FOREIGN KEY(tc_id) REFERENCES toska_optimization_technological_constraints(id) ON DELETE CASCADE,
  FOREIGN KEY(tc_tree_id) REFERENCES toska_technological_constraints_tree(id) ON DELETE CASCADE,
  PRIMARY KEY(id)
);

-- FILLING THE TREE
INSERT INTO toska_technological_constraints_tree(id, tc_id, name, display_name, is_value_editable, parent_id, type, tooltip, group_name) VALUES
  -- Минимальный диаметр
  (1, 3, 'THICKNESS', 'Минимальная толщина', TRUE, NULL, NULL, NULL, NULL),

  -- Максимальный диаметр
  (2, 2, 'THICKNESS', 'Максимальная толщина', TRUE, NULL, NULL, NULL, NULL),
  (3, 2, 'MIN_THICKNESS', 'Минимальная толщина', TRUE, NULL, NULL, NULL, NULL),
  (4, 2, 'DISTANCE', 'Дистанция', TRUE, NULL, NULL, NULL, NULL),

  -- Литье
    (6,  5, 'PULL_DIR', 'Направление литья', FALSE, NULL, NULL, NULL, NULL),
      (7,  5, 'Xdir', 'Xdir', TRUE, 6, NULL, NULL, NULL),
      (8,  5, 'Ydir', 'Ydir', TRUE, 6, NULL, NULL, NULL),
      (9,  5, 'Zdir', 'Zdir', TRUE, 6, NULL, NULL, NULL),
    (10, 5, 'ANGLE', 'Угол литья', TRUE, NULL, NULL, NULL, NULL),
    (11, 5, 'CHECK_GROUP', 'Элементы для проверки', TRUE, NULL, NULL, NULL, NULL),
    (12, 5, 'MID_PLANE', 'Срединная плоскость', FALSE, NULL, NULL, NULL, NULL),
      (13, 5, 'NONE', 'Нет', FALSE, 12, 'select', NULL, 'default'),
      (14, 5, 'AUTO', 'Автоматически', FALSE, 12, 'select', NULL, 'default'),
      (15, 5, 'AUTO_TIGHT', 'Автоматически без отверстий', FALSE, 12, 'select', NULL, 'default'),
      (16, 5, 'POINT', 'Точка', FALSE, 12, 'select', NULL, 'default'),
        (17, 5, 'X', 'Направление_X', TRUE, 16, NULL, NULL, NULL),
        (18, 5, 'Y', 'Направление_Y', TRUE, 16, NULL, NULL, NULL),
        (19, 5, 'Z', 'Направление_Z', TRUE, 16, NULL, NULL, NULL),
      (20, 5, 'SURFACE', 'Поверхность', FALSE, 12, 'select', NULL, 'default'),
      (21, 5, 'STAMP', 'Штамп', FALSE, 12, 'select', NULL, 'default'),
    (22, 5, 'PULL_CS', 'Система координат литья', TRUE, NULL, NULL, 'Формат - CS_%num%', NULL),
    (23, 5, 'RADIUS', 'Радиус', TRUE, NULL, NULL, NULL, NULL),
    (24, 5, 'CREATE_PULL_GROUP', 'Автоматически создавать группы элементов', FALSE, NULL, NULL, NULL, NULL),
      (25, 5, 'NO', 'Нет', FALSE, 24, 'select', NULL, 'default'),
      (26, 5, 'YES', 'Да', FALSE, 24, 'select', NULL, 'default'),
    (27, 5, 'FILTER_MAT', 'Фильтрация материалов', FALSE, NULL, NULL, NULL, NULL),
      (28, 5, 'NO', 'Нет', FALSE, 27, 'select', NULL, 'default'),
      (29, 5, 'YES', 'Да', FALSE, 27, 'select', NULL, 'default'),

  -- Условия симметрии
  (30, 6, 'TYPE', 'Тип симметрии', FALSE, NULL, NULL, NULL, NULL),
    (31, 6, 'POINT_SYM', 'Центральная симметрия', FALSE, 30, 'select', NULL, 'default'),
    (32, 6, 'PLANE_SYM, AXIS_1', 'Зеркальная симметрия – Ось 1', FALSE, 30, 'select', NULL, 'default'),
    (33, 6, 'PLANE_SYM, AXIS_2', 'Зеркальная симметрия – Ось 2', FALSE, 30, 'select', NULL, 'default'),
    (34, 6, 'PLANE_SYM, AXIS_3', 'Зеркальная симметрия – Ось 3', FALSE, 30, 'select', NULL, 'default'),
    (35, 6, 'ROTATION_SYM, AXIS_1', 'Осевая симметрия – Ось 1', FALSE, 30, 'select', NULL, 'default'),
      (36, 6, 'ANGLE', 'Угол', TRUE , 35, NULL, NULL, NULL),
    (37, 6, 'ROTATION_SYM, AXIS_2', 'Осевая симметрия – Ось 2', FALSE, 30, 'select', NULL, 'default'),
      (38, 6, 'ANGLE', 'Угол', TRUE , 37, NULL, NULL, NULL),
    (39, 6, 'ROTATION_SYM, AXIS_3', 'Осевая симметрия – Ось 3', FALSE, 30, 'select', NULL, 'default'),
      (40, 6, 'ANGLE', 'Угол', TRUE , 39, NULL, NULL, NULL),
    (41, 6, 'CYCLIC_SYM, AXIS_1', 'Циклическая симметрия – Ось 1', FALSE, 30, 'select', NULL, 'default'),
      (42, 6, 'TRANSLATION', 'Смещение', TRUE , 41, NULL, NULL, NULL),
    (43, 6, 'CYCLIC_SYM, AXIS_2', 'Циклическая симметрия – Ось 2', FALSE, 30, 'select', NULL, 'default'),
      (44, 6, 'TRANSLATION', 'Смещение', TRUE , 43, NULL, NULL, NULL),
    (45, 6, 'CYCLIC_SYM, AXIS_3', 'Циклическая симметрия – Ось 3', FALSE, 30, 'select', NULL, 'default'),
      (46, 6, 'TRANSLATION', 'Смещение', TRUE , 45, NULL, NULL, NULL),
    (47, 6, 'LINK_SYM', 'Симметрия групп', FALSE, 30, 'select', NULL, 'default'),
      (48, 6, 'GROUP_MASTER', 'Отсчетная группа', TRUE, 47, NULL, NULL, NULL),
      (49, 6, 'CS_CHILD', 'Зависимые СК', TRUE, 47, NULL, 'Формат - CS_%num1, CS_%num2%', NULL),
      (50, 6, 'MASTER', 'Подобие', FALSE, 47, NULL, NULL, NULL),
        (51, 6, 'AUTO', 'Автоматически', FALSE, 50, 'select', NULL, 'default'),
        (52, 6, 'MAX', 'Максимальное', FALSE, 50, 'select', NULL, 'default'),
        (53, 6, 'AVG', 'Среднее', FALSE, 50, 'select', NULL, 'default'),
        (54, 6, 'GROUP', 'Директивное', FALSE, 50, 'select', NULL, 'default'),
  (55, 6, 'Radius', 'Радиус', TRUE, NULL, NULL, NULL, NULL),
  (57, 6, 'CS', 'Система координат', TRUE, NULL, NULL, 'Формат - CS_%num%', NULL),
  (58, 6, 'IGNORE_FROZEN', 'Исключить фикс. обл.', FALSE, NULL, NULL, NULL, NULL),
    (59, 6, 'NO', 'Нет', FALSE, 58, 'select', NULL, 'default'),
    (60, 6, 'YES', 'Да', FALSE, 58, 'select', NULL, 'default')
;

-- MIGRATING DATA

INSERT INTO toska_technological_constraints_tree_values (tc_tree_id, tc_id, value)
SELECT
  field_id AS tc_tree_id,
  tech_constr_id AS tc_id,
  value AS value
FROM toska_technological_constraints_fields_values;

-- REMOVING OLD TABLES

DROP VIEW toska_technological_constraints_fields_join;
DROP TABLE toska_technological_constraints_fields_values;
DROP TABLE toska_technological_constraints_fields;

-- VIEWS

CREATE VIEW toska_technological_constraints_extended AS
  SELECT ttc.*, EXISTS(SELECT * FROM toska_technological_constraints_tree WHERE tc_id = ttc.id) as has_fields
  FROM toska_technological_constraints ttc
  ORDER BY ttc.name;

CREATE VIEW toska_technological_constraints_tree_extended AS
  SELECT ttct.*, EXISTS(SELECT * FROM toska_technological_constraints_tree child WHERE child.parent_id = ttct.id) as is_parent
  FROM toska_technological_constraints_tree ttct
  ORDER BY ttct.name;

CREATE VIEW toska_technological_constraints_tree_values_join AS
  SELECT ttctv.*, ttct.display_name, ttct.is_value_editable, ttct.parent_id, ttct.name, ttct.is_parent, ttct.type, ttct.tooltip, ttct.group_name
  FROM toska_technological_constraints_tree_extended ttct
    JOIN toska_technological_constraints_tree_values ttctv
      ON ttctv.tc_tree_id = ttct.id;

CREATE RULE toska_technological_constraints_tree_values_join_CREATE
AS ON INSERT TO toska_technological_constraints_tree_values_join DO INSTEAD (
  INSERT INTO toska_technological_constraints_tree_values (tc_tree_id, tc_id, value)
  VALUES (NEW.tc_tree_id, NEW.tc_id, NEW.value)
  RETURNING
    toska_technological_constraints_tree_values.*,
    NULL :: VARCHAR(255) AS field_name,
    FALSE as is_value_setable,
    NULL :: INTEGER as parent_id,
    NULL :: VARCHAR(255) AS name,
    NULL :: BOOLEAN AS is_parent,
    NULL :: VARCHAR(255) AS type,
    NULL :: VARCHAR(255) AS tooltip,
    NULL :: VARCHAR(255) AS group_name
);

CREATE RULE toska_technological_constraints_tree_values_join_UPDATE
AS ON UPDATE TO toska_technological_constraints_tree_values_join DO INSTEAD (
  UPDATE toska_technological_constraints_tree_values
  SET value = NEW.value
  WHERE tc_tree_id = NEW.tc_tree_id);

CREATE RULE toska_technological_constraints_tree_values_join_DELETE
AS ON DELETE TO toska_technological_constraints_tree_values_join DO INSTEAD (
  DELETE FROM toska_technological_constraints_tree_values WHERE id = OLD.id
);

-- TRIGGERS

CREATE FUNCTION toska_technological_constraints_tree_values_checks()
  RETURNS TRIGGER AS $body$
BEGIN

  IF EXISTS(SELECT * FROM toska_technological_constraints_tree WHERE parent_id = NEW.tc_tree_id)
  THEN
    RAISE EXCEPTION 'Tosca technological constraints field values must be leafs, tc_tree_id = %', NEW.tc_tree_id;
  END IF;

  IF NOT EXISTS(SELECT * FROM toska_technological_constraints_tree WHERE id = NEW.tc_tree_id AND is_value_editable = TRUE)
  THEN
    NEW.value := NULL;
  END IF;

  RETURN NEW;
END;
$body$ LANGUAGE plpgsql;

CREATE TRIGGER toska_technological_constraints_tree_values_must_be_leafs_trigger
BEFORE INSERT OR UPDATE ON toska_technological_constraints_tree_values
FOR EACH ROW
EXECUTE PROCEDURE toska_technological_constraints_tree_values_checks();
