/**
 * Created by lexeek on 19.09.14.
 */

(function () {
  "use strict";
  function finiteInt(val, name) {
    if (!_.isNumber(val) || _.isNaN(val) || !_.isFinite(val) || ~~val !== val) {
      throw new TypeError("`" + name + "` must be a finite integer");
    }
    return val;
  }

  window.App.Collections.GridBaseCollection = Backbone.PageableCollection.extend({
    setPageableData: function (data) {
      this.queryParams.pageable = {
        "size": this.state.pageSize = (data.size || this.state.pageSize) || this.queryParams.pageable.size,
        "page": this.state.currentPage = (data.page || this.state.currentPage) || this.queryParams.pageable&&this.queryParams.pageable.page
      };
    },

    setDefaultParams: function () {
      var filters = {},
        path = window.App.Helpers.getParamFromHash('path', true),
        pid = window.App.Helpers.getParamFromHash('id');

      if (this.options.showContent === 'directory') {
        if (pid) {
          filters.pid = parseInt(pid);
        }
      } else {
        if (path) {
            filters.path = path + '%';
        }
      }

      this.queryParams = {
        "filters": filters,
        "sort": [],
        "pageable": {
          "size": 25, //Default page size - sets in page table tool
          "page": 1
        },
        currentPage: null,
        pageSize: null,
        totalPages: null,
        totalRecords: null,
        sortKey: null
      };
    },

    setQueryParams: function (data) {
      this.setDefaultParams();
      data = data || {};

      this.setPageableData(data);
      this.queryParams.filters = (data.filters || this.state.filters || this.queryParams.filters);
      this.queryParams.sort = data.sort || [];
    },

    getPage: function (index, options) {
      var pageNum = index;
      switch (index) {
        case "first":
          pageNum = this.state.firstPage;
          break;
        case "prev":
          pageNum = this.state.currentPage - 1;
          break;
        case "next":
          pageNum = this.state.currentPage + 1;
          break;
        case "last":
          pageNum = this.state.totalPages;
          break;
        default:
          pageNum = finiteInt(index, "index");
      }

      this.setPageableData({page: pageNum});
      Backbone.PageableCollection.prototype.getPage.apply(this, arguments);
    },

    getQueryParams: function (param) {
      if (param) {
        return _.clone(_.pick(this.queryParams, param))[param];
      }
      return _.clone(this.queryParams);
    },

    setSorting: function (field, direction, c) {
      var sortData = [],
        data = {};

      if (!field) {
        this.setQueryParams({sort: sortData});
        return;
      }
      data.direction = direction > 0 ? "DESC" : "ASC";
      data.field = field;
      if (field === "status") data.field = "simulationStatus"
      sortData.push(data);
      this.setQueryParams({sort: sortData});
    },

    initialize: function (models, options) {
      this.options = options || {};

      if (this.options.filters) {
        this.setQueryParams(this.options);
      } else {
        this.setQueryParams();
      }

      this.fetched = false;
      var that = this;

      this.listenTo(this, 'sync', function () {
        that.fetched = true;
      })
    },

    getModel: function () {
      return this.model || {};
    },

    parseState: function (resp, queryParams, state, options) {
      var newState = _.clone(this.state);
      newState.pageSize = resp.size;
      newState.totalRecords = resp.totalElements;
      newState.totalPages = resp.totalPages;
      newState.lastPage = resp.totalPages;
      newState.filters = queryParams.filters;

      return newState;
    },

    parseRecords: function (resp, options) {
      this.setQueryParams({filters: resp.filterValues});

      if (this.options.pageable) {
        return resp.content;
      } else {
        return resp;
      }
    },

    sync: function (method, model, options) {
      if (this.options.pageable || this.options.usePost) {
        options.data = JSON.stringify(options.data);
      }

      return Backbone.sync(method, model, options);
    },

    getTotalRecords: function () {
      return this.state.totalRecords ? this.state.totalRecords : this.length;
    },

    fetch: function (options) {
      options = options || {};

      if (this.options.pageable || this.options.usePost) {
        options.type = "POST";
        options.contentType = 'application/json';
      }

      if (!this.url) {
        return;
      }
      var spinner = new App.Utils.Spinner();
      spinner.show();

      var callbacks = {
        success: function (collection, response, options) {
          spinner.hide();
        },
        error: function (model, err) {
          spinner.hide();
          window.alertify.error(err.status + ": " + err.statusText || "Server Error " + err.message);
        }
      };
      try {
        if (this.options.pageable) {
          Backbone.PageableCollection.prototype.fetch.call(this, _.extend(options, callbacks));
        } else {
          Backbone.Collection.prototype.fetch.call(this, _.extend(options, callbacks));
        }
      } catch (e) {
        console.log("backgrid_base_collection.js catch ", e);
      }

    }

  });

})();