(function () {
  "use strict";

  App.Controllers.ProjectController = Marionette.Controller.extend({

    initialize: function (options) {
      this.options = options;
    },

    showProject: function (options) {
      this._showPage(options, "project");
    },

    ProjectHomePage: function (options) {
      this.currentPageView = new window.App.Views.ProjectHomeView(options);
      this._showPage(options, "home");
    },

    ProjectTaskPage: function (options) {
      var that = this;
      var task = new App.Models.Task({id: options.id});
      task.fetch({
        success: function () {
          that.currentPageView = new window.App.Views.ProjectTaskView({model: task});
          that._showPage(options, "project");
        }
      });
    },

    ProjectMaterialsListPage: function (options) {
      this.currentPageView = new window.App.Views.MaterialsListView(_.extend({
        gridCollection: new App.Collections.Materials([], {
          pageable: true,
          filters: options.filters || null
        }),
        actions: ["open", "new", "delete"],
        type: "materials"
      }, options));
      this._showPage(options, "materials");
    },


    ProjectMaterialCreatePage: function (options) {
      this._showPage(options, "project");
      this.createMaterialView = new window.App.Views.CreateMaterialView(options);
      this.createModal = new Backbone.BootstrapModal({
        content: this.createMaterialView,
        title: "Создание Материала",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.createMaterialView.passModal(this.createModal);
    },

    ProjectMaterialParametersPage: function (options) {
      var that = this;
      var model = new window.App.Models.Material({ id: options.id });
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.MaterialParametersPage({
          model: model,
          mode: options.mode,
          modelName: "materials"
        });
        that._showPage(options, "materials");
      });
    },

    ProjectMaterialFilesPage: function (options) {
      var that = this;
      var model = new window.App.Models.Material({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.MaterialFilesPage({
          model: model,
          mode: options.mode,
          modelName: "materials"
        });
        that._showPage(options, "materials");
      });
    },

    ProjectCompositeMaterialsListPage: function (options) {
      this.currentPageView = new window.App.Views.CompositeMaterialsListView(_.extend({
        gridCollection: new App.Collections.CompositeMaterials([], {
          pageable: true,
          filters: options.filters || null
        }),
        actions: ["open", "new", "delete"],
        type: "compositeMaterials"
      }, options));
      this._showPage(options, "materials");
    },

    ProjectCompositeMaterialCreatePage: function (options) {
      this._showPage(options, "project");
      this.createMaterialView = new window.App.Views.CreateCompositeMaterialView(options);
      this.createModal = new Backbone.BootstrapModal({
        content: this.createMaterialView,
        title: "Создание Композиционного Материала",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.createMaterialView.passModal(this.createModal);
    },

    ProjectCompositeMaterialParametersPage: function (options) {
      var that = this;
      var model = new window.App.Models.CompositeMaterial({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.CompositeMaterialParametersPage({
          model: model,
          mode: options.mode,
          modelName: "compositeMaterials"
        });
        that._showPage(options, "materials");
      });
    },

    ProjectCompositeMaterialEffectiveCharacteristicsPage: function (options) {
      var that = this;
      var model = new window.App.Models.CompositeMaterial({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.CompositeMaterialEffectiveCharacteristicsPage({
          model: model,
          mode: options.mode,
          type: "effectiveCharacteristics",
          modelName: "compositeMaterials"
        });
        that._showPage(options, "materials");
      });
    },

    ProjectCompositeMaterialEffectiveCharacteristicCreatePage: function (options) {
      this._showPage(options, "project");
      this.createEffectiveChatacteristicView = new window.App.Views.CreateEffectiveCharacteristicView({compositeMaterialId: options.id});
      this.createModal = new Backbone.BootstrapModal({
        content: this.createEffectiveChatacteristicView,
        title: "Создание Эффективной Характеристики",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.createEffectiveChatacteristicView.passModal(this.createModal);
    },

    ProjectCompositeMaterialSimulationsPage: function (options) {
      var that = this;
      var model = new window.App.Models.CompositeMaterial({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.CompositeMaterialSimulationsPage({
          model: model,
          mode: options.mode,
          type: "simulations",
          modelName: "compositeMaterials"
        });
        that._showPage(options, "materials");
      });
    },

    ProjectCompositeMaterialSimulationCreatePage: function (options) {
      this._showPage(options, "project");
      this.createSimulationView = new window.App.Views.CreateSimulationView({compositeMaterialId: options.id});
      this.createModal = new Backbone.BootstrapModal({
        content: this.createSimulationView,
        title: "Создание Виртуального Испытания",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.createSimulationView.passModal(this.createModal);
    },

    ProjectCompositeMaterialFilesPage: function (options) {
      var that = this;
      var model = new window.App.Models.CompositeMaterial({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.CompositeMaterialFilesPage({
          model: model,
          mode: options.mode,
          fileType: options.fileType,
          modelName: "compositeMaterials",
          isFile: options.fileType != "effectiveCharacteristics" && options.fileType != "simulations"
        });
        that._showPage(options, "materials");
      });
    },

    ProjectEffectiveCharacteristicPage: function (options) {
      var that = this;
      var model = new window.App.Models.EffectiveCharacteristic({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.EffectiveCharacteristicParametersPage({
          model: model,
          mode: options.mode,
          modelName: "effectiveCharacteristics"
        });
        that._showPage(options, "compositeMaterials");
      });
    },

    ProjectEffectiveCharacteristicResultsPage: function (options) {
      var that = this;
      var model = new window.App.Models.EffectiveCharacteristic({ id: options.id} );
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.EffectiveCharacteristicResultsPage({
          model: model,
          mode: options.mode,
          modelName: "effectiveCharacteristics"
        });
        that._showPage(options, "compositeMaterials");
      });
    },

    ProjectSimulationParametersPage: function (options) {
      var that = this;
      var model = new window.App.Models.Simulation({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.SimulationParametersPage({
          model: model,
          mode: options.mode,
          modelName: "simulations"
        });
        that._showPage(options, "compositeMaterials");
      });
    },

    ProjectSimulationResultsPage: function (options) {
      var that = this;
      var model = new window.App.Models.Simulation({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.SimulationResultsPage({
          model: model,
          mode: options.mode,
          modelName: "simulations"
        });
        that._showPage(options, "compositeMaterials");
      });
    },

    ProjectUsersPage: function (options) {
      this.currentPageView = new window.App.Views.Users({
        gridCollection: new window.App.Collections.Users([], {
          pageable: true,
          filters: options.filters || null
        }),
        actions: window.App.Data.UserRole == "Administrator" ? ["open", "new", "delete"] : ["open"],
        type: "user"
      }, options);
      this._showPage(options, "users");
    },

    ProjectUserPage: function (options) {
      var that = this;
      this.user = new window.App.Models.User({id: options.id});
      this.user.fetch().done(function () {
        that.currentPageView = new window.App.Views.ProfileView({
          model: that.user,
          mode: options.mode
        });
        that._showPage(options, "users");
      });
    },

    ProjectCreateUserPage: function (options) {
      this.currentPageView = new window.App.Views.CreateUserView({
        mode: options.mode
      });
      this._showPage(options, "users");
    },

    ProjectTechnologiesListPage: function (options) {
      this.currentPageView = new window.App.Views.TechnologiesListView(_.extend({
        gridCollection: new App.Collections.Technologies([], {
          pageable: true,
          filters: options.filters || null
        }),
        actions: ["open", "new", "delete"],
        type: "technologies"
      }, options));
      this._showPage(options, "technologies");
    },

    ProjectTechnologyCreatePage: function (options) {
      this._showPage(options, "project");
      this.createTechnologyView = new window.App.Views.CreateTechnologyView();
      this.createModal = new Backbone.BootstrapModal({
        content: this.createTechnologyView,
        title: "Создание Технологии",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.createTechnologyView.passModal(this.createModal);
    },

    ProjectTechnologyParameterCreatePage: function (options) {
      var parameterTypeName;
      switch (options.parameterType) {
        case "technologyDefects":
          this.view = new window.App.Views.CreateTechnologyDefectView(options);
          parameterTypeName = "Дефект";
          break;
        case "technologyParameters":
          this.view = new window.App.Views.CreateTechnologyParameterView(options);
          parameterTypeName = "Характеристику";
          break;
        case "technologyEquipments":
          this.view = new window.App.Views.CreateTechnologyParameterView(options);
          parameterTypeName = "Оборудование";
          break;
      }

      this.createModal = new Backbone.BootstrapModal({
        content: this.view,
        title: "Сохранить " + parameterTypeName,
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.view.passModal(this.createModal);
    },

    ProjectTechnologyParametersPage: function (options) {
      var that = this;
      var model = new window.App.Models.Technology({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.TechnologyParametersPage({
          model: model,
          mode: options.mode,
          modelName: "technologies"
        });
        that._showPage(options, "technologies");
      });
    },

    ProjectTechnologySimulationCreatePage: function (options) {
        this._showPage(options, "project");
        this.createTechnologySimulationView = new window.App.Views.CreateTechnologySimulationView({technologyId: options.id});
        this.createModal = new Backbone.BootstrapModal({
            content: this.createTechnologySimulationView,
            title: "Создание Технологического Расчета",
            animate: true,
            cancelText: "Отменить",
            okText: "Сохранить"
        }).open();
        this.createTechnologySimulationView.passModal(this.createModal);
    },

    ProjectTechnologySimulationsPage: function (options) {
        var that = this;
        var model = new window.App.Models.Technology({id: options.id});
        model.fetch().done(function () {
            that.currentPageView = new window.App.Views.TechnologySimulationsPage({
                model: model,
                mode: options.mode,
                type: "technologySimulations",
                fileType: "technologySimulations",
                modelName: "technologies"
            });
            that._showPage(options, "technologies");
        });
    },

    ProjectTechnologySimulationParametersPage: function (options) {
        var that = this;
        var model = new window.App.Models.TechnologySimulation({id: options.id});
        model.fetch().done(function () {
            that.currentPageView = new window.App.Views.TechnologySimulationParametersPage({
                model: model,
                mode: options.mode,
                modelName: "technologySimulations"
            });
            that._showPage(options, "technologies");
        });
    },

    ProjectTechnologySimulationCopyPage: function (options) {
        this._showPage(options, "project");
        this.copyTechnologySimulationView = new window.App.Views.CopyTechnologySimulationView(_.extend({ modal: this.createModal, model: this.currentPageView.model }, options))
        this.createModal = new Backbone.BootstrapModal({
            allowCancel: false,
            close: true,
            content: this.copyTechnologySimulationView,
            title: "Создание оптимизации",
            animate: true,
            cancelText: "Отменить",
            okText: "Сохранить",

        }).open();
        this.copyTechnologySimulationView.passModal(this.createModal);
    },

    ProjectConstructionElementsListPage: function (options) {
      this.currentPageView = new window.App.Views.ConstructionElementsListView(_.extend({
        gridCollection: new App.Collections.ConstructionElements([], {
          pageable: true,
          filters: options.filters || null
        }),
        actions: ["open", "new", "delete"],
        type: "constructionElements"
      }, options));
      this._showPage(options, "constructionElements");
    },

    ProjectConstructionElementsCreatePage: function (options) {
      this.createConstructionElementView = new window.App.Views.CreateConstructionElementView(options);
      this.createModal = new Backbone.BootstrapModal({
        content: this.createConstructionElementView,
        title: "Создание Элемента Конструкции",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this._showPage(options, "project");
      this.createConstructionElementView.passModal(this.createModal);
    },

    ProjectConstructionElementsParametersPage: function (options) {
      var that = this;
      var model = new window.App.Models.ConstructionElement({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.ConstructionElementParametersPage({
          model: model,
          mode: options.mode,
          modelName: "constructionElements"
        });
        that._showPage(options, "constructionElements");
      });
    },

    ProjectConstructionElementsFilesPage: function (options) {
      var that = this;
      var model = new window.App.Models.ConstructionElement({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.ConstructionElementFilesPage({
          model: model,
          fileType: options.fileType,
          mode: options.mode,
          modelName: "constructionElements"
        });
        that._showPage(options, "constructionElements");
      });
    },

    ProjectCreateFilePage: function (options) {
      this._showPage(options, "project");
      this.createFileView = new window.App.Views.CreateFileView(_.extend({modal: this.createModal}, options))
      this.createModal = new Backbone.BootstrapModal({
        content: this.createFileView,
        title: "Загрузка Файла",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.createFileView.passModal(this.createModal);
    },

    ProjectFileParametersPage: function (options) {
      var that = this;
      var model = new window.App.Models.File({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.FileParametersPage({
          model: model,
          mode: options.mode,
          modelName: "file"
        });
        that._showPage(options, "project");
      });
    },

    ProjectOptimizationsListPage: function (options) {
      this.currentPageView = new window.App.Views.OptimizationsListView(_.extend({
        gridCollection: new App.Collections.Optimizations([], {
          pageable: true,
          filters: options.filters || null,
          showContent: window.App.Helpers.getParamFromHash('showContent')
        }),
        actions: ["open", "new", "delete"],
        type: "optimizations"
      }, options));
      this._showPage(options, "optimizations");
    },

    ProjectOptimizationCreatePage: function (options) {
      this._showPage(options, "project");
      this.createOptimizationView = new window.App.Views.CreateOptimizationView(_.extend({ modal: this.createModal }, options))
      this.createModal = new Backbone.BootstrapModal({
        allowCancel: false,
        close: true,
        content: this.createOptimizationView,
        title: "Создание оптимизации",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить",

      }).open();
      this.createOptimizationView.passModal(this.createModal);
    },

    ProjectOptimizationCopyPage: function (options) {
        this._showPage(options, "project");
        this.copyOptimizationView = new window.App.Views.CopyOptimizationView(_.extend({ modal: this.createModal, model: this.currentPageView.model }, options))
        this.createModal = new Backbone.BootstrapModal({
            allowCancel: false,
            close: true,
            content: this.copyOptimizationView,
            title: "Создание оптимизации",
            animate: true,
            cancelText: "Отменить",
            okText: "Сохранить",

        }).open();
        this.copyOptimizationView.passModal(this.createModal);
    },

    /**
     * Create Optimization Folder from Optimization tree
     * @param options
     *
     */
    ProjectOptimizationFolderCreatePage: function (options) {
      this._showPage(options, "project");
      this.createOptimizationFolderView = new window.App.Views.CreateOptimizationFolderView(_.extend({}, options))
      this.createModal = new Backbone.BootstrapModal({
        content: this.createOptimizationFolderView,
        title: "Создание директории оптимизаций",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.createOptimizationFolderView.passModal(this.createModal);
    },

    ProjectOptimizationParametersPage: function (options) {
      var that = this;
      var model = new window.App.Models.Optimization({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.OptimizationParametersPage({
          model: model,
          mode: options.mode,
          modelName: "optimizations"
        });
        that._showPage(options, "optimizations");
      });
    },

    ProjectOptimizationSettingsPage: function (options) {
      var that = this;
      var model = new window.App.Models.Optimization({ id: options.id });
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.OptimizationSettingsPage({
          model: model,
          mode: options.mode,
          tab: options.tab,
          modelName: "optimizations"
        });
        that._showPage(options, "optimizations");
      });
    },

    ProjectOptimizationLevelsPage: function (options) {
      var that = this;
      var model = new window.App.Models.Optimization({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.OptimizationLevelsPage({
          model: model,
          mode: options.mode,
          modelName: "optimizations"
        });
        that._showPage(options, "optimizations");
      });
    },

    ProjectOptimizationReferencesPage: function (options) {
      var that = this;
      var model = new window.App.Models.Optimization({id: options.id});
      model.fetch().done(function () {
        that.currentPageView = new window.App.Views.OptimizationReferencesPage({
          model: model,
          mode: options.mode,
          modelName: "optimizations"
        });
        that._showPage(options, "optimizations");
      });
    },

    ProjectQueuePage: function (options) {
      this.currentPageView = new window.App.Views.ProjectQueuePage(options);
      this._showPage({}, "queue");
    },

    _showPage: function (options, menuItem) {
      var projectView = new window.App.Views.ProjectView();
      this.options.region.show(projectView);

      this.navigationView = projectView.navRegion.currentView;
      this.mainRegion = projectView.projectMainRegion;

      if (this.currentPageView && !this.currentPageView.isDestroyed) {
        this.navigationView.setActiveMenuItem(menuItem);
        this.mainRegion.show(this.currentPageView);
      }
    }

  })
}());