(function () {
    "use strict";

    window.App.Helpers.AnimationView = {

        showAnimation: function (Obj) {
            let description = _.isNull(Obj.description) ? "Отсутствует" : Obj.description;
            $(".js-description").html('<div>'+description+'</div>');
            if (!Detector.webgl) Detector.addGetWebGLMessage();
            var container, stats, controls, aId;
            var camera, scene, renderer, mesh, mScale, aspect;
            var fitPropObject = {}, spaceCount = 0;
            var xhr;
            init();
            //animate();
            function init() {
                container = document.createElement('div');
                container.className = "animationElement";
                var rightPlace = document.querySelector(".js-animation");
                rightPlace.appendChild(container);

                aspect = window.innerWidth / window.innerHeight;
                camera = new THREE.PerspectiveCamera(5, aspect, 0.1, 1);
                var vFOV = camera.fov * Math.PI / 180;

                scene = new THREE.Scene();
                scene.fog = new THREE.Fog(0xffffff, 0.1, 1000000);

                // ASCII file

                // Binary files
                var material = new THREE.MeshPhongMaterial({color: 0x2A5982, specular: 0x0, shininess: 100});
                // Colored binary STL
                var loader = new THREE.STLLoader();
                loader.load('rest/results/' + Obj.id + '/content', function (geometry) {
                        var meshMaterial = material;
                        if (geometry.hasColors) {
                            meshMaterial = new THREE.MeshPhongMaterial({
                                opacity: geometry.alpha,
                                vertexColors: THREE.VertexColors
                            });
                        }
                        geometry.computeVertexNormals();
                        geometry.center();
                        mesh = new THREE.Mesh(geometry, meshMaterial);
                        geometry.computeBoundingBox();
                        fitPropObject.boxsize = geometry.boundingBox.getSize();
                        fitPropObject.maxBoxsize = Math.max(fitPropObject.boxsize.x, fitPropObject.boxsize.y, fitPropObject.boxsize.z);
                        geometry.computeBoundingSphere();
                        fitPropObject.radius = geometry.boundingSphere.radius;
                        fitPropObject.dF = Math.abs(0.5 * (0.5 * fitPropObject.maxBoxsize + fitPropObject.radius) / Math.tan(vFOV / 2));
                        camera.far = 1000 * fitPropObject.dF;
                        camera.near = fitPropObject.dF * 0.001;
                        camera.position.x = fitPropObject.dF;
                        camera.position.y = 0;
                        camera.position.z = fitPropObject.dF;
                        scene.fog.far = 1000 * fitPropObject.dF;
                        scene.fog.near = fitPropObject.dF * 0.001;
                        camera.updateProjectionMatrix();
                        var axis = new THREE.AxisHelper(fitPropObject.radius / 5);
                        scene.add(axis);

                        //Mesh
                        mScale = 1;
                        mesh.scale.set(mScale, mScale, mScale);
                        mesh.castShadow = true;
                        mesh.receiveShadow = true;
                        scene.add(mesh);

                        // Lights
                        var ambLight = new THREE.AmbientLight(0x000000);
                        scene.add(ambLight);
                        var hemiLight = new THREE.HemisphereLight(0xffffff, 0xffffff, 1);
                        hemiLight.color.setStyle("rgb(255, 255, 255)");
                        hemiLight.groundColor.setStyle("rgb(28, 75, 112)");
                        hemiLight.position.set(0, 1, 0);
                        scene.add(hemiLight);
                        addShadowedLight( 1, 1, 1, 0xffffff, 1.35 );
                        //addShadowedLight(0, 1, 0, 0x000000, 1);

                        // renderer
                        var canvas = document.createElement("canvas");
                        canvas.id = "canvas3D";
                        //canvas.className = "col-md-11";
                        renderer = new THREE.WebGLRenderer({canvas: canvas, antialias: true});
                        renderer.setClearColor(scene.fog.color);
                        renderer.setPixelRatio(window.devicePixelRatio);
                        renderer.setSize(0.55 * window.innerWidth, 0.55 * window.innerHeight);
                        renderer.gammaInput = true;
                        renderer.gammaOutput = true;
                        renderer.shadowMap.enabled = true;
                        renderer.shadowMap.renderReverseSided = false;
                        container.appendChild(renderer.domElement);

                        //var marq = document.createElement('marquee')
                        //marq.className = 'marquee';
                        //marq.innerHTML = 'Для остановки / повторного запуска анимации нажмите "Space"'
                        //container.appendChild(marq);
                        // stats
                        stats = new Stats();
                        //container.appendChild(stats.dom);
                        //
                        window.addEventListener('resize', onWindowResize, false);
                        window.addEventListener('keydown', onSpace, false);
                        container = document.getElementById("canvas3D");
                        container.addEventListener('wheel', onWheel, false );
                        controls = new THREE.TrackballControls( camera );
                        controls.rotateSpeed = 10.0;
                        controls.noZoom = true;
                        //controls.noPan = true;
                        controls.staticMoving = true;

                        animate();
                    },
                    function (xhr) {
                        if (xhr.lengthComputable) {
                            console.log((xhr.loaded / xhr.total * 100) + '% loaded');
                        }
                    },
                    function (xhr) {
                        console.error('An error happened', xhr);
                    }
                );
            }

            function addShadowedLight(x, y, z, color, intensity) {
                var directionalLight = new THREE.DirectionalLight(color, intensity);
                directionalLight.position.set(x, y, z);
                scene.add(directionalLight);
                directionalLight.castShadow = true;
                var d = 1;
                directionalLight.shadow.camera.left = -d;
                directionalLight.shadow.camera.right = d;
                directionalLight.shadow.camera.top = d;
                directionalLight.shadow.camera.bottom = -d;
                directionalLight.shadow.camera.near = 1;
                directionalLight.shadow.camera.far = 4;
                directionalLight.shadow.mapSize.width = 1024;
                directionalLight.shadow.mapSize.height = 1024;
                directionalLight.shadow.bias = -0.005;
            }

            function onWindowResize() {
                camera.aspect = window.innerWidth / window.innerHeight;
                renderer.setSize( 0.55 * window.innerWidth, 0.55 * window.innerHeight );
                camera.updateProjectionMatrix();
                controls.handleResize();
                //renderer.render( scene, camera );
            }

            function onWheel(e) {
                e.preventDefault();
                if (e.deltaY > 0) {
                    mScale += 0.1;
                } else {
                    mScale -= 0.1;
                }
                mesh.scale.set(mScale, mScale, mScale);
            }

            function onSpace(e){
                if (e.keyCode == 32) {
                    spaceCount++;
                    cancelAnimationFrame(aId);
                    (spaceCount % 2 == 0) ? animate() : animateWOrotation();
                }
                return;
            }

            function animate() {
                aId = requestAnimationFrame(animate);
                funcRotationRender();
                stats.update();
                controls.update();
            }

            function funcRotationRender() {
                //var timer = Date.now() * 0.0005;
                //camera.position.x = fitPropObject.dF * Math.cos(timer);
                //camera.position.y = 0;
                //camera.position.z = fitPropObject.dF * Math.sin(timer);
                mesh.rotation.y += 0.01;
                //camera.lookAt(cameraTarget);
                renderer.render(scene, camera);
            }

            function animateWOrotation(){
                aId = requestAnimationFrame(animateWOrotation);
                renderer.render(scene, camera);
                controls.update();
            }
        }
    }
})();
