(function () {
  "use strict";

  window.App.Views.CreateMaterialView = Marionette.LayoutView.extend({
    template: "./views/templates/project/material/create_material.html.ejs",
    className: "create-material row",

    regions: {
      mainRegion: ".js-main-region",
      treeRegion: ".js-tree-region"
    },

    ui: {
      name:         ".js-name",
      manufacture:  ".js-manufacture",
      source:       ".js-source",
      path:         ".js-path"
    },

    events: {
      "click @ui.path": "onPathClick"
    },

    initialize: function (options) {
      this.options = options || {};
      this.pid = options.treeNode && options.treeNode.id;
      this.path = options.treeNode && options.treeNode.path;
      this.model = new window.App.Models.Material();
      this.listenTo(this, "ok", this.onSubmit);
      this.listenTo(this, "cancel", this.onCancel);
    },

    onPathClick: function () {
      this.$(".js-main-region").addClass("hidden");
      this.md.$el.find(".btn").addClass('disabled');
      this.pathModalView = new window.App.Views.CreateModalPath({ treeType: "materials", onlyFolders: true });
      this.getRegion("treeRegion").show(this.pathModalView);
      this.listenTo(this.pathModalView, "path:select:canceled", this.pathSelectCanceled);
      this.listenTo(this.pathModalView, "path:select:done", this.pathSelectDone);
    },

    pathSelectCanceled: function () {
      this.$(".js-main-region").removeClass("hidden");
      this.md.$el.find(".btn").removeClass('disabled');
      this.getRegion("treeRegion").empty();
    },

    pathSelectDone: function (treeNode) {
      this.pathSelectCanceled();
      this.pid = treeNode.id;
      this.ui.path.text(window.App.Helpers.ShowPath(treeNode.path));
    },

    passModal: function (md) {
      this.md = md;
    },

    onCancel: function() {
      window.App.navigate("#project/materials", { trigger: true });
    },

    onSubmit: function(md) {
      md.preventClose();
      var name = this.ui.name.val();

      if (_.isEmpty(name))
        this.ui.name.closest(".form-group").addClass("has-error");
      else if (_.isNull(this.pathFormInp))
        this.ui.path.closest(".form-group").addClass("has-error");
      else {
        this.model.save({
          name: name,
          pid: parseInt(this.pid),
          manufacture: this.ui.manufacture.val(),
          source: this.ui.source.val()
        }, {
          success: function (model, resp) {
            window.App.vent.trigger('updateTreeNodes', {id: model.get("pid")}, 'materials', window.App.Data.EntityTypes[2].type, "add");
            window.App.navigate("#project/materials/" + model.get("id") + "/parameters/edit", { trigger: true });
            md.close();
          },
          error: function () {

          }
        });
      }
    },

    templateHelpers: function () {
      return {
        nodeId: this.options.treeNode ? this.options.treeNode.id : "",
        nodePath: this.options.treeNode ? this.options.treeNode.path : ""
      }
    }

  });

}());