(function () {
  "use strict";

  window.App.Views.MaterialPlotViewOld = Marionette.LayoutView.extend({
    template: "./views/templates/project/material/material_plot.html.ejs",
    className: "ui-material-plot",

    ui: {
      plotContainer:    ".js-plot",
      selectParameter:  ".js-select-parameter",
      table:            ".js-table",
      removeValue:      ".js-remove-value"
    },

    events: {
      "change @ui.selectParameter": "onSelectParameterChange",
      "click @ui.removeValue":      "onClickRemoveValue",
      "keyup .js-temperature":      "onInputValueChange",
      "keyup .js-value":            "onInputValueChange",
      "change .js-temperature":     "onInputValueChange",
      "change .js-value":           "onInputValueChange",
      "click .js-add-value":        "onAddValueBtnClick"
    },

    initialize: function (options) {
      this.options = options || {};
      this.currentParameterId = this.collection.at(0).id;
    },

    onRender: function() {},

    onShow: function () {
      this.updateTableAndPlot();
    },

    generatePlot: function () {
      var chart = c3.generate({
        bindto: ".js-plot",
        size: {
          height: 340,
          width: $(".js-plot").width()
        },
        data: {
          x: "T",
          columns: this.mapPlotData()
        }
      });
    },

    mapPlotData: function () {
      var parameterModel = this.collection.findWhere({ id: this.currentParameterId });
      var valuesList = parameterModel.get("valuesList");
      var columns = [
        ["T"],
        [parameterModel.get("name")]
      ];
      _.each(valuesList, function(v) {
        columns[0].push(v.temperature.toFixed(2));
        columns[1].push(v.paramValue.toFixed(2));
      });
      return columns;
    },

    onSelectParameterChange: function () {
      var parameterId = parseInt(this.ui.selectParameter.val());
      this.currentParameterId = this.collection.findWhere({ id: parameterId }).id;
      this.updateTableAndPlot();
    },

    renderParametersTable: function () {
      var tableTemplate = new EJS({ url: "./views/templates/project/material/material_parameter_table.html.ejs" }).render({ parameterValues: this.mapPlotData() });
      this.ui.table.empty();
      this.ui.table.append(tableTemplate);
    },

    updateTableAndPlot: function () {
      this.generatePlot();
      this.renderParametersTable();
    },

    onClickRemoveValue: function (e) {
      var that = this;
      var index = $(e.target).closest("tr").data("index");
      var parameterModel = this.collection.findWhere({ id: this.currentParameterId });
      parameterModel.get("valuesList").splice(index, 1);
      parameterModel.save({}, {
        success: function (model, response) {
          that.updateTableAndPlot();
        }
      });
    },

    onInputValueChange: function () {
      if (!_.isEmpty(this.$(".js-temperature").val()) && !_.isEmpty(this.$(".js-value").val()))
        this.$(".js-add-value").removeClass("disabled");
      else
        this.$(".js-add-value").addClass("disabled");
    },

    onAddValueBtnClick: function () {
      var that = this;
      var parameterModel = this.collection.findWhere({ id: this.currentParameterId });
      parameterModel.get("valuesList").push({
        temperature: parseFloat(this.$(".js-temperature").val()),
        paramValue: parseFloat(this.$(".js-value").val())
      });
      parameterModel.save({}, {
        success: function () {
          that.updateTableAndPlot();
        }
      });
    },

    templateHelpers: function() {
      return {
        parameters: this.collection.map(function (m) { return { name: m.get("name"), id: m.get("id") } })
      }
    }

  });

}());