(function () {
  "use strict";

  window.App.Views.BaseRunModal = Backbone.Marionette.LayoutView.extend({
    template: "./views/templates/project/modals/base_run_modal.html.ejs",
    className: "ui-base-run-modal",

    behaviors: {
      PluginsInit: {
        select2Events: [],
        select2: [
          new window.App.Helpers.Select2Config({
            el: ".js-solver",
            placeholder: "Выберите расчетный пакет",
            url: "rest/solvers/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          }),
          new window.App.Helpers.Select2Config({
            el: ".js-node",
            placeholder: "Выберите расчетный узел",
            url: "rest/nodes/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          })
        ]
      }
    },

    ui: {
      "solver": ".js-solver",
      "node": ".js-node",
      "cores": ".js-cores",
      "memory": ".js-memory",
      'cluster': '.js-cluster',
      'numOfNodes': '.js-num-of-nodes',
      'isParallel': '.js-dis'
    },

    initialize: function (options) {
      this.options = options || {};
      this.clusterType = null;
      this.listenTo(this, "ok", this.onSubmit);
      if (this.options.template) this.template = this.options.template;
    },

    passModal: function (md) {
      this.md = md;
    },

    toggleFieldDisable: function (field, disable) {
      this.ui[field].prop('disabled', disable);
      this.ui[field].closest(".form-group").removeClass("has-error");
    },

    toggleFieldHide: function (field, hide) {
      this.ui[field].closest(".form-group").css('display', hide?'none':'block').removeClass("has-error");
    },

    onSubmit: function (md) {
      md.preventClose();
      var task = new window.App.Models.Task();
      var solver = this.ui.solver.val();
      var nodeId = this.ui.node.val();
      var cores = this.ui.cores.val();
      var memory = this.ui.memory.val();
      if (!solver) {
        this.ui.solver.closest(".form-group").addClass("has-error");
        return false;
      } else {
        this.ui.solver.closest(".form-group").removeClass("has-error");
      }
      if (!nodeId) {
        this.ui.node.closest(".form-group").addClass("has-error");
        return false;
      } else {
        this.ui.node.closest(".form-group").removeClass("has-error");
      }
      task.save({
        "solverName": solver,
        "nodeId": parseInt(nodeId),
        "numOfCores": cores,
        "memory": memory,
        "parentName": this.options.parentName,
        "parentType": this.options.parentType,
        "parentId": this.options.parentId
      }, {
        success: function () {
          window.alertify.success("Задача успешно создана");
          md.close();
          window.App.navigate("project/queue", {trigger: true});
        }
      });
    }

  });

})();