(function () {
  "use strict";

  window.App.Views.ChangeReferenceModal = Marionette.LayoutView.extend({
    template: "./views/templates/project/modals/change_reference_modal.html.ejs",
    regions: {
      mainRegion: ".js-main-region",
      treeRegion: ".js-tree-region"
    },

    ui: {
      path: ".js-path",
      reference: ".js-reference"
    },

    events: {
      "click @ui.path": "onPathClick"
    },

    behaviors: {
      PluginsInit: {
        select2Events: [],
        select2: [
          new window.App.Helpers.Select2Config({
            el: ".js-reference",
            placeholder: "Выберите объект",
            url: "rest/optimizations/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          })
        ]
      }
    },

    initialize: function () {
      this.listenTo(this, "ok", this.onSubmit);
    },

    onPathClick: function () {
      this.$(".js-main-region").addClass("hidden");
      this.pathModalView = new window.App.Views.CreateModalPath({ treeType: "optimizations", onlyFolders: false });
      this.md.$el.find(".btn").addClass('disabled');
      this.getRegion("treeRegion").show(this.pathModalView);
      this.listenTo(this.pathModalView, "path:select:canceled", this.pathSelectCanceled);
      this.listenTo(this.pathModalView, "path:select:done", this.pathSelectDone);
    },

    pathSelectCanceled: function () {
      this.$(".js-main-region").removeClass("hidden");
      this.getRegion("treeRegion").empty();
    },

    pathSelectDone: function (treeNode) {
      this.pathSelectCanceled();
      this.pid = treeNode.id;
      this.ui.path.text(window.App.Helpers.ShowPath(treeNode.path));
      this.initReferenceSuggest(treeNode.pathNames)
      this.md.$el.find(".btn").removeClass('disabled');
    },

    passModal: function (md) {
      this.md = md;
    },

    onSubmit: function (md) {
      md.preventClose();
      var referenceId = this.ui.reference.val();
      if (_.isNull(referenceId)) {
        window.alertify.error("Вы должны выбрать конкретный объект.")
      } else {
        this.trigger("reference:selected", referenceId);
        md.close();
      }
    },

    initReferenceSuggest: function (pathNames) {
      this.ui.reference.select2(
        new window.App.Helpers.Select2Config({
          el: ".js-equipment",
          placeholder: "Выберите объект",
          url: "rest/optimizations/suggest",
          fieldName: "pathNames",
          term: pathNames,
          isNumber: false,
          allowClear: true,
          minimumInputLength: 0
        })
      );
    }
    
  });


})();