window.OptimizationToscaTechnologicalConstraint = React.createClass({
  getInitialState: function () {
    return {
      data: this.props.data,
      fields: {},
      tree: []
    }
  },

  onChange: function (e) {
    let options = { name: e.target.id, value: e.target.value != -1 ? parseInt(e.target.value) : null };
    this.saveInputChanges(options);
  },

  saveInputChanges: function (options) {
    let updatedState = this.state.data;
    updatedState[options.name] = options.value;
    $.ajax({
      type: "PUT",
      url: "rest/optimizations/" + this.state.data.optimizationId + "/Tosca/technologicalConstraints/" + this.state.data.id,
      contentType: "application/json",
      data: JSON.stringify(updatedState),
      success: (state) => {
        if (options.name == "technologicalConstraintId") {
          this.updateFields();
        }
        this.setState({ data: state });
      }
    });
  },

  componentDidMount: function () {
    this.updateFields();
  },

  updateFields: function () {
    this.fetchFieldsOptimization().then((fields) => {

      this.fetchFieldsDictionary().then((tree) => {
        var mappedTree = this.mapTree(tree, fields);
        if (!_.isEmpty(mappedTree.toDelete)) {
          this.deleteArrayOfNodes(mappedTree.toDelete);
        } else {
          this.setState({ tree: mappedTree.tree, fields: fields });
        }
      });
    });
  },

  fetchFieldsOptimization: function () {
    return new Promise((resolve, reject) => {
      $.get("rest/optimizations/" + this.state.data.optimizationId + "/Tosca/technologicalConstraints/" + this.state.data.id + "/fields", resolve);
    });
  },

  fetchFieldsDictionary: function () {
    return new Promise((resolve, reject) => {
      $.get("rest/optimizationType/" + this.state.data.optimizationTypeId + "/Tosca/technologicalConstraints/" + this.state.data.technologicalConstraintId + "/tree", resolve);
    });
  },

  mapParameters: function () {
    return _.map(this.props.technologicalConstraintsDictionary, (technologicalConstraintInDictionary) => {
      return <option key={technologicalConstraintInDictionary.id}
                     value={technologicalConstraintInDictionary.id}>{technologicalConstraintInDictionary.name}</option>
    });
  },

  checkIfNodeIsDisabiled: function (node, fields) {
    var isDisabled;
    if (!_.isNull(node.dependencies)) {
      switch (node.dependencies.condition) {
        case "NONE_SELECTED":
          _.each(node.dependencies.nodes, (dependencyId) => {
            if (!_.isUndefined(fields[dependencyId])) {
              isDisabled = true;
            }
          });
          break;
        case "ANY_SELECTED":
          isDisabled = true;
          _.each(node.dependencies.nodes, (dependencyId) => {
            if (!_.isUndefined(fields[dependencyId])) {
              isDisabled = false;
            }
          });
          break;
        case "ALL_SELECTED":
          _.each(node.dependencies.nodes, (dependencyId) => {
            if (_.isUndefined(fields[dependencyId])) {
              isDisabled = true;
            }
          });
          break;
      }
      return isDisabled;
    } else {
      return false;
    }
  },

  deleteArrayOfNodes: function (ids) {
    $.ajax({
      type: "DELETE",
      url: "rest/optimizations/" + this.state.data.optimizationId + "/Tosca/technologicalConstraints/" + this.state.data.id + "/fields?ids=" + ids.join(","),
      success: this.updateFields
    })
  },

  mapTree: function (nodes, fields) {
    var mappedTree = {
      tree: [],
      toDelete: []
    };
    mappedTree.tree = nodes.map((node, index) => {
      node.isDisabled = this.checkIfNodeIsDisabiled(node, fields);

      if (!_.isUndefined(fields[node.id])) {
        if (node.isDisabled) {
          mappedTree.toDelete.push(node.id);
        }
        node.isSelected = true;
        node.value = fields[node.id].value;
      }

      if (!_.isEmpty(node.children)) {
        var groupedChildren = _.groupBy(node.children, "type");
        node.children = [];

        _.each(groupedChildren, (values, key) => {
          if (key === "select") {
            node.selectedId = null;
            _.each(values, (child, index) => {
              child.isDisabled = this.checkIfNodeIsDisabiled(child, fields);

              if (!_.isUndefined(fields[child.id])) {
                if (!child.isDisabled) {
                  node.selectedId = child.id;

                  if (!_.isEmpty(child.children)) {
                    var childrenTree = this.mapTree([child], fields);

                    child.type = null;

                    mappedTree.toDelete = mappedTree.toDelete.concat(childrenTree.toDelete);
                    node.children = node.children.concat(childrenTree.tree)
                  }

                } else {
                  mappedTree.toDelete.push(child.id);
                }
              }
            });
            node.type = "list";
            node.selectableValues = values.filter((value) => !value.isDisabled);
            if (node.id == 174) console.log(_.pluck(node.selectableValues, "displayName"))
          } else {
            var childrenTree = this.mapTree(values, fields);
            mappedTree.toDelete = mappedTree.toDelete.concat(childrenTree.toDelete);
            node.children = node.children.concat(childrenTree.tree);
          }
        });
      }
      return node;
    });
    return mappedTree;
  },

  render: function () {
    return (
      <div className="ui-optimization-row-container clearfix">
        <div className="clearfix" style={{"marginBottom": "15px"}}>
          <div className="col-md-6">
            <div className={"form-group"}>
              <select id="setId" disabled={this.props.isView} className="form-control"
                      value={this.state.data.setId || -1} placeholder="Не задано" onChange={this.onChange}>
                <option value={-1}>Не задано</option>
                {this.props.sets.toJSON().map((setItem) => {
                  return <option key={setItem.id} value={setItem.id}>{setItem.name}</option>
                })}
              </select>
            </div>
          </div>
          <div className="col-md-6">
            <div className={"form-group"}>
              <select id="technologicalConstraintId" disabled={this.props.isView} className="form-control"
                      value={this.state.data.technologicalConstraintId || undefined} onChange={this.onChange}>
                {this.mapParameters()}
              </select>
            </div>
          </div>
        </div>
        {!_.isEmpty(this.state.tree) ?
        <window.OptimizationTechnologicalConstraintsFields isView={this.props.isView}
                                                                onSelectUpdate={this.updateFields}
                                                                optimizationFieldsURL={"rest/optimizations/" + this.state.data.optimizationId + "/Tosca/technologicalConstraints/" + this.state.data.id + "/fields"}
                                                                tree={this.state.tree} fields={this.state.fields}
                                                                data={this.state.data}/> : ""}
      </div>
    )
  }
});
