(function () {
  "use strict";

  window.App.Views.OptimizationDependenciesView = Marionette.LayoutView.extend({
    template: "./views/templates/project/optimization/optimization_dependencies.html.ejs",
    className: "ui-optimization-setting-files row",
    regions: {
      materialsRegion: ".js-materials-region",
      technologiesRegion: ".js-technologies-region",
      loadConditionsRegion: ".js-loadConditions-region"
    },
    onRender: function () {
      this.showTables();
    },

    showTables: function () {
      this.showTable(new window.App.Views.MaterialsListView(_.extend({
        gridCollection: new App.Collections.OptimizationMaterials([], {
          pageable: true,
          filters: null,
          optimizationId: this.model.id
        }),
        showTools: false,
        actions: ["add-to-optimization", "remove-from-optimization"],
        type: "materials"
      }, this.options || {})), this.getRegion("materialsRegion"));

      this.showTable(new window.App.Views.TechnologiesListView(_.extend({
        gridCollection: new App.Collections.OptimizationTechnologies([], {
          pageable: true,
          filters: null,
          optimizationId: this.model.id
        }),
        actions: ["add-to-optimization", "remove-from-optimization"],
        type: "technologies",
        showTools: false
      }, this.options)), this.getRegion("technologiesRegion"));

      var loadConditionsCollection = new window.App.Collections.Files([], {
        objectType: "optimizations",
        objectId: this.model.id,
        optimizationId: this.model.id,
        fileType: "loadConditions",
        pageable: true,
        filters: this.options.filters || null
      });
      loadConditionsCollection.columnsUrl = function () {
        return "rest/optimizations/dependencies/columns";
      };

      this.showTable(new window.App.Views.FilesListView(_.extend({
        gridCollection: loadConditionsCollection,
        actions: ["new", "remove-from-optimization", "export"],
        isFile: true,
        title: "Условия Нагр.",
        showTools: false
      }, {})), this.getRegion("loadConditionsRegion"));
    },

    showTable: function (view, region) {
      region.show(view);
    }
  });


})();