(function () {
  "use strict";

  window.App.Views.QueueStatisticBar = Backbone.Marionette.ItemView.extend({
    template: "./views/templates/project/queue/queue_statistic_bar.html.ejs",
    className: "ui-statistics-chart-bar-view",

    ui: {
      info: ".js-statistic-bar-info"
    },

    events: {
      "click @ui.info": "onInfoClick"
    },

    initialize: function () {
      this.currentInfoBy = "nodes";
      this.stacks = this.model.get("total")[this.currentInfoBy];
      this.stacksNum = this.getStacksNum();
      this.stacksInPerCent = this.getStacksAfterRoundingAndBalancing();
      this.listenTo(this.model, "change", this.render);
    },

    getStacksNum: function () {
      var stacksNum = 0;
      _.each(this.stacks, function (value, key) {
        stacksNum = stacksNum + value;
      });
      return stacksNum;
    },

    getStacksInPerCent: function () {
      var that = this;
      var stacksInPerCent = {};
      _.each(this.stacks, function (value, key) {
        stacksInPerCent[key] = 100 / that.stacksNum * value;
      });
      return stacksInPerCent;
    },

    getStacksAfterRoundingAndBalancing: function () {
      var minStackWidth = 5;
      var stacksInPerCent = this.getStacksInPerCent();
      var roundingError = 0;
      _.each(stacksInPerCent, function (value, key) {
        if (value > 0 && value < minStackWidth) {
          stacksInPerCent[key] = minStackWidth;
          roundingError = roundingError + (minStackWidth - value);
        }
      });
      var stackWithMaxValue = this.getStackWithMaxValue(stacksInPerCent);
      stacksInPerCent[stackWithMaxValue.key] = stackWithMaxValue.value - roundingError;
      return stacksInPerCent;
    },

    getStackWithMaxValue: function (stacksInPerCent) {
      var max = {key: null, value: -1};
      _.each(stacksInPerCent, function (value, key) {
        if (value > max.value) {
          max = {key: key, value: value}
        }
      });
      return max;
    },

    onInfoClick: function () {
      window.App.navigate("project/queue/clusters/" + this.model.id + "/nodes", { trigger: true });
    },

    templateHelpers: function () {
      var that = this;
      return {
        stacksNum: this.stacksNum,
        countWidth: function (stackName) {
          return that.stacksInPerCent[stackName];
        },
        countNum: function (stackName) {
          return that.stacks[stackName];
        }
      }
    }

  });

})();