(function () {
  "use strict";

  window.App.Views.QueueTableWrap = window.App.Views.CollapsableBlock.extend({
    template: "./views/templates/project/queue/queue_table_wrap.html.ejs",
    regions: {
      tableRegion: ".js-table-region"
    },

    ui: _.extend({
      "mine": ".js-mine"
    }, window.App.Views.CollapsableBlock.prototype.ui),

    events: _.extend({
      "click @ui.mine": "onMineClick"
    }, window.App.Views.CollapsableBlock.prototype.events),

    initialize: function (options) {
      this.options = options || {};
      this.mine = false;
    },

    onRender: function () {
      this.showTable()
    },

    onMineClick: function () {
      this.mine = !this.mine;
      this.ui.mine.removeClass("btn-default btn-info active");
      var className = this.mine ? "btn-info active" : "btn-default";
      this.ui.mine.addClass(className);

      var filters = this.view.gridView.BackGrid.collection.getQueryParams(['filters']);
      filters.ownedOnly = this.mine;
      this.view.gridView.BackGrid.collection.setQueryParams({ filters: filters });
      this.view.gridView.BackGrid.collection.fetch();
    },

    showTable: function () {
      this.collection = new App.Collections.Tasks([], {
        pageable: true,
        filters: { ownedOnly: this.mine } || null,
        type: this.options.tableName
      });
      this.view = new window.App.Views.QueueTasks(_.extend({
        gridCollection: this.collection,
        actions: this.mapActions(this.options.tableName),
        type: "task",
        tableName: this.options.tableName,
        title: this.options.tableName.capitalize(),
        draggableRows: this.options.tableName == "queue"
      }, {}));
      this.getRegion('tableRegion').show(this.view);
    },

    mapActions: function (tableName) {
      var actions = [];
      switch (tableName) {
        case "queue": actions = actions.concat(["cancel_task"]); break;
        case "running": actions = actions.concat(["stop_task"]); break;
        default: break;
      }
      return actions;
    },

    templateHelpers: function () {
      return {
        title: this.options.tableName.capitalize()
      }
    }
  });

}());