(function () {
  "use strict";
  window.App.Views.SimulationParametersPage = window.App.Views.BasePage.extend({
    template: "./views/templates/project/simulation/simulation_parameters_page.html.ejs",

    regions: _.extend({
      "parametersRegion": ".js-parameters-region",
      "experimentRegion": ".js-experiment-region"
    }, window.App.Views.BasePage.prototype.regions),

    ui: _.extend({
      "results": ".js-results",
      "report": ".js-report"
    }, window.App.Views.BasePage.prototype.ui),

    events: _.extend({
      "click @ui.results": "onResultsClick",
      "click @ui.report": "onReportClick"
    }, window.App.Views.BasePage.prototype.events),

    initialize: function (options) {
      window.App.Views.BasePage.prototype.initialize.apply(this, arguments);
      this.controls = ["edit", "run"];
      const FINISHED_STATUS_ID = 4;
      if (this.model.get("statusId") === FINISHED_STATUS_ID) this.controls.push("report");
      if (this.model.get("resultsCount") !== 0) this.controls.push("results");
    },

    onRender: function () {
      this.initAndShowMainView(this.getRegion("mainRegion"));
      this.initAndShowParametersView(this.getRegion("parametersRegion"));
      this.initAndShowExpirementView(this.getRegion("experimentRegion"));
      window.App.Views.BasePage.prototype.onRender.call(this);
    },

    initAndShowMainView: function (region) {
      this.mainView = new window.App.Views.SimulationMainView({
        layoutView: this,
        mode: this.mode,
        model: this.model
      });
      region.show(this.mainView);
    },

    initAndShowControlsView: function (region) {
      this.controlsView = new window.App.Views.SimulationControlsView({
        layoutView: this,
        title: this.model.get("name"),
        controls: this.controls,
        mode: this.mode,
        model: this.model
      });
      region.show(this.controlsView);
    },


    initAndShowExpirementView: function (region) {
      this.expirementView = new window.App.Views.SimulationExperimentView({
        layoutView: this,
        mode: this.mode,
        model: this.model
      });
      region.show(this.expirementView);
    },

    initAndShowParametersView: function (region) {
      this.parametersView = new window.App.Views.SimulationParametersView({
        layoutView: this,
        mode: this.mode,
        model: this.model
      });
      region.show(this.parametersView);
    },

    onSaveClick: function () {
      var that = this;
      this.expirementView.trigger("save:experiment:file");
      this.model.save(null, {
        success: function () {
          window.App.navigate("#project/" + that.options.modelName + "/" + that.model.id + "/" + that.page + "/view", { trigger: true });
        },
        error: function (xhr) {
          console.error("Failed to save: ", xhr);
        }
      });
    },

    onResultsClick: function () {
      window.App.navigate("project/simulations/" + this.model.id + "/results/view", { trigger: true });
    },

    onReportClick: function () {
      window.open("rest/simulations/" + this.model.id + "/report");
    },

    onRunClick: function () {
      this.runModalView = new window.App.Views.BaseRunModal({
        "parentType": "simulation",
        "parentName": this.model.get("name"),
        "parentId": this.model.id
      });
      this.runModal = new Backbone.BootstrapModal({
        content: this.runModalView,
        title: "Запуск расчета",
        animate: true,
        cancelText: "Отменить",
        okText: "Запустить"
      }).open();
      this.runModalView.passModal(this.runModal);
    }

  });

})();