(function () {
  "use strict";

  var AddRibView = Marionette.LayoutView.extend({
    template: "./views/templates/project/technology/technology_simulation_add_rib.html.ejs",
    ui: {
      position:   ".js-position",
      origin_x:   ".js-originX",
      origin_y:   ".js-originY",
      angle:  ".js-angle",
      steps:      ".js-step"
    },

    initialize: function () {
      this.listenTo(this, "ok", this.onSubmit);
    },

    onSubmit: function (md) {
      md.preventClose();
      var isValid = true;
      _.each(this.ui, function($item) {
        if (_.isEmpty($item.val())) {
          isValid = false;
          $item.closest(".form-group").removeClass("has-success").addClass("has-error");
        } else {
          $item.closest(".form-group").removeClass("has-error").addClass("has-success");
        }
      });
      if (isValid) {
        this.trigger("rib:added", {
          position: parseInt(this.ui.position.val()) - 1,
          origin_x: parseFloat(this.ui.origin_x.val()),
          origin_y: parseFloat(this.ui.origin_y.val()),
          angle: parseFloat(this.ui.angle.val()),
          steps: this.ui.steps.val()
        });
        md.close();
      }
    }

  });

  window.App.Views.TechnologySimulationRibsView = Marionette.LayoutView.extend({
    template: "./views/templates/project/technology/technology_simulation_ribs.html.ejs",
    className: "row",
    ui: {
      addRib:               ".js-add-rib",
      remove:                 ".js-remove"
    },

    events: {
      "click @ui.addRib":               "onAddRibClick",
      "click @ui.remove":                 "onRemoveClick"
    },

    initialize: function () {
      this.listenTo(this.collection, "sync", this.render);
    },

    onShow: function () {
      this.$(".ui-sortable-ribs").sortable({ axis: "x" });
    },

    onAddRibClick: function () {
      this.addRibView = new AddRibView();
      this.modal = new Backbone.BootstrapModal({
        content: this.addRibView,
        title: "Новое ребро",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.listenTo(this.addRibView, "rib:added", this.addRib);
    },

    addRib: function (options) {
      var that = this;
      var newRib = new window.App.Models.RibParameter({ technologySimulationId: that.options.technologySimulationId });
      newRib.set(options);
      newRib.save({}, {
        success: function () {
          that.collection.fetch();
        },
        error: function (model, xhr) {
          if (xhr.status == 200) that.collection.fetch();
        }
      });
    },

    onRemoveClick: function (e) {
      var that = this;
      var id = $(e.target).data("id");
      var model = this.collection.findWhere({ id: id });
      model.destroy({
        dataType: "text",
        success: function () {
          that.collection.fetch();
        },
        error: function (xhr) {
          if (xhr.status == 200) that.collection.fetch();
        }
      });
    },

    templateHelpers: function () {
      return {
        ribs: this.collection.toJSON()
      }
    }
  });

}());