/**
 *
 * Sidebar View
 *
 */

(function () {
    "use strict";

    window.App.Views.zTreeBase = Backbone.Marionette.LayoutView.extend({
        template: false,

        className: 'ztree',

        attributes: {
            "data-toggle": "context",
            "data-target": "#zTree-contextmenu"
        },
        regions: {},

        events: {},

        ui: {},

        settings: function () {
            var that = this;
            that.doubleClickExpandFlag = false;

            return {
                treeNode: {
                    iconSkin: 'testestewerwew'
                },
                edit: {
                    enable: true,
                    showRemoveBtn: false,
                    showRenameBtn: false
                },
                callback: {
                    beforeRename: that.beforeRename(),
                    onClick: that.onClickTreeNode(),
                    beforeClick: that.onBeforeClickNode(),
                    onCollapse: that.onCollapseTreeNode(),
                    onRightClick: that.onRightClickTree(),
                    onExpand: that.onExpandNode,
                    onAsyncSuccess: that.triggerOnAsyncSuccess()
                },
                view: {
                    dblClickExpand: that.onDblClickExpand(),
                    showLine: true,
                    showTitle: true,
                    selectedMulti: false,
                    fontCss: function (treeId, treeNode) {
                        return {color:treeNode.color}
                    }
                },

                data: {
                    key: {
                        name: 'name',
                        title:'altText'
                    },
                    simpleData: {
                        enable: true,
                        idKey: "id",
                        pIdKey: "pid",
                        rootPId: 0
                    }
                },

                async: {
                    enable: true,
                    url: that.model.url(),
                    type: "GET",
                    autoParam: ['id=id'], //set attr mapping (BUT! CML uses realId instead of id)
                    dataFilter: function (treeId, parentNode, responseData) {
                        responseData = _.map(responseData, function (o, i) {
                            o.iconSkin = o.itemType.toLocaleLowerCase();
                            return o;
                        });

                        that.doubleClickExpandFlag = false;
                        return responseData;
                    }
                }
            }
        },

        initialize: function (options) {
            _.bind(this.expandNodeByData, this);

            this.spinner = new App.Utils.Spinner();
            this.options = options || {};

            this.settings = _.extend(this.settings(), options.treeConfig || {});

            this.listenTo(this.model, 'request', this.spinner.show.bind(this.spinner));
            this.listenTo(this.model, 'sync', this.spinner.hide.bind(this.spinner));
        },

        initListeners: function () {
            _.bindAll(this, 'onCancelSelectNodes');
            this.listenTo(this, "ztree:expand:all", this.expandAll);
            this._zTreeChannel = Backbone.Wreqr.radio.channel('ztree')
            this._zTreeChannel.vent.on('ztree:cancel:selectnodes', this.onCancelSelectNodes);
        },

        fetchData: function () {
            var that = this;
            this.model.fetch().done(function () {
                that.buildTree();
            });
        },

        onShow: function () {
            //this.fetchData();
            this.buildTree();
        },

        buildTree: function () {
            this.initListeners();

            $.fn.zTree.getRootNode = function (treeNode, itemType) {
                if (_.isEmpty(treeNode)) {
                    return null;
                }
                if (!treeNode.parent && !treeNode.getParentNode()) {
                    return treeNode;
                } else if (itemType && treeNode.itemType === itemType) {
                    return treeNode;
                } else {
                    if (!treeNode.getParentNode()) {
                        return treeNode
                    }
                    return $.fn.zTree.getRootNode(treeNode.getParentNode(), itemType);
                }
            };

            $.fn.zTree.getTreeLevelData = function (treeId, treeNode, itemType) {
                var levelData = {};

                if (!treeNode) {
                    console.warn('Node data is not passed!');
                    return levelData;
                }

                function getLevelData(treeNode, itemType) {

                    if (treeNode) {
                        var levelType = treeNode.itemType;
                        levelData.id = parseInt(treeNode.id);
                        levelData.typeRoot = itemType;
                        levelData.typeRootData = treeNode;
                        if (levelType === itemType) {
                            console.info("levelType === itemType", levelType);
                            return;
                        }

                        getLevelData(treeNode.getParentNode(), itemType);
                    }
                }

                getLevelData(treeNode, itemType);
                levelData.root = $.fn.zTree.getRootNode(treeNode);
                return levelData;
            };

            this.zTreeObj = $.fn.zTree.init(this.$el, this.settings);

            this.initZtreeEvents();

            return this.zTreeObj;
        },


        initZtreeEvents: function () {
            var that = this;
            //todo do not refresh all when add new project
            window.App.vent.on('updateTreeNodes', function (treeNode, treeId, nodeType, actionType, origin, childNodeId, parentNode) {
                var isSilent = false;
                var nodeToUpdate = null,
                    zTreeObj = $.fn.zTree.getZTreeObj(that.options.id),
                    _treeId = zTreeObj.setting.treeId;

                if (treeNode) {
                    nodeToUpdate = zTreeObj.getNodeByParam('id', treeNode.id); //INFO find node from each of trees
                    switch (actionType) {
                        case  "add":
                            if (nodeToUpdate) {
                                nodeToUpdate.isParent = true;
                                nodeToUpdate.childrenTypes = nodeToUpdate.childrenTypes || [];
                                nodeToUpdate.childrenTypes.push(nodeType);
                                nodeToUpdate.childrenTypes = _.uniq(nodeToUpdate.childrenTypes);
                            } else {
                                break;
                            }

                            //do not expand collapsed node of other tree
                            if (nodeToUpdate && nodeToUpdate.open === false && zTreeObj.setting.treeId !== treeId) {
                                isSilent = true;
                            }
                            break;
                        case "remove":
                            var nodeToRemove = nodeToUpdate;
                            var parentNodeToUpdate = zTreeObj.getNodeByParam('id', parentNode && parentNode.id);

                            var updateChildrenTypes = function (parentNodeToUpdate) {
                                parentNodeToUpdate.childrenTypes = parentNode.childrenTypes || [];

                                if (treeId === _treeId) {
                                    parentNodeToUpdate.isParent = parentNode.isParent;
                                }
                            };

                            if (nodeToRemove) {
                                zTreeObj.removeNode(nodeToRemove);
                                isSilent = false;
                                parentNodeToUpdate ? updateChildrenTypes(parentNodeToUpdate) : null;
                            } else {
                                parentNodeToUpdate ? updateChildrenTypes(parentNodeToUpdate) : null;
                            }
                            zTreeObj.updateNode(parentNodeToUpdate);
                            nodeToUpdate = parentNodeToUpdate;
                            break;
                        case "update":
                            var nodesToUpdate = zTreeObj.getNodesByParam('id', treeNode.id);
                            _.map(nodesToUpdate, function (n, k) {
                                n.name = treeNode.name;
                                zTreeObj.updateNode(n);
                            });
                            return;
                    }
                }

                if (!nodeToUpdate && (nodeType && nodeType !== window.App.Data.EntityTypes[0].type)) {
                    return;
                }

                return zTreeObj.reAsyncChildNodes(nodeToUpdate, "refresh", isSilent);
            });

            this.listenToOnce(this, "ztree:expand:bydata", this.expandNodeByData);
        },

        triggerOnAsyncSuccess: function () {
            var that = this;

            return function (e, treeId, treeNode, msg) {
                that.trigger('ztree:async:success', e, treeId, treeNode, msg);
            }

        },

        expandNodeByData: function (options) {
            this.options.expandData = options.expandData || {};
            if (_.isUndefined(this.options.expandData) || _.isEmpty(this.options.expandData)) {
                return;
            }

            var that = this;
            var projectNode = that.zTreeObj.getNodeByParam('id', this.options.expandData.parentProjectId);

            var expandDataNodeFilter = function (node) {
                return node.itemType === "CalculationType" && parseInt(node.id) === parseInt(that.options.expandData.calculationTypeTopLevelId);
            };

            that.setAsyncUrlParams();
            that.zTreeObj.reAsyncChildNodes(projectNode, 'refresh', false)

            var expandNode = this.zTreeObj.getNodesByFilter(expandDataNodeFilter, true, projectNode);

            if (!expandNode) {
                console.warn('Node not found! ExpandData: ', that.options.expandData);
                return;
            }
            //expand parent if target node has no children
            if (expandNode && expandNode.children) {
                this.zTreeObj.expandNode(expandNode, true, false, true);
            } else {
                this.zTreeObj.expandNode(expandNode.getParentNode(), true, false, true);
            }

        },

        //initZtreeOptions: function () {
        //  //Expand node on data from options.expandData
        //  this.expandNodeByData();
        //},
        getSelected: function () {
            return this.zTreeObj.getSelectedNodes()[0];
        },

        //INFO разворачивание всех рутов дерева
        expandAll: function () {
            this.zTreeObj.expandAll(true);
        },

        redrawTree: function () {

        },

        beforeRename: function () {
            return function (treeId, treeNode) {
            }
        },
        onShowRenameBtn: function (treeId, treeNode) {
            return !treeNode.isFirstNode;
        },
        onShowRemoveBtn: function (treeId, treeNode) {
            return !treeNode.isLastNode
        },

        onCollapseTreeNode: function () {
            var that = this;

            return (function (event, treeId, treeNode) {
                that.zTreeObj.expandNode(treeNode, false, true, true, false);
            })
        },

        onBeforeClickNode: function () {
            var that = this;
            return (function (treeId, treeNode) {

                if (!that.options.allowedType) {
                    return true;
                }
                return treeNode.itemType === that.options.allowedType;
            });
        },

        onCancelSelectNodes: function (treeId) {
            if (typeof treeId !== 'undefined' && this.zTreeObj.setting.treeId !== treeId) {
                return this.zTreeObj.cancelSelectedNode();
            }

        },

        onClickTreeNode: function () {
            var that = this;
            return (function (event, treeId, treeNode) {
                that.trigger("ztree:onclick", event, treeId, treeNode);
                that._zTreeChannel.vent.trigger('ztree:cancel:selectnodes', treeId);
            });
        },

        //INFO разворачиваем все дерево по даблклику
        onDblClickExpand: function () {
            var that = this;
            return (function (treeId, treeNode) {
                that.setAsyncUrlParams(treeId, treeNode);
                that.doubleClickExpandFlag = true;
                that.zTreeObj.reAsyncChildNodes(treeNode, 'refresh', false);
            });
        },

        onExpandNode: function (event, treeId, treeNode) {
            var zTreeObj = $.fn.zTree.getZTreeObj(treeId);

            function expandNext(node) {
                if (!node.children || (node.children && node.children.length === 0 || node.children.length > 1)) {
                    console.info("ztree_base.js nothing to expand ");
                    return;
                }

                var child = node.children[0];
                zTreeObj.expandNode(child, true, false, true);
                expandNext(child);
            }

            expandNext(treeNode);
        },

        //http://www.ztree.me/v3/demo/en/super/rightClickMenu.html
        onRightClickTree: function () {
            var that = this;

            return function (e, treeId, treeNode) {
                that.createContextMenu($(e.target), treeId, treeNode, e)
            };
        },

        editNodeItem: function ($el, treeId, treeNode, e, str) {
            function sendChanges() {
                var requestData = {};
                requestData.id = Math.abs(treeNode.id); //info, realId - not neded
                requestData.itemId = Math.abs(treeNode.id);
                requestData.name = str;
                requestData.itemType = treeNode.itemType;
                $.ajax("rest/tree/" + treeId + "/" + treeNode.id, {
                    type: "PUT",
                    contentType: "application/json",
                    data: JSON.stringify(requestData),
                    success: function (res) {
                        treeNode.name = res.name;
                        window.App.vent.trigger('updateTreeNodes', treeNode, treeId, null, "update", null);
                        window.App.vent.trigger('apply:changes');
                    }
                });
            }

            switch (treeNode.itemType) {
                case 'Material':
                    window.App.navigate("project/" + window.App.Data.Trees[0].type + "/" + treeNode.id + "/show?name=" + str, {
                        params: {path: treeNode.path},
                        trigger: true
                    });
                    break;
                case 'Submodel':
                    window.App.navigate("project/submodel/" + treeNode.id + "/show?name=" + str, {
                        params: {path: treeNode.path},
                        trigger: true
                    });
                    break;
                case 'Multivariant':
                    window.App.navigate("project/multivariant/" + treeNode.id + "/show?name=" + str, {
                        params: {path: treeNode.path},
                        trigger: true
                    });
                    break;
                default:
                    sendChanges();
            }
        },

        addNewNodeItem: function ($el, treeId, treeNode, e, str) {
            var onSameLevel = $el.attr('data-samelevel'),
                typeToAdd = $el.attr('data-itemType'),
                newData = {
                    pid: null //top level - root
                };

            if (e) {
                if (treeNode) {
                    var filterData = $.fn.zTree.getTreeLevelData(treeId, treeNode);

                    if (onSameLevel === "true") {
                        filterData.pid = treeNode.pid === 0 ? null : treeNode.pid; //fixme - null if we have pid == 0
                    } else {
                        filterData.pid = treeNode.id;
                    }

                    newData = _.extend(newData, _.omit(filterData, "root"));

                    if (_.isEmpty(str) || _.isNull(str)) {
                        str = "";
                    }
                }
                newData.name = str;
                newData.itemType = typeToAdd;

                switch (typeToAdd) {
                    //TODO create MATERIAL
                    case window.App.Data.EntityTypes[2].type :
                        window.App.navigate("project/" + window.App.Data.Trees[0].type + "/create?name=" + str, {
                            params: {path: treeNode.path},
                            trigger: true
                        });
                        break;
                    //TODO LVMS-101
                    default:
                        createNewNode();
                }
            }


            function createNewNode() {
                $.ajax("rest/tree/" + treeId, {
                    type: "POST",
                    contentType: "application/json",
                    data: JSON.stringify(newData),
                    success: function (res, status) {
                        if (!_.isEmpty(treeNode) && onSameLevel === "true") {
                            window.App.vent.trigger('updateTreeNodes', treeNode.getParentNode(), treeId, typeToAdd, "add", onSameLevel);
                        } else {
                            if (_.isEmpty(treeNode)) {
                                treeNode = res;
                            }
                            window.App.vent.trigger('updateTreeNodes', treeNode, treeId, typeToAdd, "add", onSameLevel);
                        }
                    },
                    error: function () {
                    }
                });
            }
        },

        createContextMenu: function ($el, treeId, treeNode, e) {
            //disable contextmenu for non-sidebar trees
            var trees = ['materials', 'technologies', 'optimizations', 'constructionElements', 'compositeMaterials'];
            var contextDisabled = false;
            if (trees.indexOf(treeId) === -1) {
                console.warn('tree with id:  ' + treeId + ", not found");
                return
            }

            var adminRights = {
                canCreateBranches: true,
                canCreateObjects: true,
                canModify: true,
                canView: true
            };

            // for now all users have all rights in tree.
            var access = adminRights;

            if (!access || treeNode && (!access.canCreateBranches && !access.canCreateObjects && !access.canModify)) {
                console.warn('Invalid Access Rights - user can do nothing');
                return;
            }

            var that = this,
                nodeType = treeNode && treeNode.itemType,
                $menu = $("#zTree-contextmenu"),
                $actionsBlock = $menu.find('.actions.menu-block'),
                $parentBlock = $menu.find('.parent.menu-block'),
                $childBlock = $menu.find('.child.menu-block'),
                $divider = $menu.find('.divider');

            $actionsBlock.hide();
            $parentBlock.hide();
            $childBlock.hide();
            $divider.hide();

            $menu.find('li').remove(); //reset previous menu

            /**
             * Add Item to context review.
             * @param type
             * @param title
             * @param onSameLevel
             * @param action
             */
            function addMenuItem(type, title, onSameLevel, action) {
                var $menuItem = $('<li></li>'),
                    action = action || "add",
                    $menuItemLink = $('<a class="menu-item-' + action + '"></a>');

                $menuItemLink.attr('data-itemtype', type);
                $menuItemLink.attr('data-samelevel', onSameLevel);

                $menuItem.append($menuItemLink);

                if (action === "add") {
                    if (!access.canCreateBranches) {
                        console.warn('User has no rights to create objects!');
                        return;
                    }

                    $menuItemLink.text(title);

                    if (onSameLevel) {
                        $parentBlock.show();
                        $parentBlock.after($menuItem);
                    } else {
                        $divider.show();
                        $childBlock.show();
                        $childBlock.after($menuItem);
                    }
                } else {
                    if (!access.canModify && !access.canCreateObjects) {
                        console.warn('User has no rights to create objects!');
                        return;
                    }

                    $menuItemLink.text(title);
                    $actionsBlock.show();

                    $actionsBlock.after($menuItem)
                }
                function notImplemented() {
                    alert("Not implemented yet");
                }

                function onMenuItemClick(e) {
                    var $item = $(this);
                    var typeToAdd = $item.attr('data-itemType');

                    $('.ajs-input').val(''); //hint, to clear previously added value of alertify input, need to read doc more deep
                    $menu.removeClass('open');

                    switch ($item.attr('class')) {
                        case "menu-item-add":
                            var filterData = $.fn.zTree.getTreeLevelData(treeId, treeNode);
                            treeNode = treeNode || {};
                            var path = 'id=' + treeNode.id + "&path=" + encodeURIComponent(window.App.Helpers.ShowPath(treeNode.path));

                            switch (typeToAdd) {
                                case  window.App.Data.EntityTypes[2].type:
                                    var rootNode = $.fn.zTree.getRootNode(treeNode) || {};
                                    path = 'id=' + rootNode.id + "&path=" + encodeURIComponent(window.App.Helpers.ShowPath(rootNode.path));
                                    window.App.navigate("project/" + treeId + "/create?" + path, {trigger: true, params: filterData});
                                    break;
                                case  window.App.Data.EntityTypes[3].type : //CompositeMaterial
                                    window.App.navigate("project/" + treeId + "/create?" + path, {trigger: true, params: filterData});
                                    break;
                                case window.App.Data.EntityTypes[4].type: //File
                                    switch (treeId) {
                                        case window.App.Data.Trees[0].type :
                                            window.App.navigate("project/files/" + treeId + "/" + treeNode.id + "/materialFiles/create",
                                                {trigger: true, params: filterData});
                                            break;
                                        case window.App.Data.Trees[4].type :
                                            window.App.navigate("project/files/" + treeId + "/" + treeNode.id + "/materialFiles/create",
                                                {trigger: true, params: filterData});
                                            break;
                                    }
                                    break;
                                case window.App.Data.EntityTypes[5].type:
                                    window.App.navigate("project/" + treeId + "/create", {trigger: true, params: filterData});
                                    break;
                                case window.App.Data.EntityTypes[6].type:
                                    var path = 'id=' + treeNode.id + "&path=" + encodeURIComponent(window.App.Helpers.ShowPath(treeNode.path));
                                    window.App.navigate("project/" + treeId + "/create?" + path, {trigger: true, params: filterData});
                                    break;
                                case  window.App.Data.EntityTypes[8].type : //OptimizationFolder
                                    filterData = $.fn.zTree.getTreeLevelData(treeId, treeNode, "ConstructionElement");

                                    window.App.navigate("project/" + treeId + "/" + typeToAdd + "/" + treeNode.itemType + "/create?pid=" + treeNode.id +
                                        "&constructionElementId=" + filterData.typeRootData.id + "&treeId=" + treeId, {
                                        trigger: true,
                                        params: filterData
                                    });
                                    break;
                                case  window.App.Data.EntityTypes[0].type :
                                    window.alertify.prompt("Добавить", "Введите имя", undefined, function (e, str) {
                                        that.addNewNodeItem($item, treeId, treeNode, e, str);
                                    }, null);
                                    break;
                            }
                            break;
                        case "menu-item-edit":
                            switch (treeNode.itemType) {
                                case window.App.Data.EntityTypes[2].type:
                                    window.App.navigate("project/" + window.App.Data.Trees[0].type + "/" + treeNode.id + "/parameters/" + typeToAdd, {
                                        params: {path: treeNode.path},
                                        trigger: true
                                    });
                                    break;
                                default :
                                    window.alertify.prompt("Переименовать: " + treeNode.name, null, "Новое имя", function (e, str) {
                                        if (e) {
                                            that.editNodeItem($item, treeId, treeNode, e, str);
                                        } else {
                                            console.warn('you should never see this!!')
                                        }
                                    }, treeNode.name);
                            }
                            break;
                        case "menu-item-remove":
                            if (window.App.Data.EntityTypes[6].type === treeNode.itemType) { //ConstructionElement silent remove if directories CAD,CAE,CAO doesn't have children

                                var hasChildren;
                                hasChildren = _.find(treeNode.children, function (child, i) {
                                    if (child.children && child.children.length > 0) {
                                        return true;
                                    }
                                });

                                if (typeof hasChildren === 'undefined') { //REMOVE silent without confirmation
                                    removeTreeNode();
                                    console.warn("Remove node, silent mode");
                                    return;
                                }
                            }

                            window.alertify.confirm("Удаление", "Подтвердите удаление : " + treeNode.name, removeTreeNode, null).set('labels', {
                                ok: "Удалить",
                                cancel: "Отменить"
                            });

                            break;
                        default :
                            notImplemented();
                    }
                }

                function removeTreeNode() {
                    $.ajax("rest/tree/" + treeId + "/" + treeNode.id, {
                        type: "DELETE",
                        contentType: "application/json",
                        success: function (res) {
                            if (res.status == 'success') {
                                var parentNode = treeNode.getParentNode();

                                if (parentNode && parentNode.tId && window.App.Data.EntityTypes[6].type === treeNode.itemType) {
                                    $("#" + parentNode.tId + "_a").click();
                                }
                                window.App.vent.trigger('updateTreeNodes', treeNode, treeId, treeNode.itemType, 'remove', 'tree', null, res.value);
                                // window.App.vent.trigger('updateTreeNodes', treeNode, treeId, treeNode.itemType, 'remove');
                                window.alertify.success(res.message);
                            } else {
                                window.alertify.error(res.message);
                            }
                        }
                    });
                }

                $menuItemLink.on('click', onMenuItemClick);
            }

            switch (nodeType) {
                case window.App.Data.EntityTypes[0].type: // Folder
                    if (treeId !== window.App.Data.Trees[3].type) {
                        addMenuItem(window.App.Data.EntityTypes[0].type, window.App.Data.EntityTypes[0].name, true);
                    } else {
                        contextDisabled = true; //disable contextmenu for optimization tree if Folder node selected
                    }
                    switch (treeId) {
                        case window.App.Data.Trees[0].type: //materials
                            if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[0].type)) {
                                addMenuItem(window.App.Data.EntityTypes[2].type, window.App.Data.EntityTypes[2].name, false);
                            }
                            if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[2].type)) {
                                addMenuItem(window.App.Data.EntityTypes[0].type, window.App.Data.EntityTypes[0].name, false);
                            }
                            break;
                        case window.App.Data.Trees[1].type: //technologies
                            addMenuItem(window.App.Data.EntityTypes[5].type, window.App.Data.EntityTypes[5].name, true);
                            break;
                        case window.App.Data.Trees[4].type: //compositeMaterials
                            if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[0].type)) {
                                addMenuItem(window.App.Data.EntityTypes[3].type, window.App.Data.EntityTypes[3].name, false);
                            }
                            if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[3].type)) {
                                addMenuItem(window.App.Data.EntityTypes[0].type, window.App.Data.EntityTypes[0].name, false);
                            }
                            break;
                        case window.App.Data.Trees[2].type: //constructionElements
                            //if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[0].type)) {
                            addMenuItem(window.App.Data.EntityTypes[6].type, window.App.Data.EntityTypes[6].name, false);
                            //}
                            //if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[6].type)) {
                            addMenuItem(window.App.Data.EntityTypes[0].type, window.App.Data.EntityTypes[0].name, false);
                            //}

                            break;
                    }
                    break;
                case window.App.Data.EntityTypes[1].type: // MaterialFile
                    //do not add menu items
                    break;
                case window.App.Data.EntityTypes[2].type: // Material
                  /*do not add menu items*/
                    addMenuItem(window.App.Data.EntityTypes[4].type, window.App.Data.EntityTypes[4].name, false);
                    switch (treeId) {
                        case window.App.Data.Trees[0].type: //materials
                          /*do not add menu items*/
                            if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[0].type)) {
                                addMenuItem(window.App.Data.EntityTypes[2].type, window.App.Data.EntityTypes[2].name, true);
                            }
                            break;
                    }
                    break;
                case window.App.Data.EntityTypes[4].type: // File
                    //addMenuItem(window.App.Data.EntityTypes[4].type, window.App.Data.EntityTypes[4].name, true);
                    break;
                case window.App.Data.EntityTypes[3].type: // CompositeMaterial
                    addMenuItem(window.App.Data.EntityTypes[3].type, window.App.Data.EntityTypes[3].name, true);

                    if (!_.contains(treeNode.childrenTypes, window.App.Data.EntityTypes[0].type)) {
                        addMenuItem(window.App.Data.EntityTypes[4].type, window.App.Data.EntityTypes[4].name, false);
                    }
                    break;
                case window.App.Data.EntityTypes[6].type: // ConstructionElement
                    switch (treeId) {
                        case window.App.Data.Trees[3].type: //optimizations
                            //OptimizationFolder
                            addMenuItem(window.App.Data.EntityTypes[8].type, window.App.Data.EntityTypes[8].name, false);
                            break;
                        case window.App.Data.Trees[4].type: //compositeMaterials
                            //todo probably not already using
                            contextDisabled = true;
                            break;
                        //case window.App.Data.Trees[2].type: //constructionElements
                        //OptimizationFolder
                        //addMenuItem(window.App.Data.EntityTypes[8].type, window.App.Data.EntityTypes[8].name, false);
                        //break;

                        //case window.App.Data.Trees[2].type: //constructionElements
                        //addMenuItem(window.App.Data.EntityTypes[6].type, window.App.Data.EntityTypes[6].name, false);
                        //break;
                    }
                    break;
                case window.App.Data.EntityTypes[7].type: // ConstructionElementFolder
                    console.warn('No menu items assigned!');
                    contextDisabled = true;
                    break;
                case window.App.Data.EntityTypes[8].type: // OptimizationFolder
                    addMenuItem(window.App.Data.EntityTypes[8].type, window.App.Data.EntityTypes[8].name, false);
                    break;
                default :
                    //TODO extract to method
                    if (treeId !== window.App.Data.Trees[1].type) {
                        addMenuItem(window.App.Data.EntityTypes[0].type, window.App.Data.EntityTypes[0].name, true);
                        //if (treeId === window.App.Data.Trees[2].type) {
                        //addMenuItem(window.App.Data.EntityTypes[6].type, window.App.Data.EntityTypes[6].name, true);
                        //}
                    } else {
                        addMenuItem(window.App.Data.EntityTypes[5].type, window.App.Data.EntityTypes[5].name, true);
                    }
            }

            if (treeNode) {
                if (!treeNode.children || !treeNode.children.length ||
                    (treeId === window.App.Data.Trees[2].type && treeNode.itemType === window.App.Data.EntityTypes[6].type)) {
                    if (
                        window.App.Data.EntityTypes[6].type !== treeNode.itemType && treeId !== window.App.Data.Trees[4].type ||
                        window.App.Data.EntityTypes[6].type === treeNode.itemType && treeId === window.App.Data.Trees[2].type ||
                        window.App.Data.EntityTypes[4].type === treeNode.itemType && treeId === window.App.Data.Trees[4].type ||
                        window.App.Data.EntityTypes[3].type === treeNode.itemType && treeId === window.App.Data.Trees[4].type ||
                        window.App.Data.EntityTypes[0].type === treeNode.itemType && treeId === window.App.Data.Trees[4].type) {

                        addMenuItem('remove', window.App.Data.TreeContextMenuItems.remove, false, "remove")
                    }
                }

                //TODO extract to addEditItem method
                if (treeId !== window.App.Data.Trees[1].type
                    && window.App.Data.EntityTypes[6].type !== treeNode.itemType) { // do not add menu edit item if technology tree
                    addMenuItem('edit', window.App.Data.TreeContextMenuItems.edit, false, "edit")
                }
            }

            var xLeft = e.clientX,
                yTop = e.clientY,
                windowHeight = $(window).height(),
                menuHeight = $menu.children('.dropdown-menu').outerHeight();

            if (yTop + menuHeight >= windowHeight) {
                yTop -= menuHeight;
            }

            if (contextDisabled) {
                return;
            }
            $menu
                .addClass('open')
                .css('position', 'absolute')
                .css('z-index', 9999)
                .css('top', yTop)
                .css('left', xLeft);

            function onBodyMouseDown(event) {
                if (!(event.target.id == "zTree-contextmenu" || $(event.target).parents("#zTree-contextmenu").length > 0)) {
                    $menu.removeClass('open');
                }
            }

            $("body").bind("mousedown", onBodyMouseDown);
        },

        //TODO refactor for params object, hardcoded for recursive
        setAsyncUrlParams: function (treeId, treeNode) {
            this.zTreeObj.setting.async.otherParam.push("recursive");
            this.zTreeObj.setting.async.otherParam.push("true");
            return true;
        }

    });

})();
