/**
 * Select2
 *
 * Renders Select2 - jQuery based replacement for select boxes
 * https://gist.github.com/jbugwadia/9303389
 *
 * Usage: Works the same as Select editor, with the following extensions for Select2:
 *    schema.config: configuration object passed to Select2
 *    schema.multiple: sets 'multiple' property on the HTML <select>
 *
 * Example:
 *    schema: {title: {type:'Select2', options:['Mr','Mrs',Ms], config: {}, multiple: false}
 *
 * Also see:
 *    https://gist.github.com/powmedia/5161061
 *    https://gist.github.com/Integral/5156170
 *
 * Changelog:
 *    removed hard tabs (@fonji)
 */

//Backbone.Form.editors.Select2 = Backbone.Form.editors.Select.extend ({
//
//  render: function() {
//
//    console.log("backbone-forms-editor-select2.js this ",this )
//    this.setOptions(this.schema.options);
//    var multiple = this.schema.multiple;
//    var config = this.schema.config || {};
//
//    var elem = this;
//    setTimeout(function() {
//      if (multiple) {
//        elem.$el.prop('multiple', true);
//      }
//
//      elem.$el.select2(config);
//    }, 0);
//
//    return this;
//  }
//});

/**
 * Select2
 *
 * Renders Select2 - jQuery based replacement for select boxes
 *
 * Simply pass a 'config' object on your schema, with any options to pass into Select2.
 * See http://ivaynberg.github.com/select2/#documentation
 */
Backbone.Form.editors.Select2 = Backbone.Form.editors.Select.extend({

  /**
   * @param {Object} options.schema.config    Options to pass to select2. See http://ivaynberg.github.com/select2/#documentation
   */
  initialize: function (options) {
    Backbone.Form.editors.Base.prototype.initialize.call(this, options);

    var schema = this.schema;

    this.config = schema.config || {};
  },

  render: function () {
    var self = this;

    setTimeout(function () {
      self.$el.select2(self.config);
    }, 0);

    this.setValue();
    return this;
  },

  getValue: function () {
    if (this.schema.valueType && this.schema.valueType === "Number") {
      if(this.$el.val().length){ //tikhonov, need to validate required field
        return parseInt(this.$el.val());
      }
    }
    return this.$el.val();
  },

  setValue: function (val) {
    if (val) {
      this.$el.val(val);
    } else {
      var data = [{id: this.model.get(this.key), text: this.model.get(this.key)}];

      var fieldName = this.$el.data().field;
      if (fieldName) {
        data = [{id: this.model.get(fieldName + 'Id'), text: this.model.get(fieldName)}];
      }
      this.$el.select2({data: data})
    }
  }

});
