package ru.crp.lvms;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.springframework.core.GenericTypeResolver;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.web.dto.pagination.DataRequest;

import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import static org.junit.Assert.*;
import static ru.crp.lvms.utils.TestUtils.*;

abstract public class CRUDIT<IN extends Entity, OUT extends Entity> extends BaseIT {

    protected String url;
    private Supplier<OUT> entityCreator;
    private Supplier<IN> entityInitialisation;
    private Function<JsonNode,List<? extends Entity>> jsonFromList;
    private Function<JsonNode,List<? extends Entity>> jsonFromPage;
    private Class<IN> genericTypeIn;
    private Class<OUT> genericTypeOut;

    @Before
    public void initCRUDIT() {
        url = setUrl();
        entityCreator = setEntityCreator();
        entityInitialisation = setEntityInitialisation();
        jsonFromList = setJsonFromList();
        jsonFromPage = setJsonFromPage();
        genericTypeIn = (Class<IN>) GenericTypeResolver.resolveTypeArguments(getClass(), CRUDIT.class)[0];
        genericTypeOut = (Class<OUT>) GenericTypeResolver.resolveTypeArguments(getClass(), CRUDIT.class)[1];
    }
    
    public abstract String setUrl();
    public abstract Supplier<OUT> setEntityCreator();
    public abstract Supplier<IN> setEntityInitialisation();
    public abstract Function<JsonNode,List<? extends Entity>> setJsonFromList();
    public abstract Function<JsonNode,List<? extends Entity>> setJsonFromPage();
    public abstract void update(IN toUpdate);
    public abstract void validateAfterUpdate(IN before, OUT after);


    @Test
    public void testCreate() throws Exception {
        OUT created = entityCreator.get();
        assertNotEquals(0, created.getId());
    }


    @Test
    public void testGet() throws Exception {
        OUT created = entityCreator.get();
        System.out.println(createObjectUrl(url, created));
        OUT createdById = GETobject(mockMvc, createObjectUrl(url, created), genericTypeOut);
        assertEquals(created.getId(), createdById.getId());
    }


    @Test
    public void testUpdate() throws Exception {
        IN proxy = entityInitialisation.get();
        OUT created = CREATEobject(mockMvc, url, proxy, genericTypeOut);
        proxy.setId(created.getId());
        update(proxy);
        OUT updated = UPDATEobject(mockMvc, createObjectUrl(url, proxy), proxy, genericTypeOut);
        validateAfterUpdate(proxy, updated);
    }


    @Test
    @Ignore
    public void testDelete() throws Exception {
        OUT created = entityCreator.get();
        DELETEobject(mockMvc, createObjectUrl(url,created));
        GETAndExpectNotFound(mockMvc, createObjectUrl(url,created));
    }


    @Test
    public void testGetAll() throws Exception {
        OUT p1 = entityCreator.get();
        OUT p2 = entityCreator.get();

        JsonNode json = GETobjects(mockMvc, url);
        List<Integer> all = jsonFromList.apply(json)
                .stream()
                .map(Entity::getId)
                .collect(Collectors.toList());

        assertTrue(all.contains(p1.getId()));
        assertTrue(all.contains(p2.getId()));
    }


    @Test
    public void testPage() throws Exception {
        OUT p1 = entityCreator.get();
        OUT p2 = entityCreator.get();

        DataRequest dr = projectCreator.newDataRequest();
        JsonNode json = GETPage(mockMvc, createPageUrl(url), dr);
        List<Integer> all = jsonFromPage.apply(json)
                .stream()
                .map(Entity::getId)
                .collect(Collectors.toList());

        assertTrue(all.contains(p1.getId()));
        assertTrue(all.contains(p2.getId()));
    }
}
