package ru.crp.lvms.material;

import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.UncategorizedSQLException;
import ru.crp.lvms.BaseIT;
import ru.crp.lvms.managers.MaterialManager;
import ru.crp.lvms.model.entities.MaterialEntity;
import ru.crp.lvms.model.entities.MaterialEntityTable;

import java.util.Date;

import static ru.crp.lvms.utils.TestUtils.unique;

public class MaterialDaoIT extends BaseIT {

    @Autowired
    private MaterialManager materialManager;

    @Test
    public void declineUpdateIfMaterialLocked() throws Exception {
        boolean updateException = false;
        boolean updateAndReadException = false;
        MaterialEntity met = projectCreator.newMaterialEntity();
        met = materialManager.createAndRead(met, null);
        met.setLock(true);
        met = materialManager.updateAndRead(met, null);
        met.setName("Exception");
        try {
            materialManager.update(met, null);
        } catch (UncategorizedSQLException e) {
            updateException = true;
        }

        try {
            materialManager.updateAndRead(met, null);
        } catch (UncategorizedSQLException e) {
            updateAndReadException = true;
        }

        Assert.assertTrue("Update of locked material must be disallowed", updateException);
        Assert.assertTrue("Update and read of locked material must be disallowed", updateAndReadException);
    }


    @Test
    public void allowUpdateIfMaterialIsNotLocked() throws Exception {
        MaterialEntity met = projectCreator.newMaterialEntity();
        MaterialEntityTable insertion = materialManager.createAndRead(met, null);
        Assert.assertEquals(met.getName(), insertion.getName());
        met.setName("Exception2");
        MaterialEntityTable update = materialManager.updateAndRead(met, null);
        Assert.assertEquals("Update must be successful", met.getName(), update.getName());
    }


    @Test
    @Ignore
//    todo fix
    public void modificationDateMustChange() throws Exception {
        MaterialEntity initialEntity = materialManager.createAndRead(projectCreator.newMaterialEntity(), null);
        Date initialModificationDate = initialEntity.getModificationDate();

        initialEntity.setName(unique("updated-name"));
        MaterialEntityTable updatedEntity = materialManager.updateAndRead(initialEntity, null);
        Date updatedMoficiationDate = updatedEntity.getModificationDate();

        Assert.assertNotEquals("Modification date must be changed while changing other properties",
                initialModificationDate, updatedMoficiationDate);
    }
}
