package ru.crp.lvms.material;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Assert;
import org.junit.Test;
import ru.crp.cmlb.model.SuggestRequest;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.CRUDIT;
import ru.crp.lvms.model.entities.MaterialEntity;
import ru.crp.lvms.utils.TestUtils;
import ru.crp.lvms.web.dto.CompositeMaterialIn;
import ru.crp.lvms.web.dto.MaterialIn;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import static org.junit.Assert.assertEquals;
import static ru.crp.lvms.utils.TestUtils.*;
import static ru.crp.lvms.utils.URLs.MATERIALS_URL;

public class MaterialEndpointIT extends CRUDIT<MaterialIn, MaterialEntity> {

    @Override
    public String setUrl() {
        return MATERIALS_URL;
    }

    @Override
    public Supplier<MaterialEntity> setEntityCreator() {
        return projectCreator::createMaterial;
    }

    @Override
    public Supplier<MaterialIn> setEntityInitialisation() {
        return projectCreator::newMaterial;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromList() {
        return TestUtils::materialsFromJson;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromPage() {
        return TestUtils::materialsPageFromJson;
    }

    @Override
    public void update(MaterialIn toUpdate) {
        toUpdate.setDescription("testDescription");
        toUpdate.setName("testName");
    }

    @Override
    public void validateAfterUpdate(MaterialIn before, MaterialEntity after) {
        Assert.assertEquals(before.getDescription(), after.getDescription());
        Assert.assertEquals(before.getName(), after.getName());
    }


    @Test
    public void itShouldSuggestMaterials() throws Exception {
        List<String> files = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            MaterialIn materialIn = projectCreator.newMaterial();
            materialIn.setName("TestName" + i);
            MaterialEntity materialOut = CREATEobject(mockMvc, MATERIALS_URL, materialIn, MaterialEntity.class);
            assertEquals(materialIn.getName(), materialOut.getName());
            files.add(materialIn.getName());
        }
        SuggestRequest suggestRequest = projectCreator.newSuggestRequest("name", "%TestName%");
        MaterialEntity[] result = CREATEobject(mockMvc, MATERIALS_URL + "/suggest", suggestRequest, MaterialEntity[].class);
        Assert.assertEquals(files.size(), result.length);
        List<String> out = Arrays.asList(result).stream().map(MaterialEntity::getName).collect(Collectors.toList());
        for (String file : files) {
            Assert.assertTrue(out.contains(file));
        }
    }


    @Test
    public void ConflictShouldBeReturnedWhenRemovingMaterialWhichIsUsedInComposite() throws Exception {
        MaterialEntity material = projectCreator.createMaterial();
        CompositeMaterialIn compositeMaterial = projectCreator.newCompositeMaterial();
        compositeMaterial.setFiberId(material.getId());
        projectCreator.createCompositeMaterial(compositeMaterial);
        DELETEAndExpectConflict(mockMvc, createObjectUrl(MATERIALS_URL, material));
    }

//    @Test
//    public void itShouldBeAbleToUploadAndDownloadMaterialFile() throws Exception {
//        String content = "123456789";
//        MaterialOut material = projectCreator.createMaterial();
//        MockMultipartFile mockFile = new MockMultipartFile("file", content.getBytes());
//        ResponseMessage rm = uploadFile(MATERIALS_URL + "/materialFiles",
//                mockFile, mockMvc, ResponseMessage.class);
//        assertEquals(ResponseMessage.Status.success, rm.getStatus());
//        assertNotNull(rm.getValue());
//
//        MaterialFileEntity materialFileIn = projectCreator.newMaterialFile();
//        materialFileIn.setPid(material.getId());
//        materialFileIn.setPathToFile(rm.getValue().toString());
//        MaterialFileEntity materialFile = projectCreator.createMaterialFile(materialFileIn);
//        MvcResult result = mockMvc.perform(get(createMFUrl(materialFile) + "/file"))
//                .andExpect(status().isOk())
//                .andExpect(content().contentType(MediaType.APPLICATION_OCTET_STREAM))
//                .andReturn();
//        assertEquals(result.getResponse().getContentAsString(), content);
//
//        Files.deleteIfExists(pathToMaterialFiles.resolve(rm.getValue().toString()));
//    }
}
