package ru.crp.lvms.optimizations;

import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import ru.crp.cmlb.model.entities.FileData;
import ru.crp.lvms.BaseIT;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.mf.dao.MFOptimizationBaseSettingsDao;
import ru.crp.lvms.optimizations.mf.entities.MFOptimizationBaseSetting;
import ru.crp.lvms.optimizations.mf.enums.MFBaseSettingType;
import ru.crp.lvms.optimizations.mf.enums.MFSettingsType;
import ru.crp.lvms.optimizations.mf.managers.MFOptimizationSettingsManager;
import ru.crp.lvms.tasks.fileGenerators.MFInputFileGenerator;

import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static java.util.stream.Collectors.groupingBy;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class OptimizationInputFileGeneratorIT extends BaseIT {

    @Autowired
    private MFInputFileGenerator mfInputFileGenerator;

    @Autowired
    private MFOptimizationSettingsManager MFOptimizationSettingsManager;

    @Autowired
    private MFOptimizationBaseSettingsDao MFOptimizationBaseSettingsDao;


    @Test
    public void testSettingsOrderInFile() throws IOException {
        OptimizationEntity optimization = projectCreator.createOptimization();
        List<MFOptimizationBaseSetting> baseSettings = (List<MFOptimizationBaseSetting>) MFOptimizationSettingsManager
                .findOptimizationSettings(optimization.getId(), MFSettingsType.BaseSetting, currentUser);
        Map<Integer, List<MFOptimizationBaseSetting>> settingsByType = baseSettings.stream().collect(groupingBy(MFOptimizationBaseSetting::getTypeId));
        baseSettings.forEach(settings -> {
            settings.setCurrentValue(MFBaseSettingType.valueOf(settings.getTypeId()).getMaxValue());
            MFOptimizationBaseSettingsDao.update(settings);
        });
        List<FileData> files = new ArrayList<>();
        mfInputFileGenerator.addProjAndGeneratedFiles(files, optimization);

        assertTrue(files.size() >= 1);

        List<String> content = Files.readAllLines(files.get(0).getPathToFile());


        int expectedContentSize = MFBaseSettingType.values().length + 1;
        if (optimization.getSolverCadId() == null) {
            expectedContentSize = MFBaseSettingType.values().length;
        }
        assertEquals(expectedContentSize, content.size());

        assertEquals(settingsByType.get(MFBaseSettingType.NUM_TARGET_FUNCS.getId()).get(0).getCurrentValue() + "", content.get(0));
        assertEquals(settingsByType.get(MFBaseSettingType.NUM_CAO_VARYING_PARAMS.getId()).get(0).getCurrentValue() + "", content.get(1));
        assertEquals(settingsByType.get(MFBaseSettingType.NUM_OUTPUT_PARAMS.getId()).get(0).getCurrentValue() + "", content.get(2));
        assertEquals(settingsByType.get(MFBaseSettingType.NUM_CONSTRAINTS.getId()).get(0).getCurrentValue() + "", content.get(3));
        assertEquals("OUTPUT_FILES_PATH", content.get(4));

        Files.deleteIfExists(files.get(0).getPathToFile());
    }
}
