package ru.crp.lvms.technologies;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.web.servlet.MvcResult;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.CRUDIT;
import ru.crp.lvms.model.entities.misc.FileType;
import ru.crp.lvms.model.entities.technology.TechnologyEntity;
import ru.crp.lvms.utils.FilesTester;
import ru.crp.lvms.utils.TestUtils;

import java.io.IOException;
import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;

import static org.junit.Assert.assertNull;
import static ru.crp.lvms.utils.URLs.TECHNOLOGY_URL;

public class TechnologyEndpointIT extends CRUDIT<TechnologyEntity, TechnologyEntity> {
    @Autowired
    private FilesTester filesTester;

    private TechnologyEntity testTechnology;

    @Override
    public void setup() throws Exception {
        super.setup();
        filesTester.setMockMvc(mockMvc);
        testTechnology = projectCreator.createTechnologyEntity();
    }

    @Override
    public String setUrl() {
        return TECHNOLOGY_URL;
    }

    @Override
    public Supplier<TechnologyEntity> setEntityCreator() {
        return () -> projectCreator.createTechnologyEntity();
    }

    @Override
    public Supplier<TechnologyEntity> setEntityInitialisation() {
        return () -> projectCreator.newTechnology();
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromList() {
        return TestUtils::technologyFromJson;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromPage() {
        return TestUtils::technologyPageFromJson;
    }

    @Override
    public void update(TechnologyEntity toUpdate) {
        toUpdate.setName("testName");
    }

    @Override
    public void validateAfterUpdate(TechnologyEntity before, TechnologyEntity after) {
        Assert.assertEquals(before.getName(), after.getName());
    }


    @Override
    public void testUpdate() throws Exception {
        super.testUpdate();
    }

    @Test
    public void itShouldBeAbleToChangeImageFile() throws Exception {
        String url = getUrlForFile(FileType.ImageFile);
        filesTester.testUpdateFile(FileType.ImageFile, url);
    }


    @Test
    public void itShouldBeAbleToChangeDescriptionFile() throws Exception {
        String url = getUrlForFile(FileType.DescriptionFile);
        filesTester.testUpdateFile(FileType.DescriptionFile, url);
    }


    @Test
    public void itShouldBeAbleToDeleteImageFile() throws Exception {
        String url = getUrlForFile(FileType.ImageFile);
        filesTester.testDeleteFile(FileType.ImageFile, true, url, this::fileRemovedChecker);
    }

    @Test
    public void itShouldBeAbleToDeleteDescriptionFile() throws Exception {
        String url = getUrlForFile(FileType.DescriptionFile);
        filesTester.testDeleteFile(FileType.DescriptionFile, true, url, this::fileRemovedChecker);
    }


    private void fileRemovedChecker(FileType fileType, MvcResult result) {
        try {
            TechnologyEntity technology = jacksonMapper
                    .readValue(result.getResponse().getContentAsString(), TechnologyEntity.class);
            if (fileType == FileType.ImageFile) {
                assertNull(technology.getImageFileId());
            } else {
                assertNull(technology.getDescriptionFileId());
            }
        } catch (IOException e) {
            throw new RuntimeException();
        }
    }


    private String getUrlForFile(FileType fileType) {
        if (fileType == FileType.ImageFile) {
            return url = TECHNOLOGY_URL + "/" + testTechnology.getId() + "/imageFile";
        } else {
            return url = TECHNOLOGY_URL + "/" + testTechnology.getId() + "/descriptionFile";
        }
    }
}
