package ru.crp.lvms.utils;

import com.google.common.base.Joiner;
import net.bytebuddy.ByteBuddy;
import net.bytebuddy.dynamic.ClassLoadingStrategy;
import net.bytebuddy.instrumentation.MethodDelegation;
import net.bytebuddy.instrumentation.method.bytecode.bind.annotation.Origin;
import net.bytebuddy.instrumentation.method.bytecode.bind.annotation.SuperCall;
import net.bytebuddy.matcher.ElementMatchers;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.List;
import java.util.concurrent.Callable;

import static java.util.Arrays.asList;

public class SetterChecker implements SetterCheckerInterface {
    private static final List<String> skippedSetters = asList("setCreationDate",
            "setModificationDate", "setId", "setPath", "setPathNames", "setOwner", "setStandardFileName",
            "setLoadFileName", "setChildrenTypes", "setParent", "setReferenceId", "setReports", "setFlag",
            "setPostprocessingStatus", "setOptimizationType", "setFiles", "setReference", "setSolvingStatus",
            "setResults", "setImageFileId", "setImageFileName", "setPathToImage", "setDescription");
    Class clazz;
    HashSet<String> methods = new HashSet<>();
    boolean enabled = true;

    public SetterChecker(Class clazz) {
        this.clazz = clazz;
        for (Method method : clazz.getMethods()) {
            if (isSetter(method) && !skippedSetters.contains(method.getName())) {
                methods.add(method.getName());
            }
        }
    }

    private boolean isSetter(Method method) {
        return method.getName().startsWith("set") && method.getParameterTypes().length == 1;
    }

    public boolean allSettersCalled() {
        return !enabled || methods.isEmpty();
    }

    public void disableCheck() {
        enabled = false;
    }

    public String notCalledSetters() {
        return Joiner.on(", ").join(methods);
    }

    public Object interceptSetters(@Origin Method method, @SuperCall Callable zuper) throws Exception {
        if (isSetter(method)) {
            methods.remove(method.getName());
        }
        return zuper.call();
    }

    public static <T> T createProxy(Class<T> clazz) throws IllegalAccessException, InstantiationException, NoSuchMethodException {
        SetterCheckerInterface setterChecker = new SetterChecker(clazz);
        return new ByteBuddy()
                .subclass(clazz)
                .implement(SetterCheckerInterface.class)
                .method(ElementMatchers.named("allSettersCalled"))
                .intercept(MethodDelegation.to(setterChecker))
                .method(ElementMatchers.named("notCalledSetters"))
                .intercept(MethodDelegation.to(setterChecker))
                .method(ElementMatchers.isSetter()).intercept(MethodDelegation.to(setterChecker))
                .make()
                .load(clazz.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER)
                .getLoaded()
                .newInstance();
    }

    public static boolean checkIsProxy(Object object) {
        return SetterCheckerInterface.class.isInstance(object);
    }

    public static boolean checkAllSettersCalled(Object object) {
        return ((SetterCheckerInterface) object).allSettersCalled();
    }

    public static String notCalledSetters(Object object) {
        return ((SetterCheckerInterface) object).notCalledSetters();
    }
}
