package ru.crp.lvms.web.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ru.crp.cmls.api.TaskParentType;
import ru.crp.cmls.calculations.solvers.xml.config.ResultType;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.exceptions.NotFoundException;
import ru.crp.lvms.managers.ResultsManager;
import ru.crp.lvms.managers.misc.PreferencesManager;
import ru.crp.lvms.managers.tasks.ResultsFilesManager;
import ru.crp.lvms.model.entities.ResultEntity;
import ru.crp.lvms.optimizations.base.managers.OptimizationManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.ColumnsController;
import ru.crp.lvms.web.controllers.misc.FileHelper;
import ru.crp.lvms.web.dto.pagination.DataRequest;
import ru.crp.lvms.web.dto.pagination.PageingResponse;

import javax.annotation.Nullable;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.nio.file.Path;
import java.util.List;
import java.util.Map;

import static ru.crp.lvms.utils.URLs.RESULTS_URL;
import static ru.crp.lvms.web.controllers.misc.CrudController.deleteSeveralEntities;

@RestController
@RequestMapping(value = RESULTS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class ResultsController implements ColumnsController {

    @Autowired
    private PreferencesManager preferencesManager;

    @Autowired
    private ResultsManager resultsManager;

    @Autowired
    private ResultsFilesManager resultsFilesManager;

    @Override
    public Class getOutVoClass() {
        return ResultEntity.class;
    }

    @Override
    public PreferencesManager getPreferencesManager() {
        return preferencesManager;
    }

    @Autowired
    private OptimizationManager optimizationManager;

    @RequestMapping(value = "/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getAllOptimizationResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                         @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getResultsMapByParent(optimizationId, TaskParentType.optimization, currentUser.getUser());
    }


    @RequestMapping(value = "/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity> getOptimizationResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                    @RequestBody DataRequest dataRequest,
                                                                    @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/pictures/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getOptimizationPicturesResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                         @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getCustomResultsMapByParent(optimizationId, TaskParentType.optimization, ResultType.pictures,currentUser.getUser());
    }


    @RequestMapping(value = "/pictures/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity>getOptimizationPituresResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                    @RequestBody DataRequest dataRequest,
                                                                    @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findCustomResultsPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest,ResultType.pictures, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/targetValues/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getOptimizationTVResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                              @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getCustomResultsMapByParent(optimizationId, TaskParentType.optimization, ResultType.values,currentUser.getUser());
    }


    @RequestMapping(value = "/targetValues/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity>getOptimizationTVResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                          @RequestBody DataRequest dataRequest,
                                                                          @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findCustomResultsPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest,ResultType.values, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/animations/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getOptimizationAnimationsResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                              @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getCustomResultsMapByParent(optimizationId, TaskParentType.optimization, ResultType.animations,currentUser.getUser());
    }


    @RequestMapping(value = "/animations/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity>getOptimizationAnimationsResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                          @RequestBody DataRequest dataRequest,
                                                                          @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findCustomResultsPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest,ResultType.animations, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/curves/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getOptimizationCurvesResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                              @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getCustomResultsMapByParent(optimizationId, TaskParentType.optimization, ResultType.curves,currentUser.getUser());
    }


    @RequestMapping(value = "/curves/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity>getOptimizationCurvesResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                          @RequestBody DataRequest dataRequest,
                                                                          @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findCustomResultsPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest,ResultType.curves, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/dbFiles/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getOptimizationDatabaseResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                              @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getCustomResultsMapByParent(optimizationId, TaskParentType.optimization, ResultType.database,currentUser.getUser());
    }


    @RequestMapping(value = "/dbFiles/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity>getOptimizationDatabaseResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                          @RequestBody DataRequest dataRequest,
                                                                          @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findCustomResultsPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest,ResultType.database, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/logFiles/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getOptimizationLogsResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                              @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getCustomResultsMapByParent(optimizationId, TaskParentType.optimization, ResultType.logFiles,currentUser.getUser());
    }


    @RequestMapping(value = "/logFiles/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity>getOptimizationLogsResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                          @RequestBody DataRequest dataRequest,
                                                                          @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findCustomResultsPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest,ResultType.logFiles, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/stlVisualisations/optimizations/{optimizationId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getOptimizationStlsResults(@PathVariable("optimizationId") Integer optimizationId,
                                                                              @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getCustomResultsMapByParent(optimizationId, TaskParentType.optimization, ResultType.stlVisualisation,currentUser.getUser());
    }


    @RequestMapping(value = "/stlVisualisations/optimizations/{optimizationId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity>getOptimizationStlsResultsPage(@PathVariable("optimizationId") Integer optimizationId,
                                                                          @RequestBody DataRequest dataRequest,
                                                                          @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findCustomResultsPageByParent(optimizationId, TaskParentType.optimization,
                dataRequest.getFilters(), pageRequest,ResultType.stlVisualisation, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    //effectiveCharacteristics

    @RequestMapping(value = "/effectiveCharacteristics/{ecId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getAllECResults(@PathVariable("ecId") Integer ecId,
                                                               @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getResultsMapByParent(ecId, TaskParentType.effectiveCharacteristic, currentUser.getUser());
    }


    @RequestMapping(value = "/effectiveCharacteristics/{ecId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity> getECResultsPage(@PathVariable("ecId") Integer ecId,
                                                          @RequestBody DataRequest dataRequest,
                                                          @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getPageable().toSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findPageByParent(ecId, TaskParentType.effectiveCharacteristic,
                dataRequest.getFilters(), pageRequest, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }

    @RequestMapping(value = "/simulations/{simId}", method = RequestMethod.GET)
    public Map<ResultType, List<ResultEntity>> getAllSimulationResults(@PathVariable("simId") Integer simId,
                                                                       @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        return resultsManager.getResultsMapByParent(simId, TaskParentType.simulation, currentUser.getUser());
    }


    @RequestMapping(value = "/simulations/{simId}/list", method = RequestMethod.POST)
    public PageingResponse<ResultEntity> getSimulationResultsPage(@PathVariable("simId") Integer simId,
                                                                  @RequestBody DataRequest dataRequest,
                                                                  @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        Pageable pageRequest = dataRequest.getPageable().toSpringPageable();
        PageImpl<ResultEntity> page = resultsManager.findPageByParent(simId, TaskParentType.simulation,
                dataRequest.getFilters(), pageRequest, currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageRequest, page.getTotalElements());
    }


    @RequestMapping(value = "/{resultId}", method = RequestMethod.GET)
    public ResultEntity getResult(@PathVariable("resultId") Integer resultId, @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        ResultEntity result = resultsManager.getResult(resultId);
        checkResultExists(resultId, result);
        return result;
    }


    @RequestMapping(value = "/{resultId}/content", method = RequestMethod.GET)
    public void getResultFile(HttpServletResponse response, @PathVariable("resultId") Integer resultId,
                              @CurrentUser PortalUser currentUser) throws IOException {
        // todo check result parent access
        ResultEntity result = resultsManager.getResult(resultId);
        checkResultExists(resultId, result);
        Path pathToResult = resultsFilesManager.getPathToResult(result);
        FileHelper.writeFileToResponse(pathToResult, null, response);
    }


    @RequestMapping(value = "/{resultId}/preview", method = RequestMethod.GET)
    public Object getResultPreview(HttpServletResponse response, @PathVariable("resultId") Integer resultId,
                                   @CurrentUser PortalUser currentUser) throws IOException {
        // todo check result parent access
        ResultEntity result = resultsManager.getResult(resultId);
        checkResultExists(resultId, result);
        ResultType resultType = result.getType();
        if (resultType == ResultType.pictures || resultType == ResultType.animations) {
            Path pathToResult = resultsFilesManager.getPathToResult(result);
            FileHelper.writeFileToResponse(pathToResult, null, response);
            return null;
        }
        if (resultType == ResultType.curves) {
            return resultsFilesManager.getCurve(result);
        }

        return ResponseEntity.noContent().build();
    }


    @RequestMapping(value = "/{resultId}", method = RequestMethod.PUT)
    public ResultEntity update(@PathVariable("resultId") Integer resultId,
                               @RequestBody @Valid ResultEntity updatedEntity,
                               @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        ResultEntity existedResult = resultsManager.getResult(resultId);
        checkResultExists(resultId, existedResult);

        existedResult.setName(updatedEntity.getName());
        existedResult.setDescription(updatedEntity.getDescription());

        resultsManager.updateResult(existedResult);
        return resultsManager.getResult(resultId);
    }


    @RequestMapping(value = "/{resultId}", method = RequestMethod.DELETE)
    @ResponseStatus(HttpStatus.OK)
    public void delete(@PathVariable("resultId") Integer resultId, @CurrentUser PortalUser currentUser) {
        // todo check result parent access
        resultsManager.deleteResult(resultId);
    }

    @RequestMapping(method = RequestMethod.DELETE)
    @ResponseStatus(HttpStatus.OK)
    public void deleteSeveral(@RequestParam("ids") List<Integer> resultIds,
                              @CurrentUser PortalUser currentUser) throws BadRequestException {
        deleteSeveralEntities(ResultEntity.class, resultIds, (id) -> {
            delete(id, currentUser);
        });
    }

    private void checkResultExists(int resultId, @Nullable ResultEntity result) {
        if (result == null) {
            throw new NotFoundException("Результат с номером " + resultId + " не найден");
        }
    }
}
